@@ -17,6 +17,7 @@
  */
 
 #include <string.h>
+#include <fcntl.h>
 #ifdef WIN32
 #  include <io.h>
 #else
@@ -30,26 +31,62 @@
 #ifdef GSM_ENABLE_BLUETOOTHDEVICE
 #ifdef BLUETOOTH_RF_SEARCHING
 
-GSM_Error bluetooth_checkservicename(GSM_StateMachine *s, char *name)
+int bluetooth_checkservicename(GSM_StateMachine *s, const char *name)
 {
 	/* Phonet */
 	if (s->ConnectionType == GCT_BLUEPHONET) {
-		if (strstr(name, "Nokia PC Suite") != NULL) return ERR_NONE;
+		if (strstr(name, "Nokia PC Suite") != NULL) return 1;
 
 	/* OBEX */
 	} else if (s->ConnectionType == GCT_BLUEOBEX) {
-		if (strstr(name, "OBEX") != NULL) return ERR_NONE;
+		/* Prefer this on Nokia as this gives better access to filesystem */
+		if (strstr(name, "Nokia OBEX PC Suite Services") != NULL) {
+			if (strcmp(s->CurrentConfig->Model, "obex") == 0) {
+				return 5;
+			}
+			return 3;
+		}
+		/* For filesystem, we prefer file transfer */
+		if (strstr(name, "OBEX File Transfer") != NULL || strstr(name, "OBEX file transfer") != NULL) {
+			if (strcmp(s->CurrentConfig->Model, "obex") == 0) {
+				return 4;
+			}
+			return 3;
+		}
+		/* Ususally this name also contains OBEX, preffered for irmc */
+		if (strstr(name, "IrMC Sync") != NULL || strstr(name, "OBEX Synchronisation")) {
+			if (strcmp(s->CurrentConfig->Model, "obexirmc") == 0 ||
+				strcmp(s->CurrentConfig->Model, "seobex") == 0) {
+				return 4;
+			}
+			return 3;
+		}
+		/* Simple send of files should work here */
+		if (strstr(name, "OBEX Object Push") != NULL) {
+			if (strcmp(s->CurrentConfig->Model, "obexnone") == 0) {
+				return 3;
+			}
+			return 2;
+		}
+		/* Anything matching OBEX has lowest priority */
+		if (strstr(name, "OBEX") != NULL) return 1;
 
 	/* AT */
 	} else if (s->ConnectionType == GCT_BLUEAT) {
-		if (strstr(name, "COM") != NULL) return ERR_NONE;
 		/* Sony-Ericsson */
-		if (strstr(name, "Serial Port") != NULL) return ERR_NONE;
+		if (strstr(name, "Serial Port 1") != NULL) return 3;
+		if (strstr(name, "Serial Port") != NULL) return 2;
+		/* Motorola */
+		if (strstr(name, "Serial port") != NULL) return 2;
 		/* Siemens, Thomas Eitzenberger */
-		if (strstr(name, "SerialPort") != NULL) return ERR_NONE;
+		if (strstr(name, "SerialPort1") != NULL) return 3;
+		if (strstr(name, "SerialPort") != NULL) return 2;
+
+		if (strstr(name, "COM1") != NULL) return 3;
+		if (strstr(name, "COM") != NULL) return 1;
 	}
 
-        return ERR_UNKNOWN;
+        return 0;
 }
 
 #endif
@@ -57,32 +94,41 @@ GSM_Error bluetooth_checkservicename(GSM_StateMachine *s, char *name)
 
 #if defined (GSM_ENABLE_BLUETOOTHDEVICE) || defined (GSM_ENABLE_IRDADEVICE)
 
-int socket_read(GSM_StateMachine *s, void *buf, size_t nbytes, int hPhone)
+int socket_read(GSM_StateMachine *s UNUSED, void *buf, size_t nbytes, int hPhone)
 {
 	fd_set 		readfds;
+	int result = 0;
 #ifdef WIN32
 	struct timeval 	timer;
+#else
+	int flags;
 #endif
 
 	FD_ZERO(&readfds);
 	FD_SET(hPhone, &readfds);
 #ifndef WIN32
-	if (select(hPhone+1, &readfds, NULL, NULL, 0)) {
-		return(read(hPhone, buf, nbytes));
+	if (select(hPhone+1, &readfds, NULL, NULL, 0) > 0) {
+		flags = fcntl(hPhone, F_GETFL);
+		fcntl(hPhone, F_SETFL, flags | O_NONBLOCK);
+		result = read(hPhone, buf, nbytes);
+		if (result < 0 && errno != EINTR) {
+			return 0;
+		}
+		fcntl(hPhone, F_SETFL, flags);
 	}
 #else
 	memset(&timer,0,sizeof(timer));
-	if (select(0, &readfds, NULL, NULL, &timer) != 0) {
-		return(recv(hPhone, buf, nbytes, 0));
+	if (select(0, &readfds, NULL, NULL, &timer) > 0) {
+		result = recv(hPhone, buf, nbytes, 0);
 	}
 #endif
-	return 0;
+	return result;
 }
 
 #ifdef WIN32
-int socket_write(GSM_StateMachine *s, unsigned char *buf, size_t nbytes, int hPhone)
+int socket_write(GSM_StateMachine *s, unsigned const char *buf, size_t nbytes, int hPhone)
 #else
-int socket_write(GSM_StateMachine *s, void *buf, size_t nbytes, int hPhone)
+int socket_write(GSM_StateMachine *s, const void *buf, size_t nbytes, int hPhone)
 #endif
 {
 	int		ret;
@@ -91,7 +137,14 @@ int socket_write(GSM_StateMachine *s, void *buf, size_t nbytes, int hPhone)
 	do {
 		ret = send(hPhone, buf, nbytes - actual, 0);
         	if (ret < 0) {
-            		if (actual != nbytes) GSM_OSErrorInfo(s,"socket_write");
+            		if (actual != nbytes) {
+				GSM_OSErrorInfo(s,"socket_write");
+#ifndef WIN32
+				if (errno != EINTR) {
+					return 0;
+				}
+#endif
+			}
             		return actual;
         	}
 		actual 	+= ret;
@@ -101,7 +154,7 @@ int socket_write(GSM_StateMachine *s, void *buf, size_t nbytes, int hPhone)
 	return actual;
 }
 
-GSM_Error socket_close(GSM_StateMachine *s, int hPhone)
+GSM_Error socket_close(GSM_StateMachine *s UNUSED, int hPhone)
 {
 	shutdown(hPhone, 0);
 #ifdef WIN32
@@ -114,21 +167,6 @@ GSM_Error socket_close(GSM_StateMachine *s, int hPhone)
 
 #endif
 
-#ifdef ENABLE_LGPL
-
-GSM_Error lock_device(const char* port, char **lock_device)
-{
-	*lock_device = 0;
-	return ERR_NONE;
-}
-
-bool unlock_device(char **lock_file)
-{
-	return true;
-}
-
-#else
-
 #define max_buf_len 	128
 #define lock_path 	"/var/lock/LCK.."
 
@@ -266,8 +304,6 @@ bool unlock_device(char **lock_file)
 #endif
 }
 
-#endif
-
 int FindSerialSpeed(char *buffer)
 {
 	switch (atoi(buffer)) {
