@@ -7,7 +7,7 @@
 #ifdef GSM_ENABLE_BLUETOOTHDEVICE
 #ifdef BLUETOOTH_RF_SEARCHING
 
-GSM_Error bluetooth_checkservicename(GSM_StateMachine *s, char *name);
+int bluetooth_checkservicename(GSM_StateMachine *s, const char *name);
 
 #endif
 #endif
@@ -17,9 +17,9 @@ GSM_Error bluetooth_checkservicename(GSM_StateMachine *s, char *name);
 int socket_read(GSM_StateMachine *s, void *buf, size_t nbytes, int hPhone);
 
 #ifdef WIN32
-int socket_write(GSM_StateMachine *s, unsigned char *buf, size_t nbytes, int hPhone);
+int socket_write(GSM_StateMachine *s, unsigned const char *buf, size_t nbytes, int hPhone);
 #else
-int socket_write(GSM_StateMachine *s, void *buf, size_t nbytes, int hPhone);
+int socket_write(GSM_StateMachine *s, const void *buf, size_t nbytes, int hPhone);
 #endif
 
 GSM_Error socket_close(GSM_StateMachine *s, int hPhone);
