@@ -1,16 +1,33 @@
 /* (c) 2002-2004 by Marcin Wiacek & Michal Cihar */
 
+/**
+ * @file gsmstate.h
+ * @author Michal Čihař
+ * @author Marcin Wiacek
+ */
+
+/**
+ * @defgroup StateMachine State machine data
+ *
+ * This module implements core of state machine.
+ *
+ * @{
+ */
+
 #ifndef __gsm_state_h
 #define __gsm_state_h
 
 #include <time.h>
 
-#include "config.h"
-#include "misc/cfg.h"
+#include <gammu-file.h>
+#include <gammu-info.h>
+#include <gammu-config.h>
+#include <gammu-keys.h>
+#include <gammu-security.h>
+#include <gammu-callback.h>
+#include <gammu-settings.h>
 
-typedef struct _GSM_StateMachine 	GSM_StateMachine;
 typedef struct _GSM_User	 	GSM_User;
-typedef struct _OnePhoneModel	 	OnePhoneModel;
 typedef struct _GSM_Reply_Function	GSM_Reply_Function;
 
 #ifdef GSM_ENABLE_NOKIA3650
@@ -40,8 +57,8 @@ typedef struct _GSM_Reply_Function	GSM_Reply_Function;
 #ifdef GSM_ENABLE_ALCATEL
 #  include "phone/alcatel/alcatel.h"
 #endif
-#ifdef GSM_ENABLE_SONYERICSSON
-#  include "phone/sonyeric/sonyeric.h"
+#ifdef GSM_ENABLE_ATOBEX
+#  include "phone/atobex/atobex.h"
 #endif
 #ifdef GSM_ENABLE_OBEXGEN
 #  include "phone/obex/obexgen.h"
@@ -111,13 +128,6 @@ typedef struct _GSM_Reply_Function	GSM_Reply_Function;
 #  undef GSM_ENABLE_BLUEGNAPBUS
 #endif
 
-#if defined(GSM_ENABLE_NOKIA3320) || defined(GSM_ENABLE_NOKIA650) || defined(GSM_ENABLE_NOKIA6110) || defined(GSM_ENABLE_NOKIA7110) || defined(GSM_ENABLE_NOKIA9210)
-#  define GSM_ENABLE_NOKIA_DCT3
-#endif
-#if defined(GSM_ENABLE_NOKIA3650) || defined(GSM_ENABLE_NOKIA6510)
-#  define GSM_ENABLE_NOKIA_DCT4
-#endif
-
 #include "protocol/protocol.h"
 #if defined(GSM_ENABLE_FBUS2) || defined(GSM_ENABLE_FBUS2IRDA) || defined(GSM_ENABLE_FBUS2DLR3) || defined(GSM_ENABLE_FBUS2BLUE) || defined(GSM_ENABLE_BLUEFBUS2) || defined(GSM_ENABLE_DKU5FBUS2) || defined(GSM_ENABLE_FBUS2PL2303)
 #  include "protocol/nokia/fbus2.h"
@@ -134,7 +144,7 @@ typedef struct _GSM_Reply_Function	GSM_Reply_Function;
 #ifdef GSM_ENABLE_ALCABUS
 #  include "protocol/alcatel/alcabus.h"
 #endif
-#if defined(GSM_ENABLE_IRDAOBEX) || defined(GSM_ENABLE_BLUEOBEX)
+#if defined(GSM_ENABLE_IRDAOBEX) || defined(GSM_ENABLE_BLUEOBEX) || defined(GSM_ENABLE_ATOBEX)
 #  include "protocol/obex/obex.h"
 #endif
 #if defined(GSM_ENABLE_BLUEGNAPBUS) || defined(GSM_ENABLE_IRDAGNAPBUS)
@@ -172,26 +182,6 @@ typedef struct _GSM_Reply_Function	GSM_Reply_Function;
 #  undef GSM_ENABLE_FBUS2BLUE
 #endif
 
-#ifndef WIN32
-#  ifdef ENABLE_LGPL
-#    undef GSM_ENABLE_IRDADEVICE
-#    undef GSM_ENABLE_IRDAPHONET
-#    undef GSM_ENABLE_IRDAGNAPBUS
-#    undef GSM_ENABLE_IRDAOBEX
-#    undef GSM_ENABLE_IRDAAT
-#    undef GSM_ENABLE_DKU2AT
-#    undef GSM_ENABLE_FBUS2IRDA
-#    undef GSM_ENABLE_BLUETOOTHDEVICE
-#    undef GSM_ENABLE_BLUEPHONET
-#    undef GSM_ENABLE_BLUEOBEX
-#    undef GSM_ENABLE_BLUEAT
-#    undef GSM_ENABLE_BLUEFBUS2
-#    undef GSM_ENABLE_BLUEGNAPBUS
-#    undef GSM_ENABLE_PHONETBLUE
-#    undef GSM_ENABLE_FBUS2BLUE
-#  endif
-#endif
-
 #ifdef GSM_ENABLE_SERIALDEVICE
 #  include "device/serial/ser_w32.h"
 #  include "device/serial/ser_unx.h"
@@ -211,9 +201,7 @@ typedef struct _GSM_Reply_Function	GSM_Reply_Function;
 #include "service/gsmdata.h"
 #include "service/gsmlogo.h"
 #include "service/gsmmisc.h"
-#include "service/gsmprof.h"
 #include "service/gsmcall.h"
-#include "service/sms/gsmsms.h"
 #include "service/sms/gsmems.h"
 #include "service/sms/gsmmulti.h"
 #include "service/backup/gsmback.h"
@@ -252,17 +240,26 @@ typedef struct {
 	/**
 	 * Attempts to read nbytes from device.
 	 */
-	int       (*WriteDevice)       (GSM_StateMachine *s, void *buf, size_t nbytes);
+	int       (*WriteDevice)       (GSM_StateMachine *s, const void *buf, size_t nbytes);
 } GSM_Device_Functions;
 
 #ifdef GSM_ENABLE_SERIALDEVICE
-	extern GSM_Device_Functions SerialDevice;
+/**
+ * Serial device functions.
+ */
+extern GSM_Device_Functions SerialDevice;
 #endif
 #ifdef GSM_ENABLE_IRDADEVICE
-	extern GSM_Device_Functions IrdaDevice;
+/**
+ * IrDA device functions.
+ */
+extern GSM_Device_Functions IrdaDevice;
 #endif
 #ifdef GSM_ENABLE_BLUETOOTHDEVICE
-	extern GSM_Device_Functions BlueToothDevice;
+/**
+ * Bluetooth device functions.
+ */
+extern GSM_Device_Functions BlueToothDevice;
 #endif
 
 /**
@@ -272,17 +269,32 @@ typedef struct {
  */
 typedef struct {
 	union {
+		/**
+		 * Fake memeber to ensure union has always at least one member.
+		 */
 		char fake;
 #ifdef GSM_ENABLE_SERIALDEVICE
+		/**
+		 * Data for serial port device.
+		 */
 		GSM_Device_SerialData		Serial;
 #endif
 #ifdef GSM_ENABLE_IRDADEVICE
+		/**
+		 * Data for IrDA port device.
+		 */
 		GSM_Device_IrdaData		Irda;
 #endif
 #ifdef GSM_ENABLE_BLUETOOTHDEVICE
+		/**
+		 * Data for Bluetooth port device.
+		 */
 		GSM_Device_BlueToothData	BlueTooth;
 #endif
 	} Data;
+	/**
+	 * Functions for currently used device.
+	 */
 	GSM_Device_Functions *Functions;
 } GSM_Device;
 
@@ -295,7 +307,7 @@ typedef struct {
 	/**
 	 * Writes message to device.
 	 */
-	GSM_Error (*WriteMessage) (GSM_StateMachine *s, unsigned char *buffer,
+	GSM_Error (*WriteMessage) (GSM_StateMachine *s, unsigned const char *buffer,
 				   int length, unsigned char type);
 	/**
 	 * This one is called when character is received from device.
@@ -326,7 +338,7 @@ typedef struct {
 #ifdef GSM_ENABLE_ALCABUS
 	extern GSM_Protocol_Functions ALCABUSProtocol;
 #endif
-#if defined(GSM_ENABLE_IRDAOBEX) || defined(GSM_ENABLE_BLUEOBEX)
+#if defined(GSM_ENABLE_IRDAOBEX) || defined(GSM_ENABLE_BLUEOBEX) || defined(GSM_ENABLE_ATOBEX)
 	extern GSM_Protocol_Functions OBEXProtocol;
 #endif
 #if defined(GSM_ENABLE_BLUEGNAPBUS) || defined(GSM_ENABLE_IRDAGNAPBUS)
@@ -356,13 +368,16 @@ typedef struct {
 #ifdef GSM_ENABLE_ALCABUS
 		GSM_Protocol_ALCABUSData	ALCABUS;
 #endif
-#if defined(GSM_ENABLE_IRDAOBEX) || defined(GSM_ENABLE_BLUEOBEX)
+#if defined(GSM_ENABLE_IRDAOBEX) || defined(GSM_ENABLE_BLUEOBEX) || defined(GSM_ENABLE_ATOBEX)
 		GSM_Protocol_OBEXData		OBEX;
 #endif
 #if defined(GSM_ENABLE_BLUEGNAPBUS) || defined(GSM_ENABLE_IRDAGNAPBUS)
 		GSM_Protocol_GNAPBUSData	GNAPBUS;
 #endif
 	} Data;
+	/**
+	 * Functions for currently used protocol layer.
+	 */
 	GSM_Protocol_Functions *Functions;
 } GSM_Protocol;
 
@@ -390,6 +405,7 @@ typedef enum {
 	ID_EnableErrorInfo,
 	ID_SetOBEX,
 	ID_SetUSSD,
+	ID_GetUSSD,
 	ID_GetNote,
 	ID_SetNote,
 	ID_GetSignalQuality,
@@ -399,6 +415,8 @@ typedef enum {
 	ID_GetSMSStatus,
 	ID_AddSMSFolder,
 	ID_GetNetworkInfo,
+	ID_GetNetworkCode,
+	ID_GetNetworkName,
 	ID_GetRingtone,
 	ID_DialVoice,
 	ID_GetCalendarNotesInfo,
@@ -427,6 +445,7 @@ typedef enum {
 	ID_SetCalendarNote,
 	ID_SetIncomingSMS,
 	ID_SetIncomingCB,
+	ID_SetIncomingCall,
 	ID_GetCNMIMode,
 	ID_GetCalendarNotePos,
 	ID_Initialise,
@@ -493,6 +512,7 @@ typedef enum {
 	ID_AddFolder,
 	ID_DeleteFolder,
 	ID_DeleteFile,
+	ID_ModeSwitch,
 #ifdef GSM_ENABLE_ALCATEL
     	/* AT mode */
     	ID_SetFlowControl,
@@ -553,15 +573,15 @@ typedef struct {
 	/**
 	 * Phone IMEI (or serial number).
 	 */
-	char			IMEI[MAX_IMEI_LENGTH];
+	char			IMEI[GSM_MAX_IMEI_LENGTH];
 	/**
 	 * Phone manufacturer as reported by phone.
 	 */
-	char			Manufacturer[MAX_MANUFACTURER_LENGTH];
+	char			Manufacturer[GSM_MAX_MANUFACTURER_LENGTH];
 	/**
 	 * Phone model as reported by phone.
 	 */
-	char			Model[MAX_MODEL_LENGTH];
+	char			Model[GSM_MAX_MODEL_LENGTH];
 	/**
 	 * Model information, pointer to static @ref allmodels array.
 	 */
@@ -570,11 +590,11 @@ typedef struct {
 	 * Phone version as reported by phone. It doesn't have to be numerical
 	 * at all.
 	 */
-	char			Version[MAX_VERSION_LENGTH];
+	char			Version[GSM_MAX_VERSION_LENGTH];
 	/**
 	 * Phone version date, might be empty for some models.
 	 */
-	char			VerDate[MAX_VERSION_DATE_LENGTH];
+	char			VerDate[GSM_MAX_VERSION_DATE_LENGTH];
 	/**
 	 * Phone version as number, if applicable.
 	 */
@@ -687,13 +707,16 @@ typedef struct {
 	 */
 	unsigned char		*Netmonitor;
 	/**
-	 * Pointer to structure used internally by phone drivers.
+	 * Pointer to call diversion structure used internally by phone drivers.
 	 */
 	GSM_MultiCallDivert	*Divert;
 	/**
-	 * Pointer to structure used internally by phone drivers.
+	 * Pointer to todo structure used internally by phone drivers.
 	 */
 	GSM_ToDoEntry		*ToDo;
+	/**
+	 * Pointer to note structure used internally by phone drivers.
+	 */
 	GSM_NoteEntry		*Note;
 	/**
 	 * Used internally by phone drivers.
@@ -748,7 +771,13 @@ typedef struct {
 	 * Pointer to structure used internally by phone drivers.
 	 */
 	GSM_FileSystemStatus	*FileSystemStatus;
+	/**
+	 * Pointer to structure used internally by phone drivers.
+	 */
 	GSM_ChatSettings	*ChatSettings;
+	/**
+	 * Pointer to structure used internally by phone drivers.
+	 */
 	GSM_SyncMLSettings	*SyncMLSettings;
 
 	/**
@@ -790,7 +819,6 @@ typedef struct {
 	 * Structure with private phone modules data.
 	 */
 	struct {
-		int			 fake;
 #ifdef GSM_ENABLE_NOKIA3320
 		GSM_Phone_N3320Data	 N3320;
 #endif
@@ -815,8 +843,8 @@ typedef struct {
 #ifdef GSM_ENABLE_ALCATEL
 		GSM_Phone_ALCATELData	 ALCATEL;
 #endif
-#ifdef GSM_ENABLE_SONYERICSSON
-		GSM_Phone_SONYERICSSONData	 SONYERICSSON;
+#ifdef GSM_ENABLE_ATOBEX
+		GSM_Phone_ATOBEXData	 ATOBEX;
 #endif
 #ifdef GSM_ENABLE_OBEXGEN
 		GSM_Phone_OBEXGENData	 OBEXGEN;
@@ -1092,6 +1120,9 @@ typedef struct {
 	 * Sends SMS already saved in phone.
 	 */
 	GSM_Error (*SendSavedSMS)	(GSM_StateMachine *s, int Folder, int Location);
+	/**
+	 * Configures fast SMS sending.
+	 */
 	GSM_Error (*SetFastSMSSending)  (GSM_StateMachine *s, bool enable);
 	/**
 	 * Enable/disable notification on incoming SMS.
@@ -1117,6 +1148,10 @@ typedef struct {
 	 * Dials number and starts voice call.
 	 */
 	GSM_Error (*DialVoice)	  	(GSM_StateMachine *s, char *Number, GSM_CallShowNumber ShowNumber);
+	/**
+	 * Dials service number (usually for USSD).
+	 */
+	GSM_Error (*DialService)	(GSM_StateMachine *s, char *Number);
 	/**
 	 * Accept current incoming call.
 	 */
@@ -1237,7 +1272,13 @@ typedef struct {
 	 * Changes MMS settings.
 	 */
 	GSM_Error (*SetMMSSettings)     (GSM_StateMachine *s, GSM_MultiWAPSettings *settings);
+	/**
+	 * Lists MMS folders.
+	 */
 	GSM_Error (*GetMMSFolders)      (GSM_StateMachine *s, GSM_MMSFolders *folders);
+	/**
+	 * Retrieves next part of MMS file information.
+	 */
 	GSM_Error (*GetNextMMSFileInfo)	(GSM_StateMachine *s, unsigned char *FileID, int *MMSFolder, bool start);
 	/**
 	 * Gets bitmap.
@@ -1369,13 +1410,26 @@ typedef struct {
 	 * Gets file part from filesystem.
 	 */
 	GSM_Error (*GetFolderListing)   (GSM_StateMachine *s, GSM_File *File, bool start);
+	/**
+	 * Gets next root folder.
+	 */
 	GSM_Error (*GetNextRootFolder)  (GSM_StateMachine *s, GSM_File *File);
+	/**
+	 * Sets file system attributes.
+	 */
 	GSM_Error (*SetFileAttributes)  (GSM_StateMachine *s, GSM_File *File);
+	/**
+	 * Retrieves file part.
+	 */
 	GSM_Error (*GetFilePart)	(GSM_StateMachine *s, GSM_File *File, int *Handle, int *Size);
 	/**
 	 * Adds file part to filesystem.
 	 */
 	GSM_Error (*AddFilePart)	(GSM_StateMachine *s, GSM_File *File, int *Pos, int *Handle);
+	/**
+	 * Sends file to phone, it's up to phone to decide what to do with it.
+	 */
+	GSM_Error (*SendFilePart)	(GSM_StateMachine *s, GSM_File *File, int *Pos, int *Handle);
 	/**
 	 * Acquires filesystem status.
 	 */
@@ -1388,6 +1442,9 @@ typedef struct {
 	 * Adds folder to filesystem.
 	 */
 	GSM_Error (*AddFolder)	  	(GSM_StateMachine *s, GSM_File *File);
+	/**
+	 * Deletes folder from filesystem.
+	 */
 	GSM_Error (*DeleteFolder)	(GSM_StateMachine *s, unsigned char *ID);
 	/**
 	 * Gets GPRS access point.
@@ -1427,8 +1484,8 @@ typedef struct {
 #ifdef GSM_ENABLE_ALCATEL
 	extern GSM_Phone_Functions ALCATELPhone;
 #endif
-#ifdef GSM_ENABLE_SONYERICSSON
-	extern GSM_Phone_Functions SONYERICSSONPhone;
+#ifdef GSM_ENABLE_ATOBEX
+	extern GSM_Phone_Functions ATOBEXPhone;
 #endif
 #ifdef GSM_ENABLE_OBEXGEN
 	extern GSM_Phone_Functions OBEXGENPhone;
@@ -1437,8 +1494,17 @@ typedef struct {
 	extern GSM_Phone_Functions GNAPGENPhone;
 #endif
 
+/**
+ * Phone functions and private data.
+ */
 typedef struct {
+	/**
+	 * Private data for current phone driver.
+	 */
 	GSM_Phone_Data		 Data;
+	/**
+	 * Functions for current phone driver.
+	 */
 	GSM_Phone_Functions	*Functions;
 } GSM_Phone;
 
@@ -1447,84 +1513,36 @@ typedef struct {
 struct _GSM_User {
 	GSM_Reply_Function		*UserReplyFunctions;
 
-	void	  (*IncomingCall)	(char *Device, GSM_Call	      call);
-	void 	  (*IncomingSMS)	(char *Device, GSM_SMSMessage sms);
-	void 	  (*IncomingCB)		(char *Device, GSM_CBMessage  cb);
-	void 	  (*IncomingUSSD)	(char *Device, char	      *Text);
-	void 	  (*SendSMSStatus)	(char *Device, int 	      status, int MessageReference);
+	IncomingCallCallback IncomingCall;
+	IncomingSMSCallback IncomingSMS;
+	IncomingCBCallback IncomingCB;
+	IncomingUSSDCallback IncomingUSSD;
+	SendSMSStatusCallback SendSMSStatus;
 };
 
 /* --------------------------- Statemachine layer -------------------------- */
 
-typedef enum {
-	GCT_MBUS2=1,
-	GCT_FBUS2,
-	GCT_FBUS2DLR3,
-	GCT_DKU2AT,
-	GCT_DKU2PHONET,
-	GCT_DKU5FBUS2,
-	GCT_FBUS2PL2303,
-	GCT_FBUS2BLUE,
-	GCT_FBUS2IRDA,
-	GCT_PHONETBLUE,
-	GCT_AT,
-	GCT_BLUEGNAPBUS,
-	GCT_IRDAOBEX,
-	GCT_IRDAGNAPBUS,
-	GCT_IRDAAT,
-	GCT_IRDAPHONET,
-	GCT_BLUEFBUS2,
-	GCT_BLUEAT,
-	GCT_BLUEPHONET,
-	GCT_BLUEOBEX
-} GSM_ConnectionType;
-
-typedef struct {
-	/* Config file (or Registry or...) variables */
-	char			Model[50];	   /* Model from config file 		*/
-	char			DebugLevel[50];    /* Debug level			*/
-	char 			*Device;	   /* Device name from config file 	*/
-	char			*Connection;	   /* Connection type as string		*/
-	char			*SyncTime;	   /* Synchronize time on startup? 	*/
-	char			*LockDevice;	   /* Lock device ? (Unix)		*/
-	char			*DebugFile;        /* Name of debug file		*/
-	char 			*Localize;	   /* Name of localisation file		*/
-	char			*StartInfo;	   /* Display something during start ?  */
-	bool			UseGlobalDebugFile;/* Should we use global debug file?	*/
-	bool			DefaultModel;
-	bool			DefaultDebugLevel;
-	bool			DefaultDevice;
-	bool			DefaultConnection;
-	bool			DefaultSyncTime;
-	bool			DefaultLockDevice;
-	bool			DefaultDebugFile;
-	bool			DefaultLocalize;
-	bool			DefaultStartInfo;
-	char			TextReminder[32];	/* Text for calendar entry categories in local language */
-	char			TextMeeting[32];
-	char			TextCall[32];
-	char			TextBirthday[32];
-	char			TextMemo[32];
-} GSM_Config;
 
+/**
+ * Maximum number of concurrent configurations.
+ */
 #define MAX_CONFIG_NUM		5
 
 struct _GSM_StateMachine {
-	GSM_ConnectionType 	ConnectionType;				/* Type of connection as int			*/
-	char			*LockFile;				/* Lock file name for Unix 			*/
-	Debug_Info		di;					/* Debug information				*/
-	bool			opened;					/* Is connection opened ?			*/
-	GSM_Config		Config[MAX_CONFIG_NUM + 1];
-	GSM_Config		*CurrentConfig;				/* Config file (or Registry or...) variables 	*/
-	int			ConfigNum;
-	INI_Section 		*msg;					/* Localisation strings structure    		*/
-	int			ReplyNum;				/* How many times make sth. 			*/
-	int			Speed;					/* For some protocols used speed		*/
+	GSM_ConnectionType 	ConnectionType;				/**< Type of connection as int			*/
+	char			*LockFile;				/**< Lock file name for Unix 			*/
+	GSM_Debug_Info		di;					/**< Debug information				*/
+	bool			opened;					/**< Is connection opened ?			*/
+	GSM_Config		Config[MAX_CONFIG_NUM + 1];		/**< Configuration data */
+	GSM_Config		*CurrentConfig;				/**< Config file (or Registry or...) variables 	*/
+	int			ConfigNum;				/**< Number of actual configurations */
+	int			ReplyNum;				/**< How many times make sth. 			*/
+	int			Speed;					/**< For some protocols used speed		*/
 
-	GSM_Device		Device;
-	GSM_Protocol		Protocol;
-	GSM_Phone		Phone;
-	GSM_User		User;
+	GSM_Device		Device; /**< Device driver data and functions */
+	GSM_Protocol		Protocol; /**< Protocol driver data and functions */
+	GSM_Phone		Phone; /**< Phone driver data and functions */
+	GSM_User		User; /**< User defined functions */
 };
 
 /* ------------------------ Other general definitions ---------------------- */
@@ -1536,109 +1554,18 @@ GSM_Error GSM_TerminateConnection	(GSM_StateMachine *s);
 
 int 	  GSM_ReadDevice		(GSM_StateMachine *s, bool wait);
 
-GSM_Error GSM_WaitForOnce		(GSM_StateMachine *s, unsigned char *buffer,
+GSM_Error GSM_WaitForOnce		(GSM_StateMachine *s, unsigned const char *buffer,
 			  		 int length, unsigned char type, int time);
 
-GSM_Error GSM_WaitFor			(GSM_StateMachine *s, unsigned char *buffer,
+GSM_Error GSM_WaitFor			(GSM_StateMachine *s, unsigned const char *buffer,
 		       			 int length, unsigned char type, int time,
 					 GSM_Phone_RequestID request);
 
 GSM_Error GSM_DispatchMessage		(GSM_StateMachine *s);
 
-INI_Section 				*GSM_FindGammuRC(void);
+void 	  GSM_DumpMessageLevel2		(GSM_StateMachine *s, unsigned const char *message, int messagesize, int type);
+void 	  GSM_DumpMessageLevel3		(GSM_StateMachine *s, unsigned const char *message, int messagesize, int type);
 
-bool 	  GSM_ReadConfig		(INI_Section *cfg_info, GSM_Config *cfg, int num);
-
-void 	  GSM_DumpMessageLevel2		(GSM_StateMachine *s, unsigned char *message, int messagesize, int type);
-void 	  GSM_DumpMessageLevel3		(GSM_StateMachine *s, unsigned char *message, int messagesize, int type);
-
-/* ---------------------- Phone features ----------------------------------- */
-
-typedef enum {
-	/* n6110.c */
-	F_CAL33 = 1,	/* Calendar,3310 style - 10 reminders, Unicode, 3 coding types	*/
-	F_CAL52,	/* Calendar,5210 style - full Unicode, etc.			*/
-	F_CAL82,	/* Calendar,8250 style - "normal", but with Unicode		*/
-	F_RING_SM,	/* Ringtones returned in SM format - 33xx			*/
-	F_NORING,	/* No ringtones							*/
-	F_NOPBKUNICODE,	/* No phonebook in Unicode					*/
-	F_NOWAP,      	/* No WAP							*/
-	F_NOCALLER,	/* No caller groups						*/
-	F_NOPICTURE,	/* No Picture Images						*/
-	F_NOPICTUREUNI,	/* No Picture Images text in Unicode				*/
-	F_NOSTARTUP,	/* No startup logo						*/
-	F_NOCALENDAR,	/* No calendar							*/
-	F_NOSTARTANI,	/* Startup logo is not animated 				*/
-	F_POWER_BATT,	/* Network and battery level get from netmonitor		*/
-	F_PROFILES33,	/* Phone profiles in 3310 style					*/
-	F_PROFILES51,	/* Phone profiles in 5110 style					*/
-	F_MAGICBYTES,	/* Phone can make authentication with magic bytes		*/
-	F_NODTMF,	/* Phone can't send DTMF					*/
-	F_DISPSTATUS,	/* Phone return display status					*/
-	F_NOCALLINFO,
-
-	/* n3320.c */
-	F_DAYMONTH,	/* Day and month reversed in pbk, when compare to GSM models	*/
-
-	/* n6510.c */
-	F_PBK35,	/* Phonebook in 3510 style with ringtones ID			*/
-	F_PBKIMG,	/* Phonebook in 7250 style with picture ID			*/
-	F_PBKTONEGAL,	/* Phonebook with selecting ringtones from gallery              */
-	F_PBKSMSLIST,	/* Phonebook with SMS list					*/
-	F_PBKUSER,	/* Phonebook with user ID					*/
-	F_6230iCALLER,	/* Caller groups like in 6230i					*/
-	F_RADIO,	/* Phone with FM radio						*/
-	F_TODO63,	/* ToDo in 6310 style - 0x55 msg type				*/
-	F_TODO66,	/* ToDo in 6610 style - like calendar, with date and other	*/
-	F_NOMIDI,	/* No ringtones in MIDI						*/
-	F_BLUETOOTH,	/* Bluetooth support						*/
-	F_NOFILESYSTEM,	/* No images, ringtones, java saved in special filesystem	*/
-	F_NOMMS,	/* No MMS sets in phone						*/
-	F_NOGPRSPOINT,	/* GPRS point are not useable					*/
-	F_CAL35,	/* Calendar,3510 style - Reminder,Call,Birthday			*/
-	F_CAL65,	/* Calendar,6510 style - CBMM, method 3				*/
-	F_WAPMMSPROXY,	/* WAP & MMS settings contains first & second proxy		*/
-	F_CHAT,		/* Phone with Chat settings 					*/
-	F_SYNCML,	/* Phone with SyncML settings 					*/
-	F_FILES2,	/* filesystem version 2 					*/
-	F_NOFILE1,
-	F_6230iWAP,	/* WAP, MMS, etc. settings like in 6230i - unknown now		*/
-	F_PROFILES,	/* Profiles support available					*/
-	F_SERIES40_30,  /* Series 40 3.0						*/
-	F_SMS_FILES,  	/* SMS are read from filesystem files like in Series 40 3.0	*/
-	F_3220_MMS,
-
-	/* n6510.c && n7110.c */
-	F_VOICETAGS,	/* Voice tags available						*/
-	F_CAL62,	/* Calendar,6210 style - Call,Birthday,Memo,Meeting		*/
-	F_NOTES,
-
-	/* AT modules */
-	F_SMSONLYSENT,	/* Phone supports only sent/unsent messages			*/
-	F_BROKENCPBS, 	/* CPBS on some memories can hang phone				*/
-	F_M20SMS,	/* Siemens M20 like SMS handling				*/
-	F_SLOWWRITE,	/* Use slower writing which some phone need			*/
-	F_SMSME900,	/* SMS in ME start from location 900 - case of Sagem		*/
-	F_ALCATEL,	/* Phone supports Alcatel protocol				*/
-	F_OBEX,		/* Phone can switch to OBEX protocol from AT mode		*/
-	F_IRMC_LEVEL_2,	/* Phone supports IrMC level 2 even if it doesn't report it	*/
-} Feature;
-
-/* For models table */
-struct _OnePhoneModel {
-	char		*model;
-	char		*number;
-	char		*irdamodel;
-	Feature		features[14];
-};
-
-bool 		IsPhoneFeatureAvailable	(OnePhoneModel *model, Feature feature);
-OnePhoneModel 	*GetModelData		(char *model, char *number, char *irdamodel);
-
-#ifdef __GNUC__
-__attribute__((format(printf, 2, 3)))
-#endif
-int smprintf(GSM_StateMachine *s, const char *format, ...);
 
 void GSM_OSErrorInfo(GSM_StateMachine *s, char *description);
 
@@ -1647,6 +1574,7 @@ void GSM_GetPhoneFeaturesForBackup(GSM_StateMachine *s, GSM_Backup_Info *info);
 #endif
 
 #endif
+/*@}*/
 
 /* How should editor hadle tabs in this file? Add editor commands here.
  * vim: noexpandtab sw=8 ts=8 sts=8:
