@@ -19,11 +19,10 @@
 #include <string.h>
 #include <ctype.h>
 #include <locale.h>
-#ifndef __OpenBSD__
-#  include <wctype.h>
-#endif
+#include <limits.h>
 #ifdef WIN32
-#  include "windows.h"
+#  define WIN32_LEAN_AND_MEAN
+#  include <windows.h>
 #endif
 
 #include "../misc.h"
@@ -179,18 +178,40 @@ char *DecodeSpecialChars(unsigned char *buffer)
 	return Buf;
 }
 
-char *mystrcasestr(unsigned const char *a, unsigned const char *b)
+#ifndef HAVE_STRCASESTR
+/**
+ * Find the first occurrence of find in s, ignore case.
+ * Copyright (c) 1990, 1993 The Regents of the University of California.
+ */
+char *strcasestr(const char *s, const char *find)
 {
-	unsigned char 	A[2000], B[200];
-	int 		i;
+	char c, sc;
+	size_t len;
 
-	memset(A,0,sizeof(A));
-	memset(B,0,sizeof(B));
-	for (i=0;i<(int)strlen(a);i++) A[i] = tolower(a[i]);
-	for (i=0;i<(int)strlen(b);i++) B[i] = tolower(b[i]);
-
-	return strstr(A,B);
+	if ((c = *find++) != 0) {
+		c = tolower((unsigned char)c);
+		len = strlen(find);
+		do {
+			do {
+				if ((sc = *s++) == 0)
+					return (NULL);
+			} while ((char)tolower((unsigned char)sc) != c);
+		} while (strncasecmp(s, find, len) != 0);
+		s--;
+	}
+	return ((char *)s);
 }
+#endif
+
+#ifndef HAVE_STRCHRNUL
+char *strchrnul(char *s, int find)
+{
+	char *ret;
+	ret = strchr(s, find);
+	if (ret == NULL) return s + strlen(s);
+	return ret;
+}
+#endif
 
 size_t UnicodeLength(const unsigned char *str)
 {
@@ -229,7 +250,7 @@ int DecodeWithUnicodeAlphabet(wchar_t src, unsigned char *dest)
         }
 }
 
-void DecodeUnicode (const unsigned char *src, unsigned char *dest)
+void DecodeUnicode (const unsigned char *src, char *dest)
 {
  	int 		i=0,o=0;
  	wchar_t 	wc;
@@ -243,7 +264,7 @@ void DecodeUnicode (const unsigned char *src, unsigned char *dest)
 }
 
 /* Decode Unicode string and return as function result */
-unsigned char *DecodeUnicodeString (const unsigned char *src)
+char *DecodeUnicodeString (const unsigned char *src)
 {
  	static char dest[500];
 
@@ -254,7 +275,7 @@ unsigned char *DecodeUnicodeString (const unsigned char *src)
 /* Decode Unicode string to UTF8 or other console charset
  * and return as function result
  */
-unsigned char *DecodeUnicodeConsole(const unsigned char *src)
+char *DecodeUnicodeConsole(const unsigned char *src)
 {
  	static char dest[500];
 
@@ -282,9 +303,9 @@ unsigned char *DecodeUnicodeConsole(const unsigned char *src)
 /* Encode string to Unicode. Len is number of input chars */
 void DecodeISO88591 (unsigned char *dest, const char *src, int len)
 {
-	int 		i = 0;
+	int i;
 
-	while (src[i] != 0) {
+	for (i = 0; i < len; i++) {
 		/* Hack for Euro sign */
 		if ((unsigned char)src[i] == 0x80) {
 			dest[2 * i] = 0x20;
@@ -293,7 +314,6 @@ void DecodeISO88591 (unsigned char *dest, const char *src, int len)
 			dest[2 * i] = 0;
 			dest[(2 * i) + 1] = src[i];
 		}
-		i++;
 	}
 	dest[2 * i] = 0;
 	dest[(2 * i) + 1] = 0;
@@ -361,9 +381,15 @@ void EncodeBCD (unsigned char *dest, const unsigned char *src, int len, bool fil
 
 int DecodeWithHexBinAlphabet (unsigned char mychar)
 {
-	if (mychar>='A' && mychar<='F') return mychar-'A'+10;
-	if (mychar>='a' && mychar<='f') return mychar-'a'+10;
-	if (mychar>='0' && mychar<='9') return mychar-'0';
+	if (mychar >= 'A' && mychar <= 'F')
+		return mychar - 'A' + 10;
+
+	if (mychar >= 'a' && mychar <= 'f')
+		return mychar - 'a' + 10;
+
+	if (mychar >= '0' && mychar <= '9')
+		return mychar - '0';
+
 	return -1;
 }
 
@@ -443,7 +469,7 @@ static unsigned char GSM_DefaultAlphabetUnicode[128+1][2] =
 {
 	{0x00,0x40},{0x00,0xa3},{0x00,0x24},{0x00,0xA5},
 	{0x00,0xE8},{0x00,0xE9},{0x00,0xF9},{0x00,0xEC},/*0x08*/
-	{0x00,0xF2},{0x00,0xC7},{0x00,'\n'},{0x00,0xD8},
+	{0x00,0xF2},{0x00,0xE7},{0x00,'\n'},{0x00,0xD8},
 	{0x00,0xF8},{0x00,'\r'},{0x00,0xC5},{0x00,0xE5},
 	{0x03,0x94},{0x00,0x5f},{0x03,0xA6},{0x03,0x93},
 	{0x03,0x9B},{0x03,0xA9},{0x03,0xA0},{0x03,0xA8},
@@ -681,14 +707,11 @@ void FindDefaultAlphabetLen(const unsigned char *src, int *srclen, int *smslen,
 	*smslen = current;
 }
 
-#ifndef ENABLE_LGPL
-#  define ByteMask ((1 << Bits) - 1)
-#endif
+#define ByteMask ((1 << Bits) - 1)
 
 int GSM_UnpackEightBitsToSeven(int offset, int in_length, int out_length,
                            unsigned char *input, unsigned char *output)
 {
-#ifndef ENABLE_LGPL
 	/* (c) by Pavel Janik and Pawel Kot */
 
         unsigned char *OUTPUT 	= output; /* Current pointer to the output buffer */
@@ -724,14 +747,10 @@ int GSM_UnpackEightBitsToSeven(int offset, int in_length, int out_length,
         }
 
         return OUTPUT - output;
-#else
-	return 0;
-#endif
 }
 
 int GSM_PackSevenBitsToEight(int offset, unsigned char *input, unsigned char *output, int length)
 {
-#ifndef ENABLE_LGPL
 	/* (c) by Pavel Janik and Pawel Kot */
 
         unsigned char 	*OUTPUT = output; /* Current pointer to the output buffer */
@@ -763,9 +782,6 @@ int GSM_PackSevenBitsToEight(int offset, unsigned char *input, unsigned char *ou
                 INPUT++;
         }
         return (OUTPUT - output);
-#else
-	return 0;
-#endif
 }
 
 void GSM_UnpackSemiOctetNumber(unsigned char *retval, unsigned char *Number, bool semioctet)
@@ -867,10 +883,13 @@ int GSM_PackSemiOctetNumber(unsigned char *Number, unsigned char *Output, bool s
 	return length / 2 + 1;
 }
 
-void CopyUnicodeString(unsigned char *Dest, unsigned char *Source)
+void CopyUnicodeString(unsigned char *Dest, const unsigned char *Source)
 {
 	int j = 0;
 
+	/* No need to copy if both are on same address */
+	if (Dest == Source) return;
+
 	while (Source[j]!=0x00 || Source[j+1]!=0x00) {
 		Dest[j]		= Source[j];
 		Dest[j+1]	= Source[j+1];
@@ -899,7 +918,7 @@ void ReverseUnicodeString(unsigned char *String)
 
 /* All input is in Unicode. First char can show Unicode minor/major order.
    Output is Unicode string in Gammu minor/major order */
-void ReadUnicodeFile(unsigned char *Dest, unsigned char *Source)
+void ReadUnicodeFile(unsigned char *Dest, const unsigned char *Source)
 {
 	int j = 0, current = 0;
 
@@ -920,12 +939,12 @@ void ReadUnicodeFile(unsigned char *Dest, unsigned char *Source)
 	Dest[current++]	= 0;
 }
 
-int GetBit(unsigned char *Buffer, int BitNum)
+INLINE int GetBit(unsigned char *Buffer, int BitNum)
 {
 	return Buffer[BitNum/8] & 1<<(7-(BitNum%8));
 }
 
-int SetBit(unsigned char *Buffer, int BitNum)
+INLINE int SetBit(unsigned char *Buffer, int BitNum)
 {
 	return Buffer[BitNum/8] |= 1<<(7-(BitNum%8));
 }
@@ -1110,21 +1129,79 @@ void DecodeUnicodeSpecialNOKIAChars(unsigned char *dest, const unsigned char *sr
 	dest[current++] = 0x00;
 }
 
-bool mystrncasecmp(unsigned const char *a, unsigned const char *b, int num)
+#ifndef HAVE_STRNCASECMP
+#define TOLOWER(Ch) (isupper (Ch) ? tolower (Ch) : (Ch))
+/**
+ * Case insensitive string comparator
+ * Copyright (C) 1998, 1999, 2005 Free Software Foundation, Inc.
+ */
+int strncasecmp (const char *s1, const char *s2, size_t n)
 {
-	int i;
+	register const unsigned char *p1 = (const unsigned char *) s1;
+	register const unsigned char *p2 = (const unsigned char *) s2;
+	unsigned char c1, c2;
 
-	if (a == NULL || b == NULL) return false;
+	if (p1 == p2 || n == 0)
+		return 0;
 
-	if (num == 0) num = -1;
+	do {
+		c1 = TOLOWER (*p1);
+		c2 = TOLOWER (*p2);
 
-	for (i = 0; i != num; i++) {
-		if (a[i] == 0x00 && b[i] == 0x00) return true;
-		if (a[i] == 0x00 || b[i] == 0x00) return false;
-		if (tolower(a[i]) != tolower(b[i])) return false;
- 	}
-	return true;
+		if (--n == 0 || c1 == '\0')
+			break;
+
+		++p1;
+		++p2;
+	} while (c1 == c2);
+
+	if (UCHAR_MAX <= INT_MAX)
+		return c1 - c2;
+	else
+	/* On machines where 'char' and 'int' are types of the same size, the
+	difference of two 'unsigned char' values - including the sign bit -
+	doesn't fit in an 'int'.  */
+		return (c1 > c2 ? 1 : c1 < c2 ? -1 : 0);
 }
+#undef TOLOWER
+#endif
+
+#ifndef HAVE_STRCASECMP
+#define TOLOWER(Ch) (isupper (Ch) ? tolower (Ch) : (Ch))
+/**
+ * Case insensitive string comparator
+ * Copyright (C) 1998, 1999, 2005 Free Software Foundation, Inc.
+ */
+int strcasecmp (const char *s1, const char *s2)
+{
+	register const unsigned char *p1 = (const unsigned char *) s1;
+	register const unsigned char *p2 = (const unsigned char *) s2;
+	unsigned char c1, c2;
+
+	if (p1 == p2)
+		return 0;
+
+	do {
+		c1 = TOLOWER (*p1);
+		c2 = TOLOWER (*p2);
+
+		if (c1 == '\0')
+			break;
+
+		++p1;
+		++p2;
+	} while (c1 == c2);
+
+	if (UCHAR_MAX <= INT_MAX)
+		return c1 - c2;
+	else
+	/* On machines where 'char' and 'int' are types of the same size, the
+	difference of two 'unsigned char' values - including the sign bit -
+	doesn't fit in an 'int'.  */
+		return (c1 > c2 ? 1 : c1 < c2 ? -1 : 0);
+}
+#undef TOLOWER
+#endif
 
 /* Compares two Unicode strings without regarding to case.
  * Return true, when they're equal
@@ -1143,7 +1220,7 @@ bool mywstrncasecmp(unsigned const  char *a, unsigned const  char *b, int num)
 		if ((a[i*2] == 0x00 && a[i*2+1] == 0x00) || (b[i*2] == 0x00 && b[i*2+1] == 0x00)) return false;
  		wc  = a[i*2+1] | (a[i*2] << 8);
  		wc2 = b[i*2+1] | (b[i*2] << 8);
- 		if (mytowlower(wc) != mytowlower(wc2)) return false;
+ 		if (towlower(wc) != towlower(wc2)) return false;
  	}
 	return true;
 }
@@ -1185,18 +1262,16 @@ bool myiswspace(unsigned const char *src)
 #endif
 }
 
-/* FreeBSD boxes 4.7-STABLE does't have it, although it's ANSI standard */
-int mytowlower(wchar_t c)
-{
 #ifndef HAVE_TOWLOWER
+/* FreeBSD boxes 4.7-STABLE does't have it, although it's ANSI standard */
+wchar_t towlower(wchar_t c)
+{
 	unsigned char dest[10];
 
 	DecodeWithUnicodeAlphabet(c, dest);
 	return tolower(dest[0]);
-#else
-	return towlower(c);
-#endif
 }
+#endif
 
 /*
  * Following code is based on wcsstr from the GNU C Library, original
@@ -1216,8 +1291,10 @@ int mytowlower(wchar_t c)
 unsigned char *mywstrstr (const unsigned char *haystack, const unsigned char *needle)
 {
 /* One crazy define to convert unicode used in Gammu to standard wchar_t */
-#define tolowerwchar(x) (mytowlower((wchar_t)( (((&(x))[0] & 0xff) << 8) | (((&(x))[1] & 0xff)) )))
-	 register wchar_t b, c;
+#define tolowerwchar(x) (towlower((wchar_t)( (((&(x))[0] & 0xff) << 8) | (((&(x))[1] & 0xff)) )))
+	register wint_t a, b, c;
+	register const unsigned char *rhaystack, *rneedle;
+
 
 	if ((b = tolowerwchar(*needle)) != L'\0') {
 		haystack -= 2;				/* possible ANSI violation */
@@ -1234,9 +1311,6 @@ unsigned char *mywstrstr (const unsigned char *haystack, const unsigned char *ne
 		goto jin;
 
 		for (;;) {
-			register wchar_t a;
-			register const unsigned char *rhaystack, *rneedle;
-
 			do {
 				haystack += 2;
 				if ((a = tolowerwchar(*haystack)) == L'\0')
@@ -1286,50 +1360,54 @@ ret0:
 #undef tolowerwchar
 }
 
-/**
- * Gets line from buffer.
- *
- * @param MergeLines: determine whether merge lines ending with = (quoted printable)
- * @param Buffer: Data source to parse
- * @param Pos: Current position in data
- * @param OutBuffer: Buffer where line will be written
- * @param MaxLen: Maximal length of data to write
- */
 void MyGetLine(unsigned char *Buffer, int *Pos, unsigned char *OutBuffer, int MaxLen, bool MergeLines)
 {
 	bool skip = false;
+	bool quoted_printable = false;
+	size_t pos;
+	int tmp;
 
 	OutBuffer[0] = 0;
+	pos = 0;
 	if (Buffer == NULL) return;
-	while (1) {
-		if ((*Pos) >= MaxLen) return;
+	while ((*Pos) < MaxLen) {
 		switch (Buffer[*Pos]) {
 		case 0x00:
 			return;
 		case 0x0A:
-			if (strlen(OutBuffer) != 0 && !skip) {
-				if (MergeLines && OutBuffer[strlen(OutBuffer) - 1] == '=') {
-					OutBuffer[strlen(OutBuffer) - 1] = 0;
-					skip = true;
-				} else {
-					return;
-				}
-			}
-			break;
 		case 0x0D:
-			if (strlen(OutBuffer) != 0 && !skip) {
-				if (MergeLines && OutBuffer[strlen(OutBuffer) - 1] == '=') {
-					OutBuffer[strlen(OutBuffer) - 1] = 0;
-					skip = true;
-				} else {
-					return;
+			if (pos != 0 && !skip) {
+				if (MergeLines) {
+					/* (Quote printable new line) Does string end with = ? */
+					if (OutBuffer[pos - 1] == '=' && quoted_printable) {
+						pos--;
+						OutBuffer[pos] = 0;
+						skip = true;
+						break;
+					}
+					/* (vCard continuation) Next line start with space? */
+					tmp = *Pos + 1;
+					if (Buffer[*Pos + 1] == 0x0a || Buffer[*Pos + 1] == 0x0d) {
+						tmp += 1;
+					}
+					if (Buffer[tmp] == ' ') {
+						*Pos = tmp;
+						break;
+					}
 				}
+				return;
 			}
 			break;
-		default  :
+		default:
+			/* Detect quoted printable for possible escaping */
+			if (Buffer[*Pos] == ':' &&
+					strstr(OutBuffer, ";ENCODING=QUOTED-PRINTABLE") != NULL) {
+				quoted_printable = true;
+			}
 			skip = false;
-			OutBuffer[strlen(OutBuffer) + 1] = 0;
-			OutBuffer[strlen(OutBuffer)]     = Buffer[*Pos];
+			OutBuffer[pos]     = Buffer[*Pos];
+			pos++;
+			OutBuffer[pos] = 0;
 		}
 		(*Pos)++;
 	}
@@ -1356,7 +1434,7 @@ void StringToDouble(char *text, double *d)
 }
 
 /* When char can be converted, convert it from Unicode to UTF8 */
-int EncodeWithUTF8Alphabet2(unsigned char mychar1, unsigned char mychar2, unsigned char *ret)
+int EncodeWithUTF8Alphabet(unsigned char mychar1, unsigned char mychar2, unsigned char *ret)
 {
 	int src = mychar1*256+mychar2;
 
@@ -1385,9 +1463,11 @@ bool EncodeUTF8QuotedPrintable(unsigned char *dest, const unsigned char *src)
 	int		i,j=0,z,w;
 	unsigned char	mychar[3];
 	bool		retval = false;
+	int len;
+	len = UnicodeLength(src);
 
-	for (i = 0; i < (int)(UnicodeLength(src)); i++) {
-		z = EncodeWithUTF8Alphabet2(src[i*2],src[i*2+1],mychar);
+	for (i = 0; i < len; i++) {
+		z = EncodeWithUTF8Alphabet(src[i*2],src[i*2+1],mychar);
 		if (z>1) {
 			for (w=0;w<z;w++) {
 				sprintf(dest+j, "=%02X",mychar[w]);
@@ -1415,7 +1495,7 @@ bool EncodeUTF8(unsigned char *dest, const unsigned char *src)
 	bool		retval = false;
 
 	for (i = 0; i < (int)(UnicodeLength(src)); i++) {
-		z = EncodeWithUTF8Alphabet2(src[i*2],src[i*2+1],mychar);
+		z = EncodeWithUTF8Alphabet(src[i*2],src[i*2+1],mychar);
 		if (z>1) {
 			memcpy(dest+j,mychar,z);
 			j+=z;
@@ -1429,7 +1509,7 @@ bool EncodeUTF8(unsigned char *dest, const unsigned char *src)
 }
 
 /* Decode UTF8 char to Unicode char */
-int DecodeWithUTF8Alphabet2(unsigned char *src, wchar_t *dest, int len)
+int DecodeWithUTF8Alphabet(unsigned char *src, wchar_t *dest, int len)
 {
 	if (len < 1) return 0;
 	if (src[0] < 128) {
@@ -1489,16 +1569,16 @@ void DecodeUTF8QuotedPrintable(unsigned char *dest, const unsigned char *src, in
 				break;
 			}
 			mychar[z] = 16*DecodeWithHexBinAlphabet(src[z*3+i+1])+DecodeWithHexBinAlphabet(src[z*3+i+2]);
-			/* Is it plain ASCII? */
-			if (z==0 && mychar[0]<194) break;
 			z++;
+			/* Is it plain ASCII? */
+			if (z == 1 && mychar[0] < 194) break;
 			/* Do we already have valid UTF-8 char? */
-			if (DecodeWithUTF8Alphabet2(mychar,&ret,z) == z) break;
+			if (DecodeWithUTF8Alphabet(mychar, &ret, z) == z) break;
 		}
 		if (z>0) {
-			i+=z*3;
+			i += z * 3;
 			// we ignore wrong sequence
-			if (DecodeWithUTF8Alphabet2(mychar,&ret,z)==0) continue;
+			if (DecodeWithUTF8Alphabet(mychar,&ret,z)==0) continue;
 		} else {
 			i+=EncodeWithUnicodeAlphabet(&src[i], &ret);
 		}
@@ -1515,7 +1595,7 @@ void DecodeUTF8(unsigned char *dest, const unsigned char *src, int len)
 	wchar_t		ret;
 
 	while (i < len) {
-		z = DecodeWithUTF8Alphabet2((unsigned char *)src+i,&ret,len-i);
+		z = DecodeWithUTF8Alphabet((unsigned char *)src+i,&ret,len-i);
 		if (z<2) {
 			i+=EncodeWithUnicodeAlphabet(&src[i], &ret);
 		} else {
@@ -1528,6 +1608,83 @@ void DecodeUTF8(unsigned char *dest, const unsigned char *src, int len)
 	dest[j++] = 0;
 }
 
+void DecodeXMLUTF8(unsigned char *dest, const unsigned char *src, int len)
+{
+	char *tmp;
+	char *pos, *pos_end;
+	const char *lastpos;
+	char *entity;
+	unsigned long int c;
+	int tmplen;
+
+	/* Allocate buffer */
+	tmp = (char *)calloc(len + 1, sizeof(char));
+	if (tmp == NULL) {
+		/* We have no memory for XML decoding */
+		DecodeUTF8(dest, src, len);
+		return;
+	}
+
+	/* Find ampersand and decode the */
+	lastpos = src;
+	while ((lastpos != 0) && ((pos = strchr(lastpos, '&')) != NULL)) {
+		/* Store current string */
+		strncat(tmp, lastpos, pos - lastpos);
+		lastpos = pos;
+		/* Skip ampersand */
+		pos++;
+		/* Detect end of string */
+		if (pos == 0) break;
+		/* Find entity length */
+		pos_end = strchr(pos, ';');
+		if (pos_end - pos > 6 || pos_end == NULL) {
+			if (pos_end == NULL) {
+				dbgprintf("No entity end found, ignoring!\n");
+			} else {
+				dbgprintf("Too long html entity, ignoring!\n");
+			}
+			strncat(tmp, lastpos, 1);
+			lastpos++;
+			continue;
+		}
+		/* Create entity */
+		/* strndup would be better, but not portable */
+		entity = strdup(pos);
+		entity[pos_end - pos] = 0;
+		dbgprintf("Found XML entity: %s\n", entity);
+		if (entity == NULL) break;
+		if (entity[0] == '#') {
+			if (entity[1] == 'x' || entity[1] == 'X') {
+				c = strtoull(entity + 2, NULL, 16);
+			} else {
+				c = strtoull(entity + 1, NULL, 10);
+			}
+			dbgprintf("Unicode char 0x%04lx\n", c);
+			tmplen = strlen(tmp);
+			tmplen += EncodeWithUTF8Alphabet((c >> 8) & 0xff, c & 0xff, tmp + tmplen);
+			tmp[tmplen] = 0;
+		} else if (strcmp(entity, "amp") == 0) {
+			strcat(tmp, "&");
+		} else if (strcmp(entity, "apos") == 0) {
+			strcat(tmp, "'");
+		} else if (strcmp(entity, "gt") == 0) {
+			strcat(tmp, ">");
+		} else if (strcmp(entity, "lt") == 0) {
+			strcat(tmp, "<");
+		} else if (strcmp(entity, "quot") == 0) {
+			strcat(tmp, "\"");
+		} else {
+			dbgprintf("Could not decode XML entity!\n");
+			strncat(tmp, lastpos, pos_end - pos + 1);
+		}
+		lastpos = pos_end + 1;
+	}
+	/* Copy rest of string */
+	strcat(tmp, lastpos);
+	DecodeUTF8(dest, tmp, strlen(tmp));
+	free(tmp);
+}
+
 void DecodeUTF7(unsigned char *dest, const unsigned char *src, int len)
 {
 	int 		i=0,j=0,z,p;
@@ -1658,6 +1815,58 @@ int DecodeBASE64(const unsigned char *Input, unsigned char *Output, int Length)
 	return outpos;
 }
 
+#ifdef ICONV_FOUND
+
+#include <iconv.h>
+
+#ifdef ICONV_SECOND_ARGUMENT_IS_CONST
+#  define SECOND_ICONV_ARG const char *
+#else
+#  define SECOND_ICONV_ARG char *
+#endif
+
+bool IconvDecode(const char *charset, const char *input, const size_t inlen, unsigned char *output, size_t outlen)
+{
+	iconv_t ic;
+	size_t rest = inlen;
+	SECOND_ICONV_ARG in;
+	char *out;
+
+	ic = iconv_open("UCS-2BE", charset);
+	if (ic == (iconv_t)(-1)) return false;
+
+	/* I know I loose const here, but it's iconv choice... */
+	in = (SECOND_ICONV_ARG)input;
+	out = output;
+	iconv(ic, &in, &rest, &out, &outlen);
+
+	iconv_close(ic);
+
+	return (rest == 0);
+}
+
+bool IconvEncode(const char *charset, const unsigned char *input, const size_t inlen, char *output, size_t outlen)
+{
+	iconv_t ic;
+	size_t rest = inlen;
+	SECOND_ICONV_ARG in;
+	char *out;
+
+	ic = iconv_open(charset, "UCS-2BE");
+	if (ic == (iconv_t)(-1)) return false;
+
+	/* I know I loose const here, but it's iconv choice... */
+	in = (SECOND_ICONV_ARG)input;
+	out = output;
+	iconv(ic, &in, &rest, &out, &outlen);
+
+	iconv_close(ic);
+
+	return (rest == 0);
+}
+#endif
+
+
 /* How should editor hadle tabs in this file? Add editor commands here.
  * vim: noexpandtab sw=8 ts=8 sts=8:
  */
