@@ -1,6 +1,10 @@
 /* (c) 2002-2005 by Marcin Wiacek and Michal Cihar */
 /* Checking used compiler (c) 2002 by Michal Cihar */
 
+#include <gammu-config.h>
+#include <gammu-misc.h>
+#include "misc.h"
+
 #include <string.h>
 #include <ctype.h>
 #include <time.h>
@@ -9,29 +13,104 @@
 #include <locale.h>
 #include <sys/timeb.h>
 #ifdef WIN32
-#  include "windows.h"
+#  define WIN32_LEAN_AND_MEAN
+#  include <windows.h>
 #endif
-#if defined(linux) || defined(__linux) || defined(__linux__)
+#ifdef HAVE_SYS_UTSNAME_H
 #  include <sys/utsname.h>
 #endif
 
-#include "../gsmstate.h"
-#include "misc.h"
+/**
+ * Recalculates struct tm content. We can not use mktime directly, as it
+ * works only for dates > 1970. Day of week calculation is nased on article
+ * in Polish PC-Kurier 8/1998 page 104.
+ *
+ * @see http://www.pckurier.pl
+ */
+int RecalcDateTime(struct tm *st, const int year, const int month, const int day, const int hour, const int minute, const int second)
+{
+	const int days[] = {31,28,31,30,31,30,31,31,30,31,30,31};
+	int i, p, q, r;
 
-/* Based on article in Polish PC-Kurier 8/1998 page 104
- * Archive on http://www.pckurier.pl
+	memset(st, 0, sizeof(*st));
+
+	/* Calculate day of year */
+	st->tm_yday = day;
+	for (i = 0; i < month - 1; i++)
+		st->tm_yday += days[i];
+
+	/* Calculate day of week */
+	p = (14 - month) / 12;
+	q = month + 12 * p - 2;
+	r = year - p;
+	st->tm_wday = (day + (31 * q) / 12 + r + r / 4 - r / 100 + r / 400) % 7;
+
+
+	st->tm_hour = hour;
+	st->tm_min = minute;
+	st->tm_sec = second;
+	st->tm_year = year - 1900;
+	st->tm_mon = month - 1;
+	st->tm_mday = day;
+
+	return 1;
+}
+
+
+/**
+ * Recalculates struct tm content.
+ */
+int RecalcDate(struct tm *st, const int year, const int month, const int day)
+{
+	return RecalcDateTime(st, year, month, day, 0, 0, 0);
+}
+
+
+/**
+ * Return day of year index.
+ */
+int GetDayOfYear(int year, int month, int day)
+{
+	struct tm st;
+
+	RecalcDate(&st, year, month, day);
+
+	return st.tm_yday;
+}
+
+/**
+ * Return day of week index.
+ */
+int GetWeekOfMonth(int year, int month, int day)
+{
+	struct tm st;
+
+	RecalcDate(&st, year, month, day);
+
+	return 1 + (day - st.tm_wday) / 7;
+}
+
+/**
+ * Return day of week index.
+ */
+int GetDayOfWeek(int year, int month, int day)
+{
+	struct tm st;
+
+	RecalcDate(&st, year, month, day);
+
+	return st.tm_wday;
+}
+
+/**
+ * Return textual representation of day of week;
  */
 char *DayOfWeek (int year, int month, int day)
 {
-	int 		p,q,r,w;
 	static char 	DayOfWeekChar[10];
 
-	p=(14-month) / 12;
-	q=month+12*p-2;
-	r=year-p;
-	w=(day+(31*q) / 12 + r + r / 4 - r / 100 + r / 400) % 7;
 	strcpy(DayOfWeekChar,"");
-	switch (w) {
+	switch (GetDayOfWeek(year, month, day)) {
 		case 0: strcpy(DayOfWeekChar,"Sun"); break;
 		case 1: strcpy(DayOfWeekChar,"Mon"); break;
 		case 2: strcpy(DayOfWeekChar,"Tue"); break;
@@ -102,7 +181,7 @@ GSM_DateTime GSM_AddTime (GSM_DateTime DT , GSM_DeltaTime delta)
 
 	/* TODO: This works only for dates after 1970. But birthday dates may be before, so a more general
 	   method than mktime /localtime should be used. */
-	t_time = mktime (&tm_time);		
+	t_time = mktime (&tm_time);
 	t_time = t_time + delta.Second + 60* (delta.Minute + 60* (delta.Hour + 24*delta.Day));
 
 	Fill_GSM_DateTime ( &Date, t_time);
@@ -132,38 +211,14 @@ char *OSDateTime (GSM_DateTime dt, bool TimeZone)
 {
 	struct tm 	timeptr;
 	static char 	retval[200],retval2[200];
-	int 		p,q,r,w;
 
 #ifdef WIN32
 	setlocale(LC_ALL, ".OCP");
 #endif
 
-	/* Based on article in Polish PC-Kurier 8/1998 page 104
-	 * Archive on http://www.pckurier.pl
-	 */
-	p=(14-dt.Month) / 12;
-	q=dt.Month+12*p-2;
-	r=dt.Year-p;
-	w=(dt.Day+(31*q) / 12 + r + r / 4 - r / 100 + r / 400) % 7;
+	RecalcDateTime(&timeptr, dt.Year, dt.Month, dt.Day, dt.Hour, dt.Minute, dt.Second);
 
-	timeptr.tm_yday 	= 0; 			/* FIXME */
-	timeptr.tm_isdst 	= -1; 			/* FIXME */
-	timeptr.tm_year 	= dt.Year - 1900;
-	timeptr.tm_mon  	= dt.Month - 1;
-	timeptr.tm_mday 	= dt.Day;
-	timeptr.tm_hour 	= dt.Hour;
-	timeptr.tm_min  	= dt.Minute;
-	timeptr.tm_sec  	= dt.Second;
-	timeptr.tm_wday 	= w;
-#ifdef _BSD_SOURCE
-	timeptr.tm_zone		= NULL;
-#endif
-
-#ifdef WIN32
-	strftime(retval2, 200, "%#c", &timeptr);
-#else
 	strftime(retval2, 200, "%c", &timeptr);
-#endif
 	if (TimeZone) {
 		if (dt.Timezone >= 0) {
 			sprintf(retval," +%02i00",dt.Timezone);
@@ -195,20 +250,11 @@ char *OSDate (GSM_DateTime dt)
 {
 	struct tm 	timeptr;
 	static char 	retval[200],retval2[200];
-	int 		p,q,r,w;
 
 #ifdef WIN32
 	setlocale(LC_ALL, ".OCP");
 #endif
 
-	/* Based on article in Polish PC-Kurier 8/1998 page 104
-	 * Archive on http://www.pckurier.pl
-	 */
-	p=(14-dt.Month) / 12;
-	q=dt.Month+12*p-2;
-	r=dt.Year-p;
-	w=(dt.Day+(31*q) / 12 + r + r / 4 - r / 100 + r / 400) % 7;
-
 	timeptr.tm_yday 	= 0; 			/* FIXME */
 	timeptr.tm_isdst 	= -1; 			/* FIXME */
 	timeptr.tm_year 	= dt.Year - 1900;
@@ -217,16 +263,13 @@ char *OSDate (GSM_DateTime dt)
 	timeptr.tm_hour 	= dt.Hour;
 	timeptr.tm_min  	= dt.Minute;
 	timeptr.tm_sec  	= dt.Second;
-	timeptr.tm_wday 	= w;
+	timeptr.tm_wday 	= GetDayOfWeek(dt.Year, dt.Month, dt.Day);
 #ifdef _BSD_SOURCE
 	timeptr.tm_zone		= NULL;
 #endif
 
-#ifdef WIN32
-	strftime(retval2, 200, "%#x", &timeptr);
-#else
 	strftime(retval2, 200, "%x", &timeptr);
-#endif
+
 	/* If don't have weekday name, include it */
 	strftime(retval, 200, "%A", &timeptr);
 	if (strstr(retval2,retval)==NULL) {
@@ -262,9 +305,9 @@ bool CheckDate(GSM_DateTime *date)
 
 bool CheckTime(GSM_DateTime *date)
 {
-	return date->Hour <= 23 && date->Hour >= 0 &&
-		date->Minute <= 59 && date->Minute >= 0 &&
-		date->Second <= 59 && date->Second >= 0;
+	return date->Hour <= 23 &&
+		date->Minute <= 59 &&
+		date->Second <= 59;
 }
 
 int GetLine(FILE *File, char *Line, int count)
@@ -273,9 +316,8 @@ int GetLine(FILE *File, char *Line, int count)
 
 	if (fgets(Line, count, File) != NULL) {
 		num = strlen(Line) - 1;
-		while(1) {
+		while (num > 0) {
 			if (Line[num] != '\n' && Line[num] != '\r') break;
-			if (num == 0) break;
 			Line[num--] = 0;
 		}
 		return strlen(Line);
@@ -326,18 +368,21 @@ char *GetLineString(unsigned char *message, GSM_Lines lines, int start)
 	return retval;
 }
 
+int GetLineLength(unsigned char *message UNUSED, GSM_Lines lines, int start)
+{
+	return lines.numbers[start*2-2+1]-lines.numbers[start*2-2];
+}
+
 void CopyLineString(unsigned char *dest, unsigned char *src, GSM_Lines lines, int start)
 {
 	memcpy(dest,GetLineString(src, lines, start),strlen(GetLineString(src, lines, start)));
 	dest[strlen(GetLineString(src, lines, start))] = 0;
 }
 
-Debug_Info di = {0,NULL,false,""};
+GSM_Debug_Info di = {0,NULL,false,"",false};
 
 #ifdef DEBUG
-#ifdef __GNUC__
-__attribute__((format(printf, 1, 2)))
-#endif
+PRINTF_STYLE(1, 2)
 int dbgprintf(const char *format, ...)
 {
 	va_list			argp;
@@ -346,11 +391,14 @@ int dbgprintf(const char *format, ...)
 	unsigned char		buffer[2000];
 	GSM_DateTime 		date_time;
 
+	va_start(argp, format);
+	result = vsprintf(buffer, format, argp);
+	va_end(argp);
+
 	if (di.df != NULL && (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE)) {
-		va_start(argp, format);
-		result = vsprintf(buffer, format, argp);
 		strcat(nextline, buffer);
-		if (strstr(buffer, "\n")) {
+
+		if (strstr(buffer, "\n") != NULL) {
 			if (di.dl == DL_TEXTALLDATE) {
 				GSM_GetCurrentDateTime(&date_time);
 				fprintf(di.df,"%s %4d/%02d/%02d %02d:%02d:%02d: %s",
@@ -358,22 +406,19 @@ int dbgprintf(const char *format, ...)
 		                	date_time.Year, date_time.Month, date_time.Day,
 		                	date_time.Hour, date_time.Minute, date_time.Second,nextline);
 			} else {
-				fprintf(di.df,"%s",nextline);
+				fprintf(di.df, "%s", nextline);
 			}
 			strcpy(nextline, "");
 		}
-		fflush(di.df);
-		va_end(argp);
+
 		return result;
 	}
 	return 0;
 }
 #endif
 
-#ifdef __GNUC__
-__attribute__((format(printf, 2, 3)))
-#endif
-int smfprintf(Debug_Info *d, const char *format, ...)
+PRINTF_STYLE(2, 3)
+int smfprintf(GSM_Debug_Info *d, const char *format, ...)
 {
         va_list 		argp;
 	int 			result=0;
@@ -396,6 +441,7 @@ int smfprintf(Debug_Info *d, const char *format, ...)
 
 	va_start(argp, format);
 	result = vsprintf(buffer, format, argp);
+	va_end(argp);
 	pos = buffer;
 
 	while (*pos != 0) {
@@ -442,28 +488,42 @@ int smfprintf(Debug_Info *d, const char *format, ...)
 	}
 
 	/* Flush buffers, this might be configurable, but it could cause drop of last log messages */
-	fflush(d->df);
+	fflush(f);
 
-	va_end(argp);
 	return result;
 }
 
-bool GSM_SetDebugLevel(char *info, Debug_Info *di)
+bool GSM_SetDebugLevel(const char *info, GSM_Debug_Info *privdi)
 {
-	if (info == NULL)			{di->dl = DL_NONE;	 	return true;}
-	if (!strcmp(info,"nothing")) 		{di->dl = DL_NONE;	 	return true;}
-	if (!strcmp(info,"text")) 		{di->dl = DL_TEXT;	 	return true;}
-	if (!strcmp(info,"textall")) 		{di->dl = DL_TEXTALL;    	return true;}
-	if (!strcmp(info,"binary"))  		{di->dl = DL_BINARY;     	return true;}
-	if (!strcmp(info,"errors"))  		{di->dl = DL_TEXTERROR;  	return true;}
-	if (!strcmp(info,"textdate")) 		{di->dl = DL_TEXTDATE;	 	return true;}
-	if (!strcmp(info,"textalldate")) 	{di->dl = DL_TEXTALLDATE;    	return true;}
-	if (!strcmp(info,"errorsdate"))  	{di->dl = DL_TEXTERRORDATE;  	return true;}
+	if (info == NULL)			{privdi->dl = DL_NONE;	 	return true;}
+	if (!strcmp(info,"nothing")) 		{privdi->dl = DL_NONE;	 	return true;}
+	if (!strcmp(info,"text")) 		{privdi->dl = DL_TEXT;	 	return true;}
+	if (!strcmp(info,"textall")) 		{privdi->dl = DL_TEXTALL;    	return true;}
+	if (!strcmp(info,"binary"))  		{privdi->dl = DL_BINARY;     	return true;}
+	if (!strcmp(info,"errors"))  		{privdi->dl = DL_TEXTERROR;  	return true;}
+	if (!strcmp(info,"textdate")) 		{privdi->dl = DL_TEXTDATE;	 	return true;}
+	if (!strcmp(info,"textalldate")) 	{privdi->dl = DL_TEXTALLDATE;    	return true;}
+	if (!strcmp(info,"errorsdate"))  	{privdi->dl = DL_TEXTERRORDATE;  	return true;}
 	return false;
 }
 
+bool GSM_SetDebugCoding(const char *info, GSM_Debug_Info *privdi)
+{
+	privdi->coding = info;
+	return true;
+}
+
+bool GSM_SetDebugGlobal(bool info, GSM_Debug_Info *privdi)
+{
+	if (privdi->use_global && info) {
+		GSM_SetDebugFileDescriptor(NULL, privdi);
+	}
+	privdi->use_global = info;
+	return true;
+}
+
 /* Dumps a message */
-void DumpMessage(Debug_Info *d, const unsigned char *message, int messagesize)
+void DumpMessage(GSM_Debug_Info *d, const unsigned char *message, int messagesize)
 {
 	int 		i,j=0,len=16;
 	unsigned char	buffer[200];
@@ -503,46 +563,13 @@ void DumpMessage(Debug_Info *d, const unsigned char *message, int messagesize)
 	if (j != 0) smfprintf(d, "%s\n", buffer);
 }
 
-#ifdef WIN32
-#include <malloc.h>
-
-bool HeapCheck(char* loc) {
-   /* Check heap status */
-   int heapstatus = _heapchk();
-   switch( heapstatus )
-   {
-   case _HEAPOK:
-		dbgprintf("%s: OK - heap is fine\n", loc );
-		return false;
-		break;
-   case _HEAPEMPTY:
-	   dbgprintf("%s: OK - heap is empty\n", loc );
-		return false;
-		break;
-   case _HEAPBADBEGIN:
-	   dbgprintf( "%s: ERROR - bad start of heap\n", loc );
-		return true;
-		break;
-   case _HEAPBADNODE:
-	   dbgprintf( "%s: ERROR - bad node in heap\n", loc );
-		return true;
-		break;
-   }
-   return false;
-}
-#else
-bool HeapCheck(char* loc) {
-	return false;
-}
-#endif
-
-char *GetOS(void)
+const char *GetOS(void)
 {
 #ifdef WIN32
 	OSVERSIONINFOEX Ver;
 	bool		Extended = true;
 #endif
-#if defined(linux) || defined(__linux) || defined(__linux__)
+#ifdef HAVE_SYS_UTSNAME_H
 	struct utsname	Ver;
 #endif
 	static char 	Buffer[100] = {0x00};
@@ -570,29 +597,29 @@ char *GetOS(void)
 
 	/* no info about Win95 SP1, Win95 OSR2.1, Win95 OSR2.5.... */
 	if (Ver.dwMajorVersion == 4 && Ver.dwMinorVersion == 0 && Ver.dwBuildNumber == 950) {
-		sprintf(Buffer,"Win 95");
+		sprintf(Buffer,"Windows 95");
 	} else if (Ver.dwMajorVersion == 4 && Ver.dwMinorVersion == 0 && Ver.dwBuildNumber == 1111) {
-		sprintf(Buffer,"Win 95 OSR2.x");
+		sprintf(Buffer,"Windows 95 OSR2.x");
 
 	/* no info about Win98 SP1.... */
 	} else if (Ver.dwMajorVersion == 4 && Ver.dwMinorVersion == 10 && Ver.dwBuildNumber == 1998) {
-		sprintf(Buffer,"Win 98");
+		sprintf(Buffer,"Windows 98");
 	} else if (Ver.dwMajorVersion == 4 && Ver.dwMinorVersion == 10 && Ver.dwBuildNumber == 2222) {
-		sprintf(Buffer,"Win 98 SE");
+		sprintf(Buffer,"Windows 98 SE");
 
 	} else if (Ver.dwMajorVersion == 4 && Ver.dwMinorVersion == 90 && Ver.dwBuildNumber == 3000) {
-		sprintf(Buffer,"Win ME");
+		sprintf(Buffer,"Windows ME");
 
 	/* ---------------- NT family ------------------- */
 
 	} else if (Ver.dwMajorVersion == 4 && Ver.dwMinorVersion == 0 && Ver.dwBuildNumber == 1381) {
-		sprintf(Buffer,"Win NT 4.0");
+		sprintf(Buffer,"Windows NT 4.0");
 
 	} else if (Ver.dwMajorVersion == 5 && Ver.dwMinorVersion == 0 && Ver.dwBuildNumber == 2195) {
-		sprintf(Buffer,"Win 2000");
+		sprintf(Buffer,"Windows 2000");
 
 	} else if (Ver.dwMajorVersion == 5 && Ver.dwMinorVersion == 1 && Ver.dwBuildNumber == 2600) {
-		sprintf(Buffer,"Win XP");
+		sprintf(Buffer,"Windows XP");
 #if _MSC_VER > 1200 //6.0 has it undeclared
 		if (Extended) {
 			if (Ver.wSuiteMask & VER_SUITE_PERSONAL) {
@@ -604,18 +631,24 @@ char *GetOS(void)
 #endif
 
 	} else if (Ver.dwMajorVersion == 5 && Ver.dwMinorVersion == 2) {
-		sprintf(Buffer,"Win 2003");
+		sprintf(Buffer,"Windows 2003");
+
+	} else if (Ver.dwMajorVersion == 6 && Ver.dwMinorVersion == 0) {
+		sprintf(Buffer,"Windows Vista");
+
+	} else if (Ver.dwMajorVersion == 6 && Ver.dwMinorVersion > 0) {
+		sprintf(Buffer,"Windows Server 2007");
 
 	} else {
-		sprintf(Buffer, "Windows %i.%i.%i",Ver.dwMajorVersion,Ver.dwMinorVersion,Ver.dwBuildNumber);
+		sprintf(Buffer, "Windows %i.%i.%i",(int)Ver.dwMajorVersion,(int)Ver.dwMinorVersion,(int)Ver.dwBuildNumber);
 	}
 
 	if (Extended && Ver.wServicePackMajor != 0) {
 		sprintf(Buffer+strlen(Buffer)," SP%i",Ver.wServicePackMajor);
 	}
-#elif defined(linux) || defined(__linux) || defined(__linux__)
+#elif defined(HAVE_SYS_UTSNAME_H)
 	uname(&Ver);
-	sprintf(Buffer, "Linux, kernel %s",Ver.release);
+	sprintf(Buffer, "%s, kernel %s (%s)", Ver.sysname, Ver.release, Ver.version);
 #elif defined(__FreeBSD__)
 	sprintf(Buffer, "FreeBSD");
 #elif defined(__NetBSD__)
@@ -652,7 +685,7 @@ char *GetOS(void)
 	return Buffer;
 }
 
-char *GetCompiler(void)
+const char *GetCompiler(void)
 {
 	static char Buffer[100] = {0x00};
 
@@ -678,6 +711,10 @@ char *GetCompiler(void)
 	sprintf(Buffer, "gcc %i.%i", __GNUC__, __GNUC_MINOR__);
 #elif defined(__SUNPRO_CC)
 	sprintf(Buffer, "Sun C++ %x", __SUNPRO_CC);
+#elif defined(__INTEL_COMPILER)
+	sprintf(Buffer, "Intel Compiler %ld", __INTEL_COMPILER);
+#else
+	sprintf(Buffer, "unknown compiler");
 #endif
 
 	return Buffer;
