@@ -10,7 +10,6 @@
 
 #include "../../misc/coding/coding.h"
 #include "../../gsmcomon.h"
-#include "../../service/sms/gsmsms.h"
 #include "../pfunc.h"
 
 #include "atgen.h"
@@ -90,7 +89,7 @@ GSM_Error SIEMENS_ReplyGetBitmap(GSM_Protocol_Message msg, GSM_StateMachine *s)
 	else return ERR_UNKNOWN;
 }
 
-GSM_Error SIEMENS_ReplySetFunction (GSM_Protocol_Message msg, GSM_StateMachine *s,char *function)
+GSM_Error SIEMENS_ReplySetFunction (GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s,char *function)
 {
 	if (s->Protocol.Data.AT.EditMode) {
 	    s->Protocol.Data.AT.EditMode = false;
@@ -158,7 +157,7 @@ GSM_Error SIEMENS_ReplyGetRingtone(GSM_Protocol_Message msg, GSM_StateMachine *s
 	return ERR_NONE;
 }
 
-GSM_Error SIEMENS_GetRingtone(GSM_StateMachine *s, GSM_Ringtone *Ringtone, bool PhoneRingtone)
+GSM_Error SIEMENS_GetRingtone(GSM_StateMachine *s, GSM_Ringtone *Ringtone, bool PhoneRingtone UNUSED)
 {
 	unsigned char req[32];
 
@@ -173,7 +172,7 @@ GSM_Error SIEMENS_ReplySetRingtone(GSM_Protocol_Message msg, GSM_StateMachine *s
 	return SIEMENS_ReplySetFunction (msg, s, "Ringtone");
 }
 
-GSM_Error SIEMENS_SetRingtone(GSM_StateMachine *s, GSM_Ringtone *Ringtone, int *maxlength)
+GSM_Error SIEMENS_SetRingtone(GSM_StateMachine *s, GSM_Ringtone *Ringtone, int *maxlength UNUSED)
 {
 	GSM_Phone_Data *Phone = &s->Phone.Data;
 
@@ -245,12 +244,26 @@ GSM_Error SIEMENS_GetNextCalendar(GSM_StateMachine *s, GSM_CalendarEntry *Note,
 	return error;
 }
 
+GSM_Error SIEMENS_GetCalendar(GSM_StateMachine *s, GSM_CalendarEntry *Note)
+{
+	GSM_Phone_ATGENData	*Priv = &s->Phone.Data.Priv.ATGEN;
+	unsigned char 		req[32];
+
+	if (Priv->Manufacturer!=AT_Siemens) return ERR_NOTSUPPORTED;
+
+	s->Phone.Data.Cal = Note;
+	sprintf(req, "AT^SBNR=\"vcs\",%i\r",Note->Location);
+	smprintf(s, "Getting calendar note\n");
+
+	return GSM_WaitFor (s, req, strlen(req), 0x00, 4, ID_GetCalendarNote);
+}
+
 GSM_Error SIEMENS_ReplyAddCalendarNote(GSM_Protocol_Message msg, GSM_StateMachine *s)
 {
     return SIEMENS_ReplySetFunction (msg, s, "Calendar Note");
 }
 
-GSM_Error SIEMENS_ReplyDelCalendarNote(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error SIEMENS_ReplyDelCalendarNote(GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s)
 {
 	GSM_Phone_Data *Data = &s->Phone.Data;
 
@@ -267,12 +280,14 @@ GSM_Error SIEMENS_ReplyDelCalendarNote(GSM_Protocol_Message msg, GSM_StateMachin
 
 GSM_Error SIEMENS_DelCalendarNote(GSM_StateMachine *s, GSM_CalendarEntry *Note)
 {
-	unsigned char req[32];
+	GSM_Phone_ATGENData	*Priv = &s->Phone.Data.Priv.ATGEN;
+	unsigned char 		req[32];
 
 	if (s->Phone.Data.Priv.ATGEN.Manufacturer!=AT_Siemens) return ERR_NOTSUPPORTED;
 	s->Phone.Data.Cal = Note;
 	sprintf(req, "AT^SBNW=\"vcs\",%i,0\r",Note->Location);
 	smprintf(s, "Deleting calendar note\n");
+	Priv->FirstFreeCalendarPos = 0;
 	return GSM_WaitFor (s, req, strlen(req), 0x00, 4, ID_DeleteCalendarNote);
 }
 
@@ -296,15 +311,32 @@ GSM_Error SIEMENS_AddCalendarNote(GSM_StateMachine *s, GSM_CalendarEntry *Note)
 {
 	GSM_Phone_ATGENData	*Priv = &s->Phone.Data.Priv.ATGEN;
 	GSM_Error		error;
-	unsigned char 		req[500];
-	int			size=0;
+	unsigned char 		req[500], req1[32];
+	int			size=0, Location;
 
 	if (Priv->Manufacturer!=AT_Siemens) return ERR_NOTSUPPORTED;
 //	if (Note->Location==0x00) return ERR_INVALIDLOCATION;
 
-	s->Phone.Data.Cal = Note;
 	error=GSM_EncodeVCALENDAR(req,&size,Note,true,Siemens_VCalendar);
 
+	Note->Location		= Priv->FirstFreeCalendarPos;
+	s->Phone.Data.Cal 	= Note;
+	Note->EntriesNum 	= 0;
+	smprintf(s, "Getting VCALENDAR\n");
+	Location = Note->Location;
+	while (1){
+		Location++;
+		sprintf(req1, "AT^SBNR=\"vcs\",%i\r",Location);
+		error = GSM_WaitFor (s, req1, strlen(req1), 0x00, 4, ID_GetCalendarNote);
+		Note->Location			= Location;
+		Priv->FirstFreeCalendarPos	= Location;
+		if (error==ERR_EMPTY) break;
+		if (Location > MAX_VCALENDAR_LOCATION) {
+			Priv->FirstFreeCalendarPos = 0;
+			return ERR_FULL;
+		}
+		if (error!=ERR_NONE) return error;
+	}
 	return SetSiemensFrame (s,req,"vcs",Note->Location,ID_SetCalendarNote,size);
 }
 
