@@ -20,11 +20,12 @@
 
 #include "../../../gsmstate.h"
 #include "../../../misc/coding/coding.h"
-#include "../../../service/sms/gsmsms.h"
 #include "../../pfunc.h"
 #include "../nfunc.h"
 #include "dct3func.h"
 
+#include <gammu-config.h>
+
 #ifdef GSM_ENABLE_NOKIA_DCT3
 
 GSM_Error DCT3_DeleteWAPBookmark(GSM_StateMachine *s, GSM_WAPBookmark *bookmark)
@@ -84,7 +85,7 @@ GSM_Error DCT3_PressKey(GSM_StateMachine *s, GSM_KeyCode Key, bool Press)
 	}
 }
 
-GSM_Error DCT3_ReplyPlayTone(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error DCT3_ReplyPlayTone(GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s)
 {
 	smprintf(s, "Tone played\n");
 	return ERR_NONE;
@@ -135,7 +136,7 @@ GSM_Error DCT3_ReplyIncomingCB(GSM_Protocol_Message msg, GSM_StateMachine *s)
 	DecodeDefault(CB.Text, Buffer, i + 1, false, NULL);
 	smprintf(s, "Channel %i, text \"%s\"\n",CB.Channel,DecodeUnicodeString(CB.Text));
 	if (s->Phone.Data.EnableIncomingCB && s->User.IncomingCB!=NULL) {
-		s->User.IncomingCB(s->CurrentConfig->Device,CB);
+		s->User.IncomingCB(s,CB);
 	}
 	return ERR_NONE;
 }
@@ -182,7 +183,7 @@ GSM_Error DCT3_SetIncomingCB(GSM_StateMachine *s, bool enable)
 #endif
 }
 
-GSM_Error DCT3_ReplySetSMSC(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error DCT3_ReplySetSMSC(GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s)
 {
 	smprintf(s, "SMSC set\n");
 	return ERR_NONE;
@@ -221,7 +222,7 @@ GSM_Error DCT3_SetSMSC(GSM_StateMachine *s, GSM_SMSC *smsc)
 	return GSM_WaitFor (s, req, 35+UnicodeLength(smsc->Name), 0x02, 4, ID_SetSMSC);
 }
 
-GSM_Error DCT3_ReplyEnableSecurity(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error DCT3_ReplyEnableSecurity(GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s)
 {
 	smprintf(s, "State of security commands set\n");
 	return ERR_NONE;
@@ -273,13 +274,13 @@ GSM_Error DCT3_GetIMEI (GSM_StateMachine *s)
 	return GSM_WaitFor (s, req, 4, 0x40, 2, ID_GetIMEI);
 }
 
-GSM_Error DCT3_ReplySIMLogin(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error DCT3_ReplySIMLogin(GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s)
 {
 	smprintf(s, "Login for SIM card\n");
 	return ERR_NONE;
 }
 
-GSM_Error DCT3_ReplySIMLogout(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error DCT3_ReplySIMLogout(GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s)
 {
 	smprintf(s, "Logout for SIM card\n");
 	return ERR_NONE;
@@ -556,7 +557,7 @@ GSM_Error DCT3_GetNetworkInfo(GSM_StateMachine *s, GSM_NetworkInfo *netinfo)
 	return GSM_WaitFor (s, req, 4, 0x0a, 4, ID_GetNetworkInfo);
 }
 
-GSM_Error DCT3_ReplyDialCommand(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error DCT3_ReplyDialCommand(GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s)
 {
 	smprintf(s, "Answer for call commands\n");
 	return ERR_NONE;
@@ -1221,11 +1222,11 @@ GSM_Error DCT3_ReplySendSMSMessage(GSM_Protocol_Message msg, GSM_StateMachine *s
 	switch (msg.Buffer[3]) {
 	case 0x02:
 		smprintf(s, "SMS sent OK\n");
-		if (s->User.SendSMSStatus!=NULL) s->User.SendSMSStatus(s->CurrentConfig->Device,0,msg.Buffer[5]);
+		if (s->User.SendSMSStatus!=NULL) s->User.SendSMSStatus(s,0,msg.Buffer[5]);
 		return ERR_NONE;
 	case 0x03:
 		smprintf(s, "Error %i\n",msg.Buffer[6]);
-		if (s->User.SendSMSStatus!=NULL) s->User.SendSMSStatus(s->CurrentConfig->Device,msg.Buffer[6],-1);
+		if (s->User.SendSMSStatus!=NULL) s->User.SendSMSStatus(s,msg.Buffer[6],-1);
 		return ERR_NONE;
 	}
 	return ERR_UNKNOWNRESPONSE;
@@ -1515,7 +1516,7 @@ GSM_Error N61_91_ReplySetOpLogo(GSM_Protocol_Message msg, GSM_StateMachine *s)
 	return ERR_UNKNOWNRESPONSE;
 }
 
-GSM_Error N61_71_ReplyResetPhoneSettings(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error N61_71_ReplyResetPhoneSettings(GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s)
 {
 	smprintf(s, "Phone settings cleaned OK\n");
 	return ERR_NONE;
