@@ -13,13 +13,15 @@
  * to publish their code under.
  */
 
-#include "../../../gsmstate.h"
+#include <gammu-config.h>
 
 #ifdef GSM_ENABLE_NOKIA7110
 
 #include <string.h>
 #include <time.h>
 
+#include <gammu-nokia.h>
+
 #include "../../../misc/coding/coding.h"
 #include "../../../gsmcomon.h"
 #include "../../../service/gsmlogo.h"
@@ -111,9 +113,9 @@ static void N7110_GetSMSLocation(GSM_StateMachine *s, GSM_SMSMessage *sms, unsig
 
 	/* simulate flat SMS memory */
 	if (sms->Folder==0x00) {
-		ifolderid = sms->Location / PHONE_MAXSMSINFOLDER;
+		ifolderid = sms->Location / GSM_PHONE_MAXSMSINFOLDER;
 		*folderid = (ifolderid + 1) * 0x08;
-		*location = sms->Location - ifolderid * PHONE_MAXSMSINFOLDER;
+		*location = sms->Location - ifolderid * GSM_PHONE_MAXSMSINFOLDER;
 	} else {
 		*folderid = sms->Folder * 0x08;
 		*location = sms->Location;
@@ -125,7 +127,7 @@ static void N7110_GetSMSLocation(GSM_StateMachine *s, GSM_SMSMessage *sms, unsig
 static void N7110_SetSMSLocation(GSM_StateMachine *s, GSM_SMSMessage *sms, unsigned char folderid, int location)
 {
 	sms->Folder	= 0;
-	sms->Location	= (folderid / 0x08 - 1) * PHONE_MAXSMSINFOLDER + location;
+	sms->Location	= (folderid / 0x08 - 1) * GSM_PHONE_MAXSMSINFOLDER + location;
 	smprintf(s, "7110 folder %i & location %i -> SMS folder %i & location %i\n",
 		folderid,location,sms->Folder,sms->Location);
 }
@@ -154,6 +156,10 @@ static GSM_Error N7110_ReplyGetSMSFolders(GSM_Protocol_Message msg, GSM_StateMac
 			current=current+2+UnicodeLength(buffer)*2;
 			Data->SMSFolders->Folder[j].InboxFolder = false;
 			if (j==0) Data->SMSFolders->Folder[j].InboxFolder = true;
+			Data->SMSFolders->Folder[j].OutboxFolder = false;
+			/**
+			 * @todo We should detect outbox here.
+			 */
 			Data->SMSFolders->Folder[j].Memory = MEM_ME;
 			if (j==0 || j==1) Data->SMSFolders->Folder[j].InboxFolder = MEM_MT;
 		}
@@ -188,8 +194,8 @@ static GSM_Error N7110_ReplyGetSMSFolderStatus(GSM_Protocol_Message msg, GSM_Sta
 	smprintf(s, "Locations: ");
 	for (i=0;i<Priv->LastSMSFolder.Number;i++) {
 		Priv->LastSMSFolder.Location[i]=msg.Buffer[6+(i*2)]*256+msg.Buffer[(i*2)+7];
-		if (Priv->LastSMSFolder.Location[i] > PHONE_MAXSMSINFOLDER) {
-			smprintf(s, "Increase PHONE_MAXSMSINFOLDER\n");
+		if (Priv->LastSMSFolder.Location[i] > GSM_PHONE_MAXSMSINFOLDER) {
+			smprintf(s, "Increase GSM_PHONE_MAXSMSINFOLDER\n");
 			return ERR_UNKNOWNRESPONSE;
 		}
 		smprintf(s, "%i ",Priv->LastSMSFolder.Location[i]);
@@ -411,7 +417,7 @@ static GSM_Error N7110_GetSMSMessage(GSM_StateMachine *s, GSM_MultiSMSMessage *s
 {
 	GSM_Error		error;
 	unsigned char		folderid;
-	int			location;
+	unsigned int		location;
 	GSM_Phone_N7110Data	*Priv = &s->Phone.Data.Priv.N7110;
 	int			i;
 	bool			found = false;
@@ -433,7 +439,7 @@ static GSM_Error N7110_GetNextSMSMessage(GSM_StateMachine *s, GSM_MultiSMSMessag
 {
 	GSM_Phone_N7110Data	*Priv = &s->Phone.Data.Priv.N7110;
 	unsigned char		folderid;
-	int			location;
+	unsigned int		location;
 	GSM_Error		error;
 	int			i;
 	bool			findnextfolder = false;
@@ -570,6 +576,10 @@ static GSM_Error N7110_GetPictureImageLocation(GSM_StateMachine *s, GSM_Bitmap *
 					 0x00,			/* Folder ID */
 					 0x0f, 0x07};
 
+	/* Just to have some defined values */
+	*folder = 0;
+	*location = 0;
+
 	error=N7110_GetSMSFolders (s, &folders);
 	if (error!=ERR_NONE) return error;
 
@@ -633,7 +643,7 @@ static GSM_Error N7110_GetBitmap(GSM_StateMachine *s, GSM_Bitmap *Bitmap)
 		pbk.Location   = Bitmap->Location;
 		smprintf(s, "Getting caller group logo\n");
 		error=N7110_GetMemory(s,&pbk);
-		if (error==ERR_NONE) NOKIA_GetDefaultCallerGroupName(s,Bitmap);
+		if (error==ERR_NONE) NOKIA_GetDefaultCallerGroupName(Bitmap);
 		return error;
 	case GSM_OperatorLogo:
 		smprintf(s, "Getting operator logo\n");
@@ -832,13 +842,13 @@ static GSM_Error N7110_AddSMS(GSM_StateMachine *s, GSM_SMSMessage *sms)
 	return N7110_PrivSetSMSMessage(s, sms);
 }
 
-static GSM_Error N7110_ReplyClearOperatorLogo(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error N7110_ReplyClearOperatorLogo(GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s)
 {
 	smprintf(s, "Clearing operator logo.....\n");
 	return ERR_NONE;
 }
 
-static GSM_Error N7110_ReplySetOperatorLogo(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error N7110_ReplySetOperatorLogo(GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s)
 {
 	smprintf(s, "Setting operator logo.....\n");
 	return ERR_NONE;
@@ -1068,7 +1078,7 @@ static GSM_Error N7110_SetBitmap(GSM_StateMachine *s, GSM_Bitmap *Bitmap)
 	return ERR_NOTSUPPORTED;
 }
 
-static GSM_Error N7110_ReplyDeleteMemory(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error N7110_ReplyDeleteMemory(GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s)
 {
 	smprintf(s, "Phonebook entry deleted\n");
 	return ERR_NONE;
@@ -1109,7 +1119,7 @@ static GSM_Error N7110_SetMemory(GSM_StateMachine *s, GSM_MemoryEntry *entry)
 	req[12] = entry->Location >> 8;
 	req[13] = entry->Location & 0xff;
 
-	count   = count + N71_65_EncodePhonebookFrame(s, req+18, entry, &blocks, false, IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_VOICETAGS));
+	count   = count + N71_65_EncodePhonebookFrame(s, req+18, entry, &blocks, false, GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_VOICETAGS));
 	req[17] = blocks;
 
 	smprintf(s, "Writing phonebook entry\n");
@@ -1241,12 +1251,12 @@ static GSM_Error N7110_GetProfile(GSM_StateMachine *s, GSM_Profile *Profile)
 		error = GSM_WaitFor (s, req, 9, 0x39, 4, ID_GetProfile);
 		if (error!=ERR_NONE) return error;
 	}
-	NOKIA_GetDefaultProfileName(s, Profile);
+	NOKIA_GetDefaultProfileName(Profile);
 	Profile->Active = false;
 	return error;
 }
 
-static GSM_Error N7110_ReplySetProfileFeature(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error N7110_ReplySetProfileFeature(GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s)
 {
 	smprintf(s, "Profile feature set\n");
 	return ERR_NONE;
@@ -1334,7 +1344,7 @@ static GSM_Error N7110_ReplyIncomingSMS(GSM_Protocol_Message msg, GSM_StateMachi
 		sms.InboxFolder = true;
 		DCT3_DecodeSMSFrame(s, &sms,msg.Buffer+8);
 
-		s->User.IncomingSMS(s->CurrentConfig->Device,sms);
+		s->User.IncomingSMS(s,sms);
 	}
 	return ERR_NONE;
 }
@@ -1395,7 +1405,7 @@ static GSM_Error N7110_AddCalendar(GSM_StateMachine *s, GSM_CalendarEntry *Note)
 	return N71_65_AddCalendar2(s,Note);
 }
 
-static GSM_Error N7110_ReplyGetNetworkInfoError(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error N7110_ReplyGetNetworkInfoError(GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s)
 {
 	smprintf(s, "Probably means no PIN\n");
 	return ERR_SECURITYERROR;
@@ -1671,6 +1681,7 @@ GSM_Phone_Functions N7110Phone = {
  	NOTIMPLEMENTED,			/* 	AddSMSFolder		*/
  	NOTIMPLEMENTED,			/* 	DeleteSMSFolder		*/
 	DCT3_DialVoice,
+        NOTIMPLEMENTED,			/*	DialService		*/
 	N7110_AnswerCall,
 	DCT3_CancelCall,
  	NOTIMPLEMENTED,			/* 	HoldCall 		*/
@@ -1739,6 +1750,7 @@ GSM_Phone_Functions N7110Phone = {
 	NOTSUPPORTED,			/*	SetFileAttributes	*/
 	NOTSUPPORTED,			/*	GetFilePart		*/
 	NOTSUPPORTED,			/* 	AddFile			*/
+	NOTSUPPORTED,			/* 	SendFilePart		*/
 	NOTSUPPORTED, 			/* 	GetFileSystemStatus	*/
 	NOTSUPPORTED,			/*	DeleteFile		*/
 	NOTSUPPORTED,			/*	AddFolder		*/
