@@ -1,7 +1,7 @@
 /* (c) 2003-2006 by Marcin Wiacek */
 /* function for making CRC for filesystem (c) 2003 by Michael Schroeder */
 
-#include "../../../../gsmstate.h"
+#include <gammu-config.h>
 
 #ifdef GSM_ENABLE_NOKIA6510
 
@@ -11,7 +11,6 @@
 #include "../../../../misc/coding/coding.h"
 #include "../../../../gsmcomon.h"
 #include "../../../../service/gsmlogo.h"
-#include "../../../../service/sms/gsmsms.h"
 #include "../../nfunc.h"
 #include "../../nfuncold.h"
 #include "../../../pfunc.h"
@@ -935,7 +934,7 @@ GSM_Error N6510_ReplyGetFileFolderInfo2(GSM_Protocol_Message msg, GSM_StateMachi
 					    msg.Buffer[11]*256*256+
 					    msg.Buffer[12]*256+
 					    msg.Buffer[13];
-				smprintf(s,"Size %i bytes\n",File->Used);
+				smprintf(s,"Size " SIZE_T_FORMAT " bytes\n",File->Used);
 			}
 			File->ReadOnly = false;
 			if ((msg.Buffer[29] & 1) == 1) {
@@ -983,7 +982,7 @@ GSM_Error N6510_ReplyGetFileFolderInfo2(GSM_Protocol_Message msg, GSM_StateMachi
 static GSM_Error N6510_GetFileFolderInfo2(GSM_StateMachine *s, GSM_File *File)
 {
 	int		     	Pos=6;
-	unsigned char	   	req[200] = {
+	unsigned char	   	req[900] = {
 		N7110_FRAME_HEADER,0x6C,
 		0xFF, 0xFF}; 			// name length
 
@@ -1338,10 +1337,10 @@ static GSM_Error N6510_GetFolderListing2(GSM_StateMachine *s, GSM_File *File, bo
 	int			i;
 
 	if (start) {
-		if (mystrncasecmp(DecodeUnicodeString(File->ID_FullName),"a:",0)   ||
-		    mystrncasecmp(DecodeUnicodeString(File->ID_FullName),"a:\\",0) ||
-		    mystrncasecmp(DecodeUnicodeString(File->ID_FullName),"d:",0)   ||
-		    mystrncasecmp(DecodeUnicodeString(File->ID_FullName),"d:\\",0)) {
+		if (strcasecmp(DecodeUnicodeString(File->ID_FullName),"a:") == 0   ||
+		    strcasecmp(DecodeUnicodeString(File->ID_FullName),"a:\\") == 0 ||
+		    strcasecmp(DecodeUnicodeString(File->ID_FullName),"d:") == 0   ||
+		    strcasecmp(DecodeUnicodeString(File->ID_FullName),"d:\\") == 0) {
 		} else {
 			//we must check, if user gave folder name or not
 			error = N6510_GetFileFolderInfo2(s, File);
@@ -1524,12 +1523,12 @@ GSM_Error N6510_GetFolderListing(GSM_StateMachine *s, GSM_File *File, bool start
 	GSM_Error	error;
 	GSM_File	File2;
 
-	if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILESYSTEM)) return ERR_NOTSUPPORTED;
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILESYSTEM)) return ERR_NOTSUPPORTED;
 
 	if (DecodeUnicodeString(File->ID_FullName)[0] == 'c' ||
 	    DecodeUnicodeString(File->ID_FullName)[0] == 'C') {
-		if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_SERIES40_30) ||
-		    IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILE1)) {
+		if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_SERIES40_30) ||
+		    GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILE1)) {
 			return ERR_NOTSUPPORTED;
 		}
 
@@ -1542,7 +1541,7 @@ GSM_Error N6510_GetFolderListing(GSM_StateMachine *s, GSM_File *File, bool start
 		CopyUnicodeString(File->ID_FullName+UnicodeLength(File->ID_FullName)*2,File2.ID_FullName);
 		return error;
 	} else {
-		if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_FILES2)) {
+		if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_FILES2)) {
 			return N6510_GetFolderListing2(s,File,start);
 		} else {
 			return ERR_NOTSUPPORTED;
@@ -1556,22 +1555,22 @@ GSM_Error N6510_GetNextFileFolder(GSM_StateMachine *s, GSM_File *File, bool star
 	GSM_Phone_N6510Data     *Priv = &s->Phone.Data.Priv.N6510;
 	char		    	buf[200];
 
-	if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILESYSTEM)) return ERR_NOTSUPPORTED;
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILESYSTEM)) return ERR_NOTSUPPORTED;
 
 	if (start) {
 		Priv->Use2 = true;
-		if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_SERIES40_30)) {
+		if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_SERIES40_30)) {
 			//series 40 3.0 don't have filesystem 1
 			Priv->Use2 = false;
 		}
-		if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILE1)) {
+		if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILE1)) {
 			Priv->Use2 = false;
 		}
 	}
 	if (Priv->Use2) {
 		error = N6510_GetNextFileFolder1(s,File,start);
 		if (error == ERR_EMPTY) {
-			if (!IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_FILES2)) {
+			if (!GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_FILES2)) {
 				return error;
 			}
 			Priv->Use2      = false;
@@ -1602,12 +1601,12 @@ GSM_Error N6510_GetFilePart(GSM_StateMachine *s, GSM_File *File, int *Handle, in
 	char	    	buf[200];
 	GSM_Error       error;
 
-	if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILESYSTEM)) return ERR_NOTSUPPORTED;
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILESYSTEM)) return ERR_NOTSUPPORTED;
 
 	if (DecodeUnicodeString(File->ID_FullName)[0] == 'c' ||
 	    DecodeUnicodeString(File->ID_FullName)[0] == 'C') {
-		if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_SERIES40_30) ||
-		    IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILE1)) {
+		if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_SERIES40_30) ||
+		    GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILE1)) {
 			return ERR_NOTSUPPORTED;
 		}
 		memcpy(&File2,File,sizeof(GSM_File));
@@ -1618,7 +1617,7 @@ GSM_Error N6510_GetFilePart(GSM_StateMachine *s, GSM_File *File, int *Handle, in
 		CopyUnicodeString(File->ID_FullName,buf);
 		return error;
 	} else {
-		if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_FILES2)) {
+		if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_FILES2)) {
 			return N6510_GetFilePart2(s,File, Handle, Size);
 		} else {
 			return ERR_NOTSUPPORTED;
@@ -1631,12 +1630,12 @@ GSM_Error N6510_AddFilePart(GSM_StateMachine *s, GSM_File *File, int *Pos, int *
 	GSM_File	File2;
 	GSM_Error       error;
 
-	if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILESYSTEM)) return ERR_NOTSUPPORTED;
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILESYSTEM)) return ERR_NOTSUPPORTED;
 
 	if (DecodeUnicodeString(File->ID_FullName)[0] == 'c' ||
 	    DecodeUnicodeString(File->ID_FullName)[0] == 'C') {
-		if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_SERIES40_30) ||
-		    IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILE1)) {
+		if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_SERIES40_30) ||
+		    GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILE1)) {
 			return ERR_NOTSUPPORTED;
 		}
 
@@ -1649,7 +1648,7 @@ GSM_Error N6510_AddFilePart(GSM_StateMachine *s, GSM_File *File, int *Pos, int *
 		CopyUnicodeString(File->ID_FullName+UnicodeLength(File->ID_FullName)*2,File2.ID_FullName);
 		return error;
 	} else {
-		if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_FILES2)) {
+		if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_FILES2)) {
 			return N6510_AddFilePart2(s,File,Pos,Handle);
 		} else {
 			return ERR_NOTSUPPORTED;
@@ -1659,18 +1658,18 @@ GSM_Error N6510_AddFilePart(GSM_StateMachine *s, GSM_File *File, int *Pos, int *
 
 GSM_Error N6510_DeleteFile(GSM_StateMachine *s, unsigned char *ID)
 {
-	if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILESYSTEM)) return ERR_NOTSUPPORTED;
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILESYSTEM)) return ERR_NOTSUPPORTED;
 
 	if (DecodeUnicodeString(ID)[0] == 'c' ||
 	    DecodeUnicodeString(ID)[0] == 'C') {
-		if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_SERIES40_30) ||
-		    IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILE1)) {
+		if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_SERIES40_30) ||
+		    GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILE1)) {
 			return ERR_NOTSUPPORTED;
 		}
 
 		return N6510_DeleteFile1(s,ID+6);
 	} else {
-		if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_FILES2)) {
+		if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_FILES2)) {
 			return N6510_DeleteFile2(s,ID);
 		} else {
 			return ERR_NOTSUPPORTED;
@@ -1683,12 +1682,12 @@ GSM_Error N6510_AddFolder(GSM_StateMachine *s, GSM_File *File)
 	GSM_File	File2;
 	GSM_Error       error;
 
-	if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILESYSTEM)) return ERR_NOTSUPPORTED;
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILESYSTEM)) return ERR_NOTSUPPORTED;
 
 	if (DecodeUnicodeString(File->ID_FullName)[0] == 'c' ||
 	    DecodeUnicodeString(File->ID_FullName)[0] == 'C') {
-		if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_SERIES40_30) ||
-		    IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILE1)) {
+		if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_SERIES40_30) ||
+		    GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILE1)) {
 			return ERR_NOTSUPPORTED;
 		}
 		memcpy(&File2,File,sizeof(GSM_File));
@@ -1700,7 +1699,7 @@ GSM_Error N6510_AddFolder(GSM_StateMachine *s, GSM_File *File)
 		CopyUnicodeString(File->ID_FullName+UnicodeLength(File->ID_FullName)*2,File2.ID_FullName);
 		return error;
 	} else {
-		if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_FILES2)) {
+		if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_FILES2)) {
 			return N6510_AddFolder2(s,File);
 		} else {
 			return ERR_NOTSUPPORTED;
@@ -1710,17 +1709,17 @@ GSM_Error N6510_AddFolder(GSM_StateMachine *s, GSM_File *File)
 
 GSM_Error N6510_DeleteFolder(GSM_StateMachine *s, unsigned char *ID)
 {
-	if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILESYSTEM)) return ERR_NOTSUPPORTED;
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILESYSTEM)) return ERR_NOTSUPPORTED;
 
 	if (DecodeUnicodeString(ID)[0] == 'c' ||
 	    DecodeUnicodeString(ID)[0] == 'C') {
-		if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_SERIES40_30) ||
-		    IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILE1)) {
+		if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_SERIES40_30) ||
+		    GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILE1)) {
 			return ERR_NOTSUPPORTED;
 		}
 		return N6510_DeleteFolder1(s,ID+6);
 	} else {
-		if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_FILES2)) {
+		if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_FILES2)) {
 			return N6510_DeleteFolder2(s,ID);
 		} else {
 			return ERR_NOTSUPPORTED;
@@ -1730,13 +1729,13 @@ GSM_Error N6510_DeleteFolder(GSM_StateMachine *s, unsigned char *ID)
 
 GSM_Error N6510_GetFileSystemStatus(GSM_StateMachine *s, GSM_FileSystemStatus *status)
 {
-	if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILESYSTEM)) return ERR_NOTSUPPORTED;
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILESYSTEM)) return ERR_NOTSUPPORTED;
 
-	if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_FILES2)) {
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_FILES2)) {
 		return ERR_NOTSUPPORTED;
 	} else {
-		if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_SERIES40_30) ||
-		    IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILE1)) {
+		if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_SERIES40_30) ||
+		    GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILE1)) {
 			return ERR_NOTSUPPORTED;
 		}
 		return N6510_GetFileSystemStatus1(s,status);
@@ -1749,12 +1748,12 @@ GSM_Error N6510_SetFileAttributes(GSM_StateMachine *s, GSM_File *File)
 	char	    	buf[200];
 	GSM_Error       error;
 
-	if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILESYSTEM)) return ERR_NOTSUPPORTED;
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILESYSTEM)) return ERR_NOTSUPPORTED;
 
 	if (DecodeUnicodeString(File->ID_FullName)[0] == 'c' ||
 	    DecodeUnicodeString(File->ID_FullName)[0] == 'C') {
-		if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_SERIES40_30) ||
-		    IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILE1)) {
+		if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_SERIES40_30) ||
+		    GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILE1)) {
 			return ERR_NOTSUPPORTED;
 		}
 		memcpy(&File2,File,sizeof(GSM_File));
@@ -1765,7 +1764,7 @@ GSM_Error N6510_SetFileAttributes(GSM_StateMachine *s, GSM_File *File)
 		CopyUnicodeString(File->ID_FullName,buf);
 		return error;
 	} else {
-		if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_FILES2)) {
+		if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_FILES2)) {
 			return N6510_SetFileAttributes2(s,File);
 		} else {
 			return ERR_NOTSUPPORTED;
@@ -1779,12 +1778,12 @@ GSM_Error N6510_GetNextRootFolder(GSM_StateMachine *s, GSM_File *File)
 	GSM_File  		File2;
 	unsigned char		buffer[5];
 
-	if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILESYSTEM)) return ERR_NOTSUPPORTED;
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILESYSTEM)) return ERR_NOTSUPPORTED;
 
 	memset(&File2, 0, sizeof(File2));
 
-	if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_SERIES40_30) ||
-	    IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILE1)) {
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_SERIES40_30) ||
+	    GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILE1)) {
 		if (UnicodeLength(File->ID_FullName) == 0) {
 			EncodeUnicode(File->ID_FullName,"d:",2);
 			EncodeUnicode(File->Name,"D (Permanent_memory 2)",22);
@@ -1809,7 +1808,7 @@ GSM_Error N6510_GetNextRootFolder(GSM_StateMachine *s, GSM_File *File)
 		if (error != ERR_NONE) return error;
 	}
 
-	if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_FILES2)) {
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_FILES2)) {
 		if (UnicodeLength(File->ID_FullName) == 0) {
 			memcpy(File,&File2,sizeof(GSM_File));
 			EncodeUnicode(File->Name,"C (",3);
@@ -1956,7 +1955,7 @@ GSM_Error N6510_GetMMSFolders(GSM_StateMachine *s, GSM_MMSFolders *folders)
 	int			i;
 
 
-	if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILESYSTEM)) return ERR_NOTSUPPORTED;
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NOFILESYSTEM)) return ERR_NOTSUPPORTED;
 
 	memset(&Files, 0, sizeof(Files));
 
@@ -1965,11 +1964,11 @@ GSM_Error N6510_GetMMSFolders(GSM_StateMachine *s, GSM_MMSFolders *folders)
 		Priv->MMSFoldersID2[i][1] = 0;
 	}
 
-	if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_3220_MMS)) {
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_3220_MMS)) {
 		return N6510_PrivGet3220FilesystemMMSFolders(s,folders);
 	}
 
-	if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_SERIES40_30)) {
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_SERIES40_30)) {
 		return N6510_PrivGetFilesystemMMSFolders(s,folders);
 	}
 
@@ -2002,7 +2001,7 @@ GSM_Error N6510_GetMMSFolders(GSM_StateMachine *s, GSM_MMSFolders *folders)
 	}
 
 	//6230i
-	if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_FILES2)) {
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_FILES2)) {
 		EncodeUnicode(Files.ID_FullName,"d:/predefmessages",17);
 		folders->Number = 0;
 		Start 		= true;
@@ -2073,7 +2072,7 @@ GSM_Error N6510_GetNextMMSFileInfo(GSM_StateMachine *s, unsigned char *FileID, i
 		(*MMSFolder) = Priv->MMSFolderNum;
 		CopyUnicodeString(FileID,Priv->MMSFile.ID_FullName);
 
-		if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_SERIES40_30)) {
+		if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_SERIES40_30)) {
 			CopyUnicodeString(file.ID_FullName,FileID);
 			file.Used   = 0;
 			file.Buffer = NULL;
@@ -2133,6 +2132,10 @@ GSM_Error N6510_PrivGetFilesystemSMSFolders(GSM_StateMachine *s, GSM_SMSFolders
 		if (!strcmp(DecodeUnicodeString(Files.Name),"1")) {
 			folders->Folder[folders->Number].InboxFolder = true;
 		}
+		folders->Folder[folders->Number].OutboxFolder = false;
+		if (!strcmp(DecodeUnicodeString(Files.Name),"6")) {
+			folders->Folder[folders->Number].OutboxFolder = true;
+		}
 		if (real) {
 			CopyUnicodeString(folders->Folder[folders->Number].Name,Files.Name);
 		} else {
@@ -2168,9 +2171,9 @@ static void N26510_GetSMSLocation(GSM_StateMachine *s, GSM_SMSMessage *sms, unsi
 
 	/* simulate flat SMS memory */
 	if (sms->Folder==0x00) {
-		ifolderid = sms->Location / PHONE_MAXSMSINFOLDER;
+		ifolderid = sms->Location / GSM_PHONE_MAXSMSINFOLDER;
 		*folderid = ifolderid + 0x01;
-		*location = sms->Location - ifolderid * PHONE_MAXSMSINFOLDER;
+		*location = sms->Location - ifolderid * GSM_PHONE_MAXSMSINFOLDER;
 	} else {
 		*folderid = sms->Folder;
 		*location = sms->Location;
@@ -2183,7 +2186,7 @@ static void N26510_GetSMSLocation(GSM_StateMachine *s, GSM_SMSMessage *sms, unsi
 static void N26510_SetSMSLocation(GSM_StateMachine *s, GSM_SMSMessage *sms, unsigned char folderid, int location)
 {
 	sms->Folder	= 0;
-	sms->Location	= (folderid - 0x01) * PHONE_MAXSMSINFOLDER + location;
+	sms->Location	= (folderid - 0x01) * GSM_PHONE_MAXSMSINFOLDER + location;
 	smprintf(s, "6510 folder %i & location %i -> SMS folder %i & location %i\n",
 		folderid,location,sms->Folder,sms->Location);
 }
@@ -2245,13 +2248,15 @@ GSM_Error N6510_GetNextFilesystemSMS(GSM_StateMachine *s, GSM_MultiSMSMessage *s
 				break;
 			}
 		}
-		DumpMessage(&s->di, FFF.Buffer, FFF.Used);
+		if (FFF.Buffer != NULL)  {
+			DumpMessage(&s->di, FFF.Buffer, FFF.Used);
 
-		//0x00 = SMS, 0x01,0x03 = MMS
-		if (FFF.Buffer[6] == 0x00) break;
+			//0x00 = SMS, 0x01,0x03 = MMS
+			if (FFF.Buffer[6] == 0x00) break;
 
-		smprintf(s,"mms file");
-		free(FFF.Buffer);
+			smprintf(s,"mms file");
+			free(FFF.Buffer);
+		}
 	}
 
 	//decode file
@@ -2381,8 +2386,11 @@ GSM_Error N6510_GetNextFilesystemSMS(GSM_StateMachine *s, GSM_MultiSMSMessage *s
 		CopyUnicodeString(sms->SMS[0].Name,FFF.Buffer+i);
 		if (sms->SMS[0].Number[0]==0x00 && sms->SMS[0].Number[1]==0x00) {
 			i+=UnicodeLength(sms->SMS[0].Name)*2+5;
-			while (true) {
-				if (FFF.Buffer[i+1]==0x00) break;
+			while (FFF.Buffer[i+1] != 0x00) {
+				if (sms->SMS[0].OtherNumbersNum == GSM_SMS_OTHER_NUMBERS - 1) {
+					smprintf(s, "WARNING: Too much recipient numbers, ignoring rest!\n");
+					break;
+				}
 				smprintf(s,"recipient number %02x %02x",FFF.Buffer[i],FFF.Buffer[i+1]);
 				if (sms->SMS[0].Number[0]==0x00 && sms->SMS[0].Number[1]==0x00) {
 					CopyUnicodeString(sms->SMS[0].Number,FFF.Buffer+i);
@@ -2440,9 +2448,12 @@ GSM_Error N6510_GetNextFilesystemSMS(GSM_StateMachine *s, GSM_MultiSMSMessage *s
 			if (FFF.Buffer[176]==0x51) {
 				CopyUnicodeString(sms->SMS[0].Name,FFF.Buffer+i);
 				i+=UnicodeLength(sms->SMS[0].Name)*2+5;
-				while (true) {
-					if (FFF.Buffer[i+1]==0x00) break;
-					smprintf(s,"recipient number %02x %02x",FFF.Buffer[i],FFF.Buffer[i+1]);
+				while (FFF.Buffer[i+1] != 0x00) {
+					if (sms->SMS[0].OtherNumbersNum == GSM_SMS_OTHER_NUMBERS - 1) {
+						smprintf(s, "WARNING: Too much recipient numbers, ignoring rest!\n");
+						break;
+					}
+					smprintf(s,"recipient number 0x%02x 0x%02x ", FFF.Buffer[i],FFF.Buffer[i+1]);
 					if (sms->SMS[0].Number[0]==0x00 && sms->SMS[0].Number[1]==0x00) {
 						CopyUnicodeString(sms->SMS[0].Number,FFF.Buffer+i);
 						smprintf(s,"%s\n",DecodeUnicodeString(sms->SMS[0].Number));
@@ -2491,8 +2502,10 @@ GSM_Error N6510_GetNextFilesystemSMS(GSM_StateMachine *s, GSM_MultiSMSMessage *s
 		sms->Number = 1;
 		break;
 	default:
+		/* Fail if we don't know message type */
 		smprintf(s,"unknown sms file %02x\n",FFF.Buffer[i]);
-		break;
+		free(FFF.Buffer);
+		return ERR_UNKNOWN;
 	}
 
 	sms->SMS[0].Memory = MEM_ME;
