@@ -1,6 +1,6 @@
 /* (c) 2004 by Marcin Wiacek */
 
-#include "../../../gsmstate.h"
+#include <gammu-config.h>
 
 #ifdef GSM_ENABLE_NOKIA3320
 
@@ -206,6 +206,7 @@ GSM_Phone_Functions N3320Phone = {
  	NOTSUPPORTED,			/* 	AddSMSFolder		*/
  	NOTSUPPORTED,			/* 	DeleteSMSFolder		*/
 	NOTIMPLEMENTED,			/*	DialVoice		*/
+        NOTIMPLEMENTED,			/*	DialService		*/
 	NOTIMPLEMENTED,			/*	AnswerCall		*/
 	NOTIMPLEMENTED,			/*	CancelCall		*/
  	NOTIMPLEMENTED,			/* 	HoldCall 		*/
@@ -274,6 +275,7 @@ GSM_Phone_Functions N3320Phone = {
 	NOTSUPPORTED,			/*	SetFileAttributes	*/
 	NOTSUPPORTED,			/*	GetFilePart		*/
 	NOTIMPLEMENTED,			/*	AddFilePart		*/
+	NOTSUPPORTED,			/* 	SendFilePart		*/
 	NOTSUPPORTED,	 		/* 	GetFileSystemStatus	*/
 	NOTIMPLEMENTED,			/*	DeleteFile		*/
 	NOTIMPLEMENTED,			/*	AddFolder		*/
