@@ -6,9 +6,12 @@
 #include <ctype.h>
 #include <time.h>
 
+#include <gammu-nokia.h>
+
 #include "../../gsmstate.h"
+#include "../../gsmphones.h"
 #include "../../misc/coding/coding.h"
-#include "../../service/sms/gsmsms.h"
+#include "../../misc/locales.h"
 #include "../pfunc.h"
 #include "nfunc.h"
 
@@ -45,7 +48,7 @@ int N71_65_PackPBKBlock(GSM_StateMachine *s, int id, int size, int no, unsigned
 int N71_65_EncodePhonebookFrame(GSM_StateMachine *s, unsigned char *req, GSM_MemoryEntry *entry, int *block2, bool DCT4, bool VoiceTag)
 {
 	int		count=0, len, i, block=0, j;
-	char		string[500];
+	unsigned char	string[500];
 	unsigned char	type;
 	bool		found;
 
@@ -54,7 +57,7 @@ int N71_65_EncodePhonebookFrame(GSM_StateMachine *s, unsigned char *req, GSM_Mem
 	}
 	memset(string,0,sizeof(string));
 	found = false;
-	if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_SERIES40_30)) {
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_SERIES40_30)) {
  		for (i = 0; i < entry->EntriesNum; i++) {
 			if (entry->Entries[i].EntryType == PBK_Text_LastName ||
 			    entry->Entries[i].EntryType == PBK_Text_FirstName) {
@@ -92,6 +95,9 @@ int N71_65_EncodePhonebookFrame(GSM_StateMachine *s, unsigned char *req, GSM_Mem
  		for (i = 0; i < entry->EntriesNum; i++) {
 			if (entry->Entries[i].EntryType == PBK_Text_LastName ||
 			    entry->Entries[i].EntryType == PBK_Text_FirstName) {
+				if (UnicodeLength(string+1) > 0) {
+					string[UnicodeLength(string+1)*2+1] = ' ';
+				}
 				CopyUnicodeString(string+UnicodeLength(string+1)*2+1,entry->Entries[i].Text);
 				entry->Entries[i].AddError = ERR_DATACONVERTED;
 				found=true;
@@ -194,9 +200,9 @@ int N71_65_EncodePhonebookFrame(GSM_StateMachine *s, unsigned char *req, GSM_Mem
 			continue;
 		}
 		if (entry->Entries[i].EntryType == PBK_Caller_Group) {
-			if (!IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_PBK35)) {
+			if (!GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_PBK35)) {
 				entry->Entries[i].AddError = ERR_NONE;
-				if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_6230iCALLER)) {
+				if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_6230iCALLER)) {
 					string[0] = 0;
 					string[1] = 0;
 					count += N71_65_PackPBKBlock(s, N6510_PBK_GROUP2_ID, 2, block++, string, req + count);
@@ -210,7 +216,7 @@ int N71_65_EncodePhonebookFrame(GSM_StateMachine *s, unsigned char *req, GSM_Mem
 			continue;
 		}
 		if (entry->Entries[i].EntryType == PBK_RingtoneID) {
-			if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_PBK35)) {
+			if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_PBK35)) {
 				entry->Entries[i].AddError = ERR_NONE;
 				string[0] = 0x00;
 				string[1] = 0x00;
@@ -222,7 +228,7 @@ int N71_65_EncodePhonebookFrame(GSM_StateMachine *s, unsigned char *req, GSM_Mem
 			continue;
 		}
 		if (entry->Entries[i].EntryType == PBK_PictureID) {
-			if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_PBKIMG)) {
+			if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_PBKIMG)) {
 				entry->Entries[i].AddError = ERR_NONE;
 				string[0] = 0x00;
 				string[1] = 0x00;
@@ -239,8 +245,32 @@ int N71_65_EncodePhonebookFrame(GSM_StateMachine *s, unsigned char *req, GSM_Mem
 			}
 			continue;
 		}
+		/* Maybe we should use separate feature for these new entries... */
+		if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_SERIES40_30)) {
+			type = 0;
+			if (entry->Entries[i].EntryType == PBK_Text_FormalName) type = S4030_PBK_FORMALNAME;
+			if (entry->Entries[i].EntryType == PBK_Text_JobTitle) type = S4030_PBK_JOBTITLE;
+			if (entry->Entries[i].EntryType == PBK_Text_Company) type = S4030_PBK_COMPANY;
+			if (entry->Entries[i].EntryType == PBK_Text_NickName) type = S4030_PBK_NICKNAME;
+			if (type != 0) {
+				if (entry->Entries[i].AddError==ERR_NOTSUPPORTED) entry->Entries[i].AddError = ERR_NONE;
+				len = UnicodeLength(entry->Entries[i].Text);
+				string[0] = len*2+2;
+				CopyUnicodeString(string+1,entry->Entries[i].Text);
+				string[len*2+1] = 0;
+				count += N71_65_PackPBKBlock(s, type, len * 2 + 2, block++, string, req + count);
+				continue;
+			}
+			if (entry->Entries[i].EntryType == PBK_Date) {
+				entry->Entries[i].AddError = ERR_NONE;
+
+				NOKIA_EncodeDateTime(s, string + 1, &(entry->Entries[i].Date));
+				count += N71_65_PackPBKBlock(s, S4030_PBK_BIRTHDAY, 6, block++, string, req + count);
+				continue;
+			}
+		}
 		if (entry->Entries[i].EntryType == PBK_Text_UserID) {
-			if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_PBKUSER)) {
+			if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_PBKUSER)) {
 				entry->Entries[i].AddError = ERR_NONE;
 				string[0] = UnicodeLength(entry->Entries[i].Text)*2;
 				CopyUnicodeString(string+1,entry->Entries[i].Text);
@@ -249,6 +279,14 @@ int N71_65_EncodePhonebookFrame(GSM_StateMachine *s, unsigned char *req, GSM_Mem
 			}
 			continue;
 		}
+		if (entry->Entries[i].EntryType == PBK_PushToTalkID) {
+			entry->Entries[i].AddError = ERR_NONE;
+			string[0] = UnicodeLength(entry->Entries[i].Text)*2;
+			CopyUnicodeString(string+1,entry->Entries[i].Text);
+			count += N71_65_PackPBKBlock(s, N6510_PBK_PUSHTOTALK_ID, string[0]+2, block++, string, req+count);
+			req[count-1]--;
+			continue;
+		}
 	}
 
 	*block2=block;
@@ -268,6 +306,7 @@ GSM_Error N71_65_DecodePhonebook(GSM_StateMachine	*s,
 	int					length = 0, i, bs = 0;
 	GSM_71_65_Phonebook_Entries_Types	Type;
 	bool					found=false;
+	bool					foundbb5add=false;
 
 	entry->EntriesNum 	= 0;
 
@@ -278,7 +317,7 @@ GSM_Error N71_65_DecodePhonebook(GSM_StateMachine	*s,
 		bitmap->DefaultRingtone 	= true;
 		bitmap->FileSystemPicture 	= false;
 	}
-	if (entry->MemoryType==MEM6510_CG2 && IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_6230iCALLER)) {
+	if (entry->MemoryType==MEM6510_CG2 && GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_6230iCALLER)) {
 		bitmap->DefaultName 		= false;
 		bitmap->DefaultBitmap 		= true;
 		bitmap->DefaultRingtone 	= true;
@@ -297,7 +336,7 @@ GSM_Error N71_65_DecodePhonebook(GSM_StateMachine	*s,
 		bs = 256*Block[2]+Block[3];
 #ifdef DEBUG
 		smprintf(s, "Phonebook entry block - length %i", bs-6);
-		if (s->di.dl == DL_TEXTALL || s->di.dl == DL_TEXTALLDATE) DumpMessage(&s->di, Block+5, bs-6);
+		if (s->di.dl == DL_TEXTALL || s->di.dl == DL_TEXTALLDATE) DumpMessage(&s->di, Block+0, bs-1);
 #endif
 		if (entry->EntriesNum==GSM_PHONEBOOK_ENTRIES) {
 			smprintf(s, "Too many entries\n");
@@ -338,7 +377,7 @@ GSM_Error N71_65_DecodePhonebook(GSM_StateMachine	*s,
 		bs = 256*Block[2]+Block[3];
 #ifdef DEBUG
 		smprintf(s, "Phonebook entry block - length %i", bs-6);
-		if (s->di.dl == DL_TEXTALL || s->di.dl == DL_TEXTALLDATE) DumpMessage(&s->di, Block+5, bs-6);
+		if (s->di.dl == DL_TEXTALL || s->di.dl == DL_TEXTALLDATE) DumpMessage(&s->di, Block+0, bs-1);
 #endif
 		if (entry->EntriesNum==GSM_PHONEBOOK_ENTRIES) {
 			smprintf(s, "Too many entries\n");
@@ -471,7 +510,7 @@ GSM_Error N71_65_DecodePhonebook(GSM_StateMachine	*s,
 			entry->Entries[entry->EntriesNum].CallLength = Block[9]*256*256+Block[10]*256+Block[11];
 			entry->Entries[entry->EntriesNum].EntryType=PBK_CallLength;
 			entry->EntriesNum ++;
-			continue;			
+			continue;
 		}
 		if (Block[0] == S4030_PBK_POSTAL) {
 			if (Block[5] == S4030_PBK_POSTAL_EXTADDRESS) {
@@ -492,6 +531,11 @@ GSM_Error N71_65_DecodePhonebook(GSM_StateMachine	*s,
 			if (Block[5] == S4030_PBK_POSTAL_COUNTRY) {
 				Type = PBK_Text_Country;    	smprintf(s,"Country ");
 			}
+			if ((Type == 0x00) && (Block[7]>0)) {
+				smprintf(s, "Found new bb5 style address\n");
+				foundbb5add=true;
+				continue;
+			}
 			if (Type == 0x00) {
 				smprintf(s, "Unknown address type %02x\n",Block[5]);
 				return ERR_UNKNOWNRESPONSE;
@@ -506,6 +550,147 @@ GSM_Error N71_65_DecodePhonebook(GSM_StateMachine	*s,
 			entry->EntriesNum ++;
 			continue;
 		}
+
+
+		if ((Block[0] == S4030_PBK_POSTAL_EXTADDRESS) && (foundbb5add==true)) {
+			Type = PBK_Text_Custom1;    	smprintf(s,"Address extension ? ");
+			if (Block[5]/2>GSM_PHONEBOOK_TEXT_LENGTH) {
+				smprintf(s, "Too long text\n");
+				return ERR_UNKNOWNRESPONSE;
+			}
+			memcpy(entry->Entries[entry->EntriesNum].Text,Block+6,Block[5]);
+			entry->Entries[entry->EntriesNum].EntryType=Type;
+			smprintf(s, " \"%s\"\n",DecodeUnicodeString(entry->Entries[entry->EntriesNum].Text));
+			entry->EntriesNum ++;
+			continue;
+		}
+		if ((Block[0] == S4030_PBK_POSTAL_STREET) && (foundbb5add==true)) {
+			Type = PBK_Text_StreetAddress;  smprintf(s,"Street ");
+			if (Block[5]/2>GSM_PHONEBOOK_TEXT_LENGTH) {
+				smprintf(s, "Too long text\n");
+				return ERR_UNKNOWNRESPONSE;
+			}
+			memcpy(entry->Entries[entry->EntriesNum].Text,Block+6,Block[5]);
+			entry->Entries[entry->EntriesNum].EntryType=Type;
+			smprintf(s, " \"%s\"\n",DecodeUnicodeString(entry->Entries[entry->EntriesNum].Text));
+			entry->EntriesNum ++;
+			continue;
+		}
+		if ((Block[0] == S4030_PBK_POSTAL_CITY) && (foundbb5add==true)) {
+			Type = PBK_Text_City;    	smprintf(s,"City ");
+			if (Block[5]/2>GSM_PHONEBOOK_TEXT_LENGTH) {
+				smprintf(s, "Too long text\n");
+				return ERR_UNKNOWNRESPONSE;
+			}
+			memcpy(entry->Entries[entry->EntriesNum].Text,Block+6,Block[5]);
+			entry->Entries[entry->EntriesNum].EntryType=Type;
+			smprintf(s, " \"%s\"\n",DecodeUnicodeString(entry->Entries[entry->EntriesNum].Text));
+			entry->EntriesNum ++;
+			continue;
+		}
+		if ((Block[0] == S4030_PBK_POSTAL_STATE) && (foundbb5add==true)) {
+			Type = PBK_Text_State;    	smprintf(s,"State ");
+			if (Block[5]/2>GSM_PHONEBOOK_TEXT_LENGTH) {
+				smprintf(s, "Too long text\n");
+				return ERR_UNKNOWNRESPONSE;
+			}
+			memcpy(entry->Entries[entry->EntriesNum].Text,Block+6,Block[5]);
+			entry->Entries[entry->EntriesNum].EntryType=Type;
+			smprintf(s, " \"%s\"\n",DecodeUnicodeString(entry->Entries[entry->EntriesNum].Text));
+			entry->EntriesNum ++;
+			continue;
+		}
+		if ((Block[0] == S4030_PBK_POSTAL_POSTAL) && (foundbb5add==true)) {
+			Type = PBK_Text_Postal;    	smprintf(s,"Postal ");
+			if (Block[5]/2>GSM_PHONEBOOK_TEXT_LENGTH) {
+				smprintf(s, "Too long text\n");
+				return ERR_UNKNOWNRESPONSE;
+			}
+			memcpy(entry->Entries[entry->EntriesNum].Text,Block+6,Block[5]);
+			entry->Entries[entry->EntriesNum].EntryType=Type;
+			smprintf(s, " \"%s\"\n",DecodeUnicodeString(entry->Entries[entry->EntriesNum].Text));
+			entry->EntriesNum ++;
+			continue;
+		}
+		if ((Block[0] == S4030_PBK_POSTAL_COUNTRY) && (foundbb5add==true)) {
+			Type = PBK_Text_Country;    	smprintf(s,"Country ");
+			if (Block[5]/2>GSM_PHONEBOOK_TEXT_LENGTH) {
+				smprintf(s, "Too long text\n");
+				return ERR_UNKNOWNRESPONSE;
+			}
+			memcpy(entry->Entries[entry->EntriesNum].Text,Block+6,Block[5]);
+			entry->Entries[entry->EntriesNum].EntryType=Type;
+			smprintf(s, " \"%s\"\n",DecodeUnicodeString(entry->Entries[entry->EntriesNum].Text));
+			entry->EntriesNum ++;
+			continue;
+		}
+		if (Block[0] == S4030_PBK_FORMALNAME) {
+			Type = PBK_Text_FormalName;    	smprintf(s,"FormalName ");
+			if (Block[5]/2>GSM_PHONEBOOK_TEXT_LENGTH) {
+				smprintf(s, "Too long text\n");
+				return ERR_UNKNOWNRESPONSE;
+			}
+			memcpy(entry->Entries[entry->EntriesNum].Text,Block+6,Block[5]);
+			entry->Entries[entry->EntriesNum].EntryType=Type;
+			smprintf(s, " \"%s\"\n",DecodeUnicodeString(entry->Entries[entry->EntriesNum].Text));
+			entry->EntriesNum ++;
+			continue;
+		}
+		if (Block[0] == S4030_PBK_JOBTITLE) {
+			Type = PBK_Text_JobTitle;    	smprintf(s,"JobTitle ");
+			if (Block[5]/2>GSM_PHONEBOOK_TEXT_LENGTH) {
+				smprintf(s, "Too long text\n");
+				return ERR_UNKNOWNRESPONSE;
+			}
+			memcpy(entry->Entries[entry->EntriesNum].Text,Block+6,Block[5]);
+			entry->Entries[entry->EntriesNum].EntryType=Type;
+			smprintf(s, " \"%s\"\n",DecodeUnicodeString(entry->Entries[entry->EntriesNum].Text));
+			entry->EntriesNum ++;
+			continue;
+		}
+		if (Block[0] == S4030_PBK_COMPANY) {
+			Type = PBK_Text_Company;    	smprintf(s,"Company ");
+			if (Block[5]/2>GSM_PHONEBOOK_TEXT_LENGTH) {
+				smprintf(s, "Too long text\n");
+				return ERR_UNKNOWNRESPONSE;
+			}
+			memcpy(entry->Entries[entry->EntriesNum].Text,Block+6,Block[5]);
+			entry->Entries[entry->EntriesNum].EntryType=Type;
+			smprintf(s, " \"%s\"\n",DecodeUnicodeString(entry->Entries[entry->EntriesNum].Text));
+			entry->EntriesNum ++;
+			continue;
+		}
+		if (Block[0] == S4030_PBK_NICKNAME) {
+			Type = PBK_Text_NickName;    	smprintf(s,"NickName ");
+			if (Block[5]/2>GSM_PHONEBOOK_TEXT_LENGTH) {
+				smprintf(s, "Too long text\n");
+				return ERR_UNKNOWNRESPONSE;
+			}
+			memcpy(entry->Entries[entry->EntriesNum].Text,Block+6,Block[5]);
+			entry->Entries[entry->EntriesNum].EntryType=Type;
+			smprintf(s, " \"%s\"\n",DecodeUnicodeString(entry->Entries[entry->EntriesNum].Text));
+			entry->EntriesNum ++;
+			continue;
+		}
+		if (Block[0] == S4030_PBK_BIRTHDAY) {
+			entry->Entries[entry->EntriesNum].EntryType=PBK_Date;
+			NOKIA_DecodeDateTime(s, Block+6, &entry->Entries[entry->EntriesNum].Date);
+			// We don't have seconds available here
+			entry->Entries[entry->EntriesNum].Date.Second = 0;
+			//some phones reverse it
+			if (entry->Entries[entry->EntriesNum].Date.Year > 3000) {
+				entry->Entries[entry->EntriesNum].Date.Year = Block[7]*256+Block[6];
+			}
+			if (DayMonthReverse) {
+				i = entry->Entries[entry->EntriesNum].Date.Month;
+				entry->Entries[entry->EntriesNum].Date.Month = entry->Entries[entry->EntriesNum].Date.Day;
+				entry->Entries[entry->EntriesNum].Date.Day   = i;
+			}
+			entry->EntriesNum ++;
+			continue;
+		}
+
+
 		if (Block[0] == N7110_PBK_RINGTONE_ID) {
 			if (entry->MemoryType==MEM7110_CG) {
 				bitmap->RingtoneID=Block[5];
@@ -549,7 +734,7 @@ GSM_Error N71_65_DecodePhonebook(GSM_StateMachine	*s,
 		}
 		if (Block[0] == N6510_PBK_VOICETAG_ID) {
 			smprintf(s, "Entry %i has voice tag %i\n",Block[5]-1,Block[7]);
-			entry->Entries[Block[5]-1].VoiceTag = Block[7];
+			entry->Entries[entry->EntriesNum].VoiceTag = Block[7];
 			continue;
 		}
 
@@ -644,7 +829,15 @@ GSM_Error N71_65_DecodePhonebook(GSM_StateMachine	*s,
 			continue;
 		}
 		if (Block[0] == N6510_PBK_PUSHTOTALK_ID) {
-			smprintf(s,"SIP Address (Push to Talk address) - ignored\n");
+			smprintf(s,"SIP Address (Push to Talk address)\n");
+			if (Block[5]/2>GSM_PHONEBOOK_TEXT_LENGTH) {
+				smprintf(s, "Too long text\n");
+				return ERR_UNKNOWNRESPONSE;
+			}
+			memcpy(entry->Entries[entry->EntriesNum].Text,Block+6,Block[5]);
+			entry->Entries[entry->EntriesNum].EntryType=PBK_PushToTalkID;
+			smprintf(s, " \"%s\"\n",DecodeUnicodeString(entry->Entries[entry->EntriesNum].Text));
+			entry->EntriesNum ++;
 			continue;
 		}
 		if (Block[0] == N6510_PBK_GROUP2_ID) {
@@ -665,21 +858,21 @@ GSM_Error N71_65_DecodePhonebook(GSM_StateMachine	*s,
 	return ERR_NONE;
 }
 
-void NOKIA_GetDefaultCallerGroupName(GSM_StateMachine *s, GSM_Bitmap *Bitmap)
+void NOKIA_GetDefaultCallerGroupName(GSM_Bitmap *Bitmap)
 {
 	Bitmap->DefaultName = false;
 	if (Bitmap->Text[0]==0x00 && Bitmap->Text[1]==0x00) {
 		Bitmap->DefaultName = true;
 		switch(Bitmap->Location) {
-		case 1: EncodeUnicode(Bitmap->Text,GetMsg(s->msg,"Family"),strlen(GetMsg(s->msg,"Family")));
+		case 1: EncodeUnicode(Bitmap->Text,_("Family"),strlen(_("Family")));
 			break;
-		case 2: EncodeUnicode(Bitmap->Text,GetMsg(s->msg,"VIP"),strlen(GetMsg(s->msg,"VIP")));
+		case 2: EncodeUnicode(Bitmap->Text,_("VIP"),strlen(_("VIP")));
 			break;
-		case 3: EncodeUnicode(Bitmap->Text,GetMsg(s->msg,"Friends"),strlen(GetMsg(s->msg,"Friends")));
+		case 3: EncodeUnicode(Bitmap->Text,_("Friends"),strlen(_("Friends")));
 			break;
-		case 4: EncodeUnicode(Bitmap->Text,GetMsg(s->msg,"Colleagues"),strlen(GetMsg(s->msg,"Colleagues")));
+		case 4: EncodeUnicode(Bitmap->Text,_("Colleagues"),strlen(_("Colleagues")));
 			break;
-		case 5: EncodeUnicode(Bitmap->Text,GetMsg(s->msg,"Other"),strlen(GetMsg(s->msg,"Other")));
+		case 5: EncodeUnicode(Bitmap->Text,_("Other"),strlen(_("Other")));
 			break;
 		}
 	}
@@ -748,7 +941,7 @@ GSM_Error NOKIA_GetManufacturer(GSM_StateMachine *s)
  * 3. string (unicode, no termination)
  * This function read string to output and increases counter
  */
-void NOKIA_GetUnicodeString(GSM_StateMachine *s, int *current, unsigned char *input, unsigned char *output, bool FullLength)
+void NOKIA_GetUnicodeString(GSM_StateMachine *s UNUSED, int *current, unsigned char *input, unsigned char *output, bool FullLength)
 {
 	int length;
 
@@ -766,7 +959,7 @@ void NOKIA_GetUnicodeString(GSM_StateMachine *s, int *current, unsigned char *in
 	output[length+1] = 0;
 }
 
-int NOKIA_SetUnicodeString(GSM_StateMachine *s, unsigned char *dest, unsigned char *string, bool FullLength)
+int NOKIA_SetUnicodeString(GSM_StateMachine *s UNUSED, unsigned char *dest, unsigned char *string, bool FullLength)
 {
 	int length;
 
@@ -784,7 +977,7 @@ int NOKIA_SetUnicodeString(GSM_StateMachine *s, unsigned char *dest, unsigned ch
 }
 
 /* Returns correct ID for concrete memory type */
-GSM_MemoryType NOKIA_GetMemoryType(GSM_StateMachine *s, GSM_MemoryType memory_type, unsigned char *ID)
+GSM_MemoryType NOKIA_GetMemoryType(GSM_StateMachine *s UNUSED, GSM_MemoryType memory_type, unsigned char *ID)
 {
 	int i=0;
 
@@ -795,7 +988,7 @@ GSM_MemoryType NOKIA_GetMemoryType(GSM_StateMachine *s, GSM_MemoryType memory_ty
 	return 0xff;
 }
 
-void NOKIA_EncodeDateTime(GSM_StateMachine *s, unsigned char* buffer, GSM_DateTime *datetime)
+void NOKIA_EncodeDateTime(GSM_StateMachine *s UNUSED, unsigned char* buffer, GSM_DateTime *datetime)
 {
 	buffer[0] = datetime->Year / 256;
 	buffer[1] = datetime->Year % 256;
@@ -833,23 +1026,23 @@ void NOKIA_SortSMSFolderStatus(GSM_StateMachine *s, GSM_NOKIASMSFolder *Folder)
 	}
 }
 
-void NOKIA_GetDefaultProfileName(GSM_StateMachine *s, GSM_Profile *Profile)
+void NOKIA_GetDefaultProfileName(GSM_Profile *Profile)
 {
 	if (Profile->DefaultName) {
 		switch(Profile->Location) {
-		case 1:	EncodeUnicode(Profile->Name,GetMsg(s->msg,"General"),strlen(GetMsg(s->msg,"General")));
+		case 1:	EncodeUnicode(Profile->Name,_("General"),strlen(_("General")));
 			break;
-		case 2: EncodeUnicode(Profile->Name,GetMsg(s->msg,"Silent"),strlen(GetMsg(s->msg,"Silent")));
+		case 2: EncodeUnicode(Profile->Name,_("Silent"),strlen(_("Silent")));
 			break;
-		case 3: EncodeUnicode(Profile->Name,GetMsg(s->msg,"Meeting"),strlen(GetMsg(s->msg,"Meeting")));
+		case 3: EncodeUnicode(Profile->Name,_("Meeting"),strlen(_("Meeting")));
 			break;
-		case 4:	EncodeUnicode(Profile->Name,GetMsg(s->msg,"Outdoor"),strlen(GetMsg(s->msg,"Outdoor")));
+		case 4:	EncodeUnicode(Profile->Name,_("Outdoor"),strlen(_("Outdoor")));
 			break;
-		case 5:	EncodeUnicode(Profile->Name,GetMsg(s->msg,"Pager"),strlen(GetMsg(s->msg,"Pager")));
+		case 5:	EncodeUnicode(Profile->Name,_("Pager"),strlen(_("Pager")));
 			break;
-		case 6: EncodeUnicode(Profile->Name,GetMsg(s->msg,"Car"),strlen(GetMsg(s->msg,"Car")));
+		case 6: EncodeUnicode(Profile->Name,_("Car"),strlen(_("Car")));
 			break;
-		case 7: EncodeUnicode(Profile->Name,GetMsg(s->msg,"Headset"),strlen(GetMsg(s->msg,"Headset")));
+		case 7: EncodeUnicode(Profile->Name,_("Headset"),strlen(_("Headset")));
 			break;
 		}
 	}
@@ -1005,7 +1198,7 @@ GSM_Error DCT3DCT4_GetActiveConnectSet(GSM_StateMachine *s)
 	return GSM_WaitFor (s, GetSetreq, 4, 0x3f, 4, ID_GetConnectSet);
 }
 
-GSM_Error DCT3DCT4_ReplySetActiveConnectSet(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error DCT3DCT4_ReplySetActiveConnectSet(GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s)
 {
 	smprintf(s, "Connection settings activated\n");
 	return ERR_NONE;
@@ -1030,7 +1223,7 @@ GSM_Error DCT3DCT4_SendDTMF(GSM_StateMachine *s, char *DTMFSequence)
 	unsigned char req[100] = {N6110_FRAME_HEADER, 0x50,
 				  0x00}; 	/* Length */
 
-	if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NODTMF)) return ERR_NOTSUPPORTED;
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NODTMF)) return ERR_NOTSUPPORTED;
 	if (strlen(DTMFSequence) > 100 - 5) return ERR_NOTSUPPORTED;
 
   	req[4] = strlen(DTMFSequence);
@@ -1105,7 +1298,7 @@ GSM_Error DCT3DCT4_ReplySetWAPBookmark(GSM_Protocol_Message msg, GSM_StateMachin
 	return ERR_UNKNOWNRESPONSE;
 }
 
-GSM_Error DCT3DCT4_ReplyEnableConnectFunc(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error DCT3DCT4_ReplyEnableConnectFunc(GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s)
 {
 	smprintf(s, "Connection functions enabled\n");
 	return ERR_NONE;
@@ -1115,13 +1308,13 @@ GSM_Error DCT3DCT4_EnableWAPFunctions(GSM_StateMachine *s)
 {
 	unsigned char req[] = {N6110_FRAME_HEADER, 0x00};
 
-	if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo,F_NOWAP)) return ERR_NOTSUPPORTED;
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo,F_NOWAP)) return ERR_NOTSUPPORTED;
 
 	smprintf(s, "Enabling WAP\n");
 	return GSM_WaitFor (s, req, 4, 0x3f, 4, ID_EnableConnectFunc);
 }
 
-GSM_Error DCT3DCT4_ReplyDisableConnectFunc(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error DCT3DCT4_ReplyDisableConnectFunc(GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s)
 {
 	smprintf(s, "Connection functions disabled\n");
 	return ERR_NONE;
@@ -1131,7 +1324,7 @@ GSM_Error DCT3DCT4_DisableConnectionFunctions(GSM_StateMachine *s)
 {
 	unsigned char req[] = {N6110_FRAME_HEADER, 0x03};
 
-	if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo,F_NOWAP)) return ERR_NOTSUPPORTED;
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo,F_NOWAP)) return ERR_NOTSUPPORTED;
 
 	smprintf(s, "Disabling connection settings\n");
 	return GSM_WaitFor (s, req, 4, 0x3f, 4, ID_DisableConnectFunc);
@@ -1297,6 +1490,10 @@ GSM_Error N71_65_ReplyGetMemoryError(unsigned char error, GSM_StateMachine *s)
 		if (s->Phone.Data.Memory->MemoryType == MEM_ME) return ERR_EMPTY;
 		if (s->Phone.Data.Memory->MemoryType == MEM_SM) return ERR_EMPTY;
 		return ERR_NOTSUPPORTED;
+	case 0x31:
+		smprintf(s, "Invalid memory type?\n");
+		s->Phone.Data.Memory->EntriesNum = 0;
+		return ERR_EMPTY;
 	case 0x33:
 		smprintf(s, "Empty location\n");
 		s->Phone.Data.Memory->EntriesNum = 0;
@@ -1339,6 +1536,9 @@ GSM_Error N71_65_ReplyWritePhonebook(GSM_Protocol_Message msg, GSM_StateMachine
 		case 0x3e:
 			smprintf(s, "Too much entries\n");
 			return ERR_NOTSUPPORTED;
+		case 50:
+			smprintf(s, "Ignoring ERROR: unknown 50 (probably group contains 50 entries)\n");
+			return ERR_NONE;
 		default:
 			smprintf(s, "ERROR: unknown %i\n",msg.Buffer[10]);
 			return ERR_UNKNOWNRESPONSE;
@@ -1475,7 +1675,8 @@ GSM_Error NOKIA_SetIncomingSMS(GSM_StateMachine *s, bool enable)
 
 GSM_Error N71_65_ReplyUSSDInfo(GSM_Protocol_Message msg, GSM_StateMachine *s)
 {
-	unsigned char buffer[2000],buffer2[4000];
+	unsigned char buffer[2000];
+	GSM_USSDMessage ussd;
 
 	if (s->Phone.Data.RequestID == ID_Divert) return ERR_NONE;
 
@@ -1485,8 +1686,12 @@ GSM_Error N71_65_ReplyUSSDInfo(GSM_Protocol_Message msg, GSM_StateMachine *s)
 	smprintf(s, "USSD reply: \"%s\"\n",buffer);
 
 	if (s->Phone.Data.EnableIncomingUSSD && s->User.IncomingUSSD!=NULL) {
-		EncodeUnicode(buffer2,buffer,strlen(buffer));
-		s->User.IncomingUSSD(s->CurrentConfig->Device, buffer2);
+		EncodeUnicode(ussd.Text,buffer,strlen(buffer));
+		/**
+		 * @todo: Should determine status.
+		 */
+		ussd.Status = USSD_Unknown;
+		s->User.IncomingUSSD(s, ussd);
 	}
 
 	return ERR_NONE;
@@ -1507,7 +1712,7 @@ GSM_Error NOKIA_SetIncomingUSSD(GSM_StateMachine *s, bool enable)
 
 GSM_Error NOKIA_SetIncomingCall(GSM_StateMachine *s, bool enable)
 {
-	if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo,F_NOCALLINFO)) return ERR_NOTSUPPORTED;
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo,F_NOCALLINFO)) return ERR_NOTSUPPORTED;
 
 	s->Phone.Data.EnableIncomingCall = enable;
 #ifdef DEBUG
@@ -1646,7 +1851,7 @@ GSM_Error N71_65_ReplyCallInfo(GSM_Protocol_Message msg, GSM_StateMachine *s)
 	if (call.CallIDAvailable) smprintf(s, "Call ID    : %d\n",msg.Buffer[4]);
 	if (s->Phone.Data.EnableIncomingCall && s->User.IncomingCall!=NULL && call.Status != 0) {
 		if (call.CallIDAvailable) call.CallID = msg.Buffer[4];
-		s->User.IncomingCall(s->CurrentConfig->Device, call);
+		s->User.IncomingCall(s, call);
 	}
 	if (s->Phone.Data.RequestID == ID_DialVoice) {
 		if (msg.Buffer[3] == 0x10) return ERR_NOTSUPPORTED;
@@ -1671,7 +1876,7 @@ GSM_Error N71_65_ReplyCallInfo(GSM_Protocol_Message msg, GSM_StateMachine *s)
 }
 
 /* method 2 */
-GSM_Error N71_65_ReplyAddCalendar2(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error N71_65_ReplyAddCalendar2(GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s)
 {
 	smprintf(s, "Calendar note added\n");
 	return ERR_NONE;
@@ -1701,8 +1906,8 @@ GSM_Error N71_65_AddCalendar2(GSM_StateMachine *s, GSM_CalendarEntry *Note)
 
  	NoteType = N71_65_FindCalendarType(Note->Type, s->Phone.Data.ModelInfo);
 
-	if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_CAL62) ||
-	    IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_CAL65)) {
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_CAL62) ||
+	    GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_CAL65)) {
 		switch(NoteType) {
 		case GSM_CAL_MEETING 	: req[18] = 0x01; length = 25; break;
 		case GSM_CAL_CALL    	: req[18] = 0x02; length = 27; break;
@@ -1738,9 +1943,9 @@ GSM_Error N71_65_AddCalendar2(GSM_StateMachine *s, GSM_CalendarEntry *Note)
 		/* 6230 and probably other new models handle it differently
                    we don't make difference from 1980 year
                  */
-		if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_CAL35) ||
-		    IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_CAL65) ||
-		    IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_CAL62)) {
+		if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_CAL35) ||
+		    GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_CAL65) ||
+		    GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_CAL62)) {
 			Date.Year = 1980;
 		}
 		Date.Hour = 22; Date.Minute = 58; Date.Second = 58;
@@ -1792,7 +1997,7 @@ GSM_Error N71_65_AddCalendar2(GSM_StateMachine *s, GSM_CalendarEntry *Note)
 
 		/* Sometimes we have difference in minutes */
 		if (NoteType == GSM_CAL_MEETING) diff = diff / 60;
-		if (!IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_CAL35)) {
+		if (!GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_CAL35)) {
 			if (NoteType == GSM_CAL_MEMO || NoteType == GSM_CAL_CALL) {
 				diff = diff / 60;
 			}
@@ -1916,13 +2121,11 @@ GSM_Error N71_65_AddCalendar1(GSM_StateMachine *s, GSM_CalendarEntry *Note, int
 		req[count++] = 0xff;
 		req[count++] = 0xff;
 		if (Alarm != -1) {
-#ifndef ENABLE_LGPL
 			/* Comment from original source by Gabriele Zappi:
 			 * I try with Time.Year = Alarm.Year. If negative, I increase 1 year,
 			 * but only once ! This thing, because I may have Alarm period across
 			 * a year. (eg. Birthday on 2001-01-10 and Alarm on 2000-12-27)
 			 */
-#endif
 			DT.Year = Note->Entries[Alarm].Date.Year;
 			seconds = Fill_Time_T(DT)-Fill_Time_T(Note->Entries[Alarm].Date);
 			if (seconds<0L) {
@@ -2320,16 +2523,16 @@ GSM_CalendarNoteType N71_65_FindCalendarType(GSM_CalendarNoteType Type, OnePhone
 	case GSM_CAL_BIRTHDAY:
 		return GSM_CAL_BIRTHDAY;
 	case GSM_CAL_MEETING:
-		if (IsPhoneFeatureAvailable(model, F_CAL35)) {
+		if (GSM_IsPhoneFeatureAvailable(model, F_CAL35)) {
 			return GSM_CAL_REMINDER;
 		} else return GSM_CAL_MEETING;
 	case GSM_CAL_MEMO:
-		if (IsPhoneFeatureAvailable(model, F_CAL35)) {
+		if (GSM_IsPhoneFeatureAvailable(model, F_CAL35)) {
 			return GSM_CAL_REMINDER;
 		} else return GSM_CAL_MEMO;
 	case GSM_CAL_REMINDER:
-		if (IsPhoneFeatureAvailable(model, F_CAL62) ||
-		    IsPhoneFeatureAvailable(model, F_CAL65)) {
+		if (GSM_IsPhoneFeatureAvailable(model, F_CAL62) ||
+		    GSM_IsPhoneFeatureAvailable(model, F_CAL65)) {
 			return GSM_CAL_CALL;
 		} else return GSM_CAL_REMINDER;
 	default:
