@@ -324,6 +324,7 @@ GSM_Phone_Functions N3650Phone = {
  	NOTSUPPORTED,			/* 	AddSMSFolder		*/
  	NOTSUPPORTED,			/* 	DeleteSMSFolder		*/
 	NOTIMPLEMENTED,			/*	DialVoice		*/
+        NOTIMPLEMENTED,			/*	DialService		*/
 	NOTIMPLEMENTED,			/*	AnswerCall		*/
 	NOTIMPLEMENTED,			/*	CancelCall		*/
  	NOTIMPLEMENTED,			/* 	HoldCall 		*/
@@ -392,6 +393,7 @@ GSM_Phone_Functions N3650Phone = {
 	NOTSUPPORTED,			/*	SetFileAttributes	*/
 	N3650_GetFilePart,
 	NOTIMPLEMENTED,			/*	AddFilePart		*/
+	NOTSUPPORTED,			/* 	SendFilePart		*/
 	NOTSUPPORTED,	 		/* 	GetFileSystemStatus	*/
 	NOTIMPLEMENTED,			/*	DeleteFile		*/
 	NOTIMPLEMENTED,			/*	AddFolder		*/
