@@ -1,4 +1,5 @@
-/* (c) 2003 by Marcin Wiacek */
+/* (c) 2003 by Marcin Wiacek
+ * (C) 2007 Matthias Lechner <matthias@lmme.de> */
 
 #ifndef gnapgen_h
 #define gnapgen_h
@@ -6,7 +7,6 @@
 #include "../../gsmcomon.h"
 #include "../../gsmstate.h"
 #include "../../service/gsmmisc.h"
-#include "../../service/sms/gsmsms.h"
 
 #ifndef GSM_USED_BLUEGNAPBUS
 #  define GSM_USED_BLUEGNAPBUS
@@ -15,6 +15,13 @@
 #  define GSM_USED_IRDAGNAPBUS
 #endif
 
+typedef struct {
+	int				byte1;
+	int				byte2;
+	int				byte3;
+	int				byte4;
+} TUint32;
+
 typedef struct {
 	int				LastCalendarYear;
 	int				LastCalendarPos;
@@ -24,8 +31,16 @@ typedef struct {
 	GSM_CalendarNoteType		CalendarIconsTypes[10];
 	int				CalendarIconsNum;
 
-	GSM_NOKIASMSFolder		LastSMSFolder;
-	GSM_SMSFolders			LastSMSFolders;
+	bool				LastContactArrived;
+
+	//GSM_OneSMSFolder		CurrentSMSFolder;
+	GSM_SMSFolders			CurrentSMSFolders;
+	int				SMSFolderID[100];
+	int				CurrentFolderNumber;
+
+	TUint32				SMSIDs[5000];
+	int				SMSCount;
+	int				CurrentSMSNumber;
 
 	GSM_NOKIACalToDoLocations	LastToDo;
 
