@@ -10,7 +10,7 @@
 #include "../../gsmcomon.h"
 #include "at.h"
 
-static GSM_Error AT_WriteMessage (GSM_StateMachine *s, unsigned char *buffer,
+static GSM_Error AT_WriteMessage (GSM_StateMachine *s, unsigned const char *buffer,
 				     int length, unsigned char type)
 {
 	int i,sent = 0;
@@ -70,7 +70,7 @@ static GSM_Error AT_StateMachine(GSM_StateMachine *s, unsigned char rx_char)
 
 		{"RING"		,1},	{"NO CARRIER"	,1},
 		{"NO ANSWER"	,1},	{"+COLP"	,1},
-		{"+CLIP"	,1},
+		{"+CLIP"	,1},	{"+CRING"	,1},
 
 		{"*EBCA"	,1},
 
@@ -103,7 +103,9 @@ static GSM_Error AT_StateMachine(GSM_StateMachine *s, unsigned char rx_char)
 		if (d->Msg.Length > 0 && rx_char == 10 && d->Msg.Buffer[d->Msg.Length-2]==13) {
 			i = 0;
 			while (StartStrings[i] != NULL) {
-				if (strncmp(StartStrings[i],d->Msg.Buffer+d->LineStart,strlen(StartStrings[i])) == 0) {
+				if (strncmp(StartStrings[i],
+							d->Msg.Buffer + d->LineStart,
+							strlen(StartStrings[i])) == 0) {
 					s->Phone.Data.RequestMsg	= &d->Msg;
 					s->Phone.Data.DispatchError	= s->Phone.Functions->DispatchMessage(s);
 					d->Msg.Length			= 0;
@@ -115,9 +117,12 @@ static GSM_Error AT_StateMachine(GSM_StateMachine *s, unsigned char rx_char)
 
 			i = 0;
 			while (SpecialAnswers[i].text != NULL) {
-				if (strncmp(SpecialAnswers[i].text,d->Msg.Buffer+d->LineStart,strlen(SpecialAnswers[i].text)) == 0) {
+				if (strncmp(SpecialAnswers[i].text,
+							d->Msg.Buffer + d->LineStart,
+							strlen(SpecialAnswers[i].text)) == 0) {
 					/* We need something better here */
-				  	if (s->Phone.Data.RequestID == ID_GetNetworkInfo && strncmp(SpecialAnswers[i].text,"+CREG:",6) == 0) {
+				  	if (s->Phone.Data.RequestID == ID_GetNetworkInfo && 
+							strcmp(SpecialAnswers[i].text, "+CREG:") == 0) {
 						i++;
 						continue;
 					}
@@ -134,6 +139,7 @@ static GSM_Error AT_StateMachine(GSM_StateMachine *s, unsigned char rx_char)
 				memcpy(Msg2.Buffer,d->Msg.Buffer+d->SpecialAnswerStart,d->LineEnd - d->SpecialAnswerStart + 2);
 				Msg2.Length = d->LineEnd - d->SpecialAnswerStart + 2;
 				Msg2.Buffer[Msg2.Length] = 0;
+				Msg2.Type = 0;
 
 				s->Phone.Data.RequestMsg	= &Msg2;
 				s->Phone.Data.DispatchError	= s->Phone.Functions->DispatchMessage(s);
