@@ -3,9 +3,10 @@
 #include <string.h>
 #include <ctype.h>
 
-#include "../../phone/nokia/nfunc.h"
-#include "../../phone/nokia/dct3/n7110.h"
+#include <gammu-config.h>
+
 #include "../../misc/coding/coding.h"
+#include "../../misc/misc.h"
 #include "../gsmlogo.h"
 #include "../gsmmisc.h"
 #include "backldif.h"
@@ -54,12 +55,15 @@ GSM_Error SaveLDIF(char *FileName, GSM_Backup *backup)
 			case PBK_Text_Postal:
 				SaveLDIFText(file, "HomePostalAddress", backup->PhonePhonebook[i]->Entries[j].Text);
 				break;
+			case PBK_Text_WorkPostal:
+				/* This does not conform to standard, but I think it's better to have it saved */
+				SaveLDIFText(file, "WorkPostalAddress", backup->PhonePhonebook[i]->Entries[j].Text);
+				break;
 			case PBK_Text_URL:
 				SaveLDIFText(file, "homeurl", backup->PhonePhonebook[i]->Entries[j].Text);
 				break;
 			case PBK_Text_Name:
-				SaveLDIFText(file, "givenName", backup->PhonePhonebook[i]->Entries[j].Text);
-				SaveLDIFText(file, "cn", backup->PhonePhonebook[i]->Entries[j].Text);
+				/* Handled above */
 				break;
 			case PBK_Text_Note:
 				SaveLDIFText(file, "Description", backup->PhonePhonebook[i]->Entries[j].Text);
@@ -109,6 +113,21 @@ GSM_Error SaveLDIF(char *FileName, GSM_Backup *backup)
 			case PBK_Text_JobTitle:
 				SaveLDIFText(file, "title", backup->PhonePhonebook[i]->Entries[j].Text);
 				break;
+			case PBK_Text_WorkStreetAddress:
+				SaveLDIFText(file, "workPostalAddress", backup->PhonePhonebook[i]->Entries[j].Text);
+				break;
+			case PBK_Text_WorkCity:
+				SaveLDIFText(file, "workLocalityName", backup->PhonePhonebook[i]->Entries[j].Text);
+				break;
+			case PBK_Text_WorkState:
+				SaveLDIFText(file, "workState", backup->PhonePhonebook[i]->Entries[j].Text);
+				break;
+			case PBK_Text_WorkZip:
+				SaveLDIFText(file, "workPostalCode", backup->PhonePhonebook[i]->Entries[j].Text);
+				break;
+			case PBK_Text_WorkCountry:
+				SaveLDIFText(file, "workCountryName", backup->PhonePhonebook[i]->Entries[j].Text);
+				break;
 			case PBK_Text_StreetAddress:
 				SaveLDIFText(file, "homePostalAddress", backup->PhonePhonebook[i]->Entries[j].Text);
 				break;
@@ -124,19 +143,30 @@ GSM_Error SaveLDIF(char *FileName, GSM_Backup *backup)
 			case PBK_Text_Country:
 				SaveLDIFText(file, "mozillaHomeCountryName", backup->PhonePhonebook[i]->Entries[j].Text);
 				break;
+			case PBK_Text_LastName:
+				SaveLDIFText(file, "sn", backup->PhonePhonebook[i]->Entries[j].Text);
+				break;
+			case PBK_Text_FirstName:
+				SaveLDIFText(file, "cn", backup->PhonePhonebook[i]->Entries[j].Text);
+				break;
+			case PBK_Text_NickName:
+				SaveLDIFText(file, "nickname", backup->PhonePhonebook[i]->Entries[j].Text);
+				break;
+			case PBK_Text_FormalName:
+				SaveLDIFText(file, "cn", backup->PhonePhonebook[i]->Entries[j].Text);
+				break;
 			case PBK_Number_Other:
 			case PBK_Caller_Group:
 			case PBK_RingtoneID:
 			case PBK_PictureID:
 			case PBK_Date:
-			case PBK_RingtoneFileSystemID:
+			case PBK_LastModified:
 			case PBK_Text_UserID:
-			case PBK_SMSListID:
 			case PBK_Category:
 			case PBK_Private:
-			case PBK_Text_LastName:
-			case PBK_Text_FirstName:
 			case PBK_CallLength:
+			case PBK_Text_PictureName:
+			case PBK_PushToTalkID:
 				dbgprintf("Feature missed\n");
 				break;
 			}
@@ -241,6 +271,11 @@ static GSM_Error GSM_DecodeLDIFEntry(unsigned char *Buffer, int *Pos, GSM_Memory
 				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_Postal;
 				Pbk->EntriesNum++;
 			}
+			if (ReadLDIFText(Line, "womePostalAddress", Buff)) {
+				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
+				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_WorkPostal;
+				Pbk->EntriesNum++;
+			}
 			if (ReadLDIFText(Line, "mail", Buff)) {
 				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
 				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_Email;
