@@ -3,9 +3,10 @@
 #include <string.h>
 #include <ctype.h>
 
-#include "../../phone/nokia/nfunc.h"
-#include "../../phone/nokia/dct3/n7110.h"
+#include <gammu-config.h>
+
 #include "../../misc/coding/coding.h"
+#include "../../misc/misc.h"
 #include "../gsmlogo.h"
 #include "../gsmmisc.h"
 #include "backvcf.h"
