@@ -1,6 +1,10 @@
 /* (c) 2002-2004 by Marcin Wiacek */
 
 #include <string.h>
+#include <stdio.h>
+
+#include <gammu-debug.h>
+#include <gammu-datetime.h>
 
 #include "gsmdata.h"
 #include "../misc/coding/coding.h"
@@ -391,17 +395,17 @@ void GSM_EncodeMMSIndicatorSMSText(unsigned char *Buffer, int *Length, GSM_MMSIn
 	Buffer[(*Length)++] = 0x00;
 }
 
-GSM_Error GSM_ClearMMSMultiPart(GSM_EncodedMultiPartMMSInfo2 *info)
+GSM_Error GSM_ClearMMSMultiPart(GSM_EncodedMultiPartMMSInfo *info)
 {
 	int i;
 
-	for (i=0;i<MAX_MULTI_MMS;i++) {
+	for (i=0;i<GSM_MAX_MULTI_MMS;i++) {
 		if (info->Entries[i].File.Buffer != NULL) free(info->Entries[i].File.Buffer);
 	}
 
-	memset(info,0,sizeof(GSM_EncodedMultiPartMMSInfo2));
+	memset(info,0,sizeof(GSM_EncodedMultiPartMMSInfo));
 
-	for (i=0;i<MAX_MULTI_MMS;i++) info->Entries[i].File.Buffer = NULL;
+	for (i=0;i<GSM_MAX_MULTI_MMS;i++) info->Entries[i].File.Buffer = NULL;
 	info->DateTimeAvailable = false;
 
 	return ERR_NONE;
@@ -421,9 +425,10 @@ void GSM_AddWAPMIMEType(int type, unsigned char *buffer)
 	}
 }
 
-GSM_Error GSM_DecodeMMSFileToMultiPart(GSM_File *file, GSM_EncodedMultiPartMMSInfo2 *info)
+GSM_Error GSM_DecodeMMSFileToMultiPart(GSM_File *file, GSM_EncodedMultiPartMMSInfo *info)
 {
-	int 		pos=0,type=0,parts,j;
+	size_t pos = 0;
+	int type=0,parts,j;
 	int		i,len2,len3,len4,value2;
 	long 		value;
 	time_t 		timet;
