@@ -1,10 +1,12 @@
 /* (c) 2001-2004 by Marcin Wiacek */
 
+#define _GNU_SOURCE
 #include <string.h>
 #include <stdlib.h>
 #include <sys/stat.h>
 
-#include "../misc/misc.h"
+#include <gammu-debug.h>
+
 #include "../misc/coding/coding.h"
 #include "gsmlogo.h"
 #include "gsmnet.h"
@@ -67,6 +69,8 @@ static bool PHONE_IsPointBitmap(GSM_Phone_Bitmap_Types Type, char *buffer, int x
 {
 	int i=0, pixel;
 
+	if (x > width || y > height) return false;
+
 	switch (Type) {
 	case GSM_NokiaStartupLogo:
 	case GSM_Nokia6210StartupLogo:
@@ -222,7 +226,11 @@ void GSM_ClearPointBitmap(GSM_Bitmap *bmp, int x, int y)
 
 bool GSM_IsPointBitmap(GSM_Bitmap *bmp, int x, int y)
 {
-	if (GetBit(bmp->BitmapPoints,y*bmp->BitmapWidth+x)) return true; else return false;
+	if (GetBit(bmp->BitmapPoints, (y * bmp->BitmapWidth) + x)) {
+		return true; 
+	} else {
+		return false;
+	}
 }
 
 void GSM_ClearBitmap(GSM_Bitmap *bmp)
@@ -232,7 +240,7 @@ void GSM_ClearBitmap(GSM_Bitmap *bmp)
 
 int GSM_GetBitmapSize(GSM_Bitmap *bmp)
 {
-	return bmp->BitmapWidth*bmp->BitmapHeight/8+1;
+	return ((bmp->BitmapWidth * bmp->BitmapHeight) / 8) + 1;
 }
 
 void GSM_PrintBitmap(FILE *file, GSM_Bitmap *bitmap)
@@ -628,17 +636,17 @@ GSM_Error GSM_SaveBitmapFile(char *FileName, GSM_MultiBitmap *bitmap)
 	if (file == NULL) return ERR_CANTOPENFILE;
 
 	/* Attempt to identify filetype */
-	if (mystrcasestr(FileName,".nlm")) {
+	if (strcasestr(FileName,".nlm")) {
 		error=savenlm(file,bitmap);
-	} else if (mystrcasestr(FileName,".ngg")) {
+	} else if (strcasestr(FileName,".ngg")) {
 		error=savengg(file,bitmap);
-	} else if (mystrcasestr(FileName,".nol")) {
+	} else if (strcasestr(FileName,".nol")) {
 		error=savenol(file,bitmap);
-	} else if (mystrcasestr(FileName,".xpm")) {
+	} else if (strcasestr(FileName,".xpm")) {
 		error=savexpm(file,bitmap);
-	} else if (mystrcasestr(FileName,".nsl")) {
+	} else if (strcasestr(FileName,".nsl")) {
 		error=savensl(file,bitmap);
-	} else if (mystrcasestr(FileName,".wbmp")) {
+	} else if (strcasestr(FileName,".wbmp")) {
 		error=savewbmp(file,bitmap);
 	} else {
 		error=savebmp(file,bitmap);
@@ -855,7 +863,7 @@ static GSM_Error loadnlm (FILE *file, GSM_MultiBitmap *bitmap)
 			if (pos2!=7) {pos2=7;pos++;}
 		}
 		bitmap->Number++;
-		if (bitmap->Number == MAX_MULTI_BITMAP) break;
+		if (bitmap->Number == GSM_MAX_MULTI_BITMAP) break;
 	}
 	return (ERR_NONE);
 }
