@@ -213,7 +213,7 @@ unsigned char *GSM_Networks[] = {
 	"202 05", "PANAFON",
 	"202 09", "Q-TELECOM",
 	"202 10", "TELESTET",
-	"204 04", "LIBERTEL",
+	"204 04", "Vodafone NL",
 	"204 08", "KPN Telecom",
 	"204 12", "O2",
 	"204 16", "BEN",
@@ -231,8 +231,9 @@ unsigned char *GSM_Networks[] = {
 	"214 07", "MOVISTAR",
 	"216 01", "Pannon GSM",
 	"216 30", "T-Mobile H",
-	"216 70", "Vodafone",
+	"216 70", "Vodafone HU",
 	"218 03", "ERONET",
+	"210 01", "Cronet",
 	"218 05", "Mobilna Srpske",
 	"218 90", "GSMBIH",
 	"219 01", "CRONET",
@@ -241,9 +242,11 @@ unsigned char *GSM_Networks[] = {
 	"220 02", "ProMonte GSM",
 	"220 03", "Telekom Srbije",
 	"220 04", "MONET",
+	"220 07", "Promonte",
 	"222 01", "Telecom Italia Mobile",
 	"222 10", "OMNITEL",
 	"222 88", "Wind Telecomunicazioni SpA",
+	"222 98", "Blu s.p.a. Telefonia",
 	"222 99", "H3G",
 	"226 01", "CONNEX GSM",
 	"226 03", "Cosmorom S.A.",
@@ -251,24 +254,26 @@ unsigned char *GSM_Networks[] = {
 	"228 01", "NATEL International",
 	"228 02", "diAx Mobile AG",
 	"228 03", "ORANGE",
+	"228 08", "Tele2",
 	"230 01", "T-Mobile CZ",
-	"230 02", "EuroTel",
+	"230 02", "O2 CZ",
 	"230 03", "Vodafone CZ",
-	"231 01", "Orange",
-	"231 02", "EuroTel GSM",
+	"231 01", "Orange SK",
+	"231 02", "T-Mobile SK",
 	"232 01", "A1",
 	"232 03", "T-Mobile AT",
 	"232 05", "ONE",
 	"232 07", "tele.ring",
 	"232 10", "3 AT",
 	"234 10", "Cellnet",
-	"234 15", "Vodafone",
+	"234 15", "Vodafone UK",
 	"234 20", "3 UK",
 	"234 30", "T-Mobile UK",
 	"234 33", "ORANGE",
 	"234 50", "Jersey Telecoms GSM",
 	"234 55", "Guernsey Telecoms GSM",
 	"234 58", "PRONTO GSM",
+	"234 90", "Jersey",
 	"238 01", "TDK-MOBIL",
 	"238 02", "SONOFON",
 	"238 20", "TELIA DK",
@@ -288,6 +293,7 @@ unsigned char *GSM_Networks[] = {
 	"246 01", "OMNITEL",
 	"246 02", "Bite GSM",
 	"246 03", "TELE2",
+	"246 04", "TELE2",
 	"247 01", "LMT LV",
 	"247 02", "BALTCOM GSM",
 	"248 01", "EMT GSM",
