@@ -4,16 +4,14 @@
 #include <string.h>
 #include <time.h>
 
-#include "../../gsmcomon.h"
+#include <gammu-unicode.h>
+#include <gammu-debug.h>
+
 #include "../../misc/coding/coding.h"
-#include "../gsmcal.h"
-#include "../gsmpbk.h"
+#include "../../misc/misc.h"
 #include "../gsmlogo.h"
-#include "../gsmring.h"
-#include "../gsmdata.h"
-#include "../gsmnet.h"
-#include "gsmsms.h"
 #include "gsmmulti.h"
+#include "../gsmring.h"
 
 /* EMS Developers' Guidelines from www.sonyericsson.com
  * docs from Alcatel
@@ -22,7 +20,7 @@ GSM_Error GSM_EncodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 				    GSM_MultiSMSMessage 	*SMS,
 				    GSM_UDH			UDHType)
 {
-	unsigned char		Buffer[GSM_MAX_SMS_LENGTH*2*MAX_MULTI_SMS];
+	unsigned char		Buffer[GSM_MAX_SMS_LENGTH*2*GSM_MAX_MULTI_SMS];
 	int 			i,UsedText,j,Length,Width,Height,z,x,y;
 	unsigned int		Len;
 	int 			Used,FreeText,FreeBytes,Width2,CopiedText,CopiedSMSText;
@@ -31,7 +29,7 @@ GSM_Error GSM_EncodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 	GSM_Ringtone		Ring;
 	GSM_Coding_Type 	Coding 	= SMS_Coding_Default_No_Compression;
 	GSM_Phone_Bitmap_Types	BitmapType;
-	MultiPartSMSEntry 	*Entry;
+	GSM_MultiPartSMSEntry 	*Entry;
 	bool			start;
 	GSM_DateTime		Date;
 
@@ -42,7 +40,7 @@ GSM_Error GSM_EncodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 	if (Info->UnicodeCoding) Coding = SMS_Coding_Unicode_No_Compression;
 
 	/* Cleaning on the start */
-	for (i=0;i<MAX_MULTI_SMS;i++) {
+	for (i=0;i<GSM_MAX_MULTI_SMS;i++) {
 		GSM_SetDefaultSMSData(&SMS->SMS[i]);
 		SMS->SMS[i].UDH.Type = UDHType;
 		GSM_EncodeUDHHeader(&SMS->SMS[i].UDH);
@@ -93,7 +91,7 @@ GSM_Error GSM_EncodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 				}
 				Len += CopiedText;
 				if (Len == UnicodeLength(Entry->Buffer)) break;
-				dbgprintf("%i %i\n",Len,UnicodeLength(Entry->Buffer));
+				dbgprintf("%u %zu\n",Len,UnicodeLength(Entry->Buffer));
 			}
 			break;
 		case SMS_EMSPredefinedSound:
@@ -530,7 +528,7 @@ GSM_Error GSM_EncodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 	for (i=0;i<SMS->Number;i++) {
 		dbgprintf("UDH length %i\n",SMS->SMS[i].UDH.Length);
 		DumpMessage(&di, SMS->SMS[i].UDH.Text, SMS->SMS[i].UDH.Length);
-		dbgprintf("SMS length %i\n",UnicodeLength(SMS->SMS[i].Text)*2);
+		dbgprintf("SMS length %zu\n",UnicodeLength(SMS->SMS[i].Text)*2);
 		DumpMessage(&di, SMS->SMS[i].Text, UnicodeLength(SMS->SMS[i].Text)*2);
 	}
 #endif
@@ -547,27 +545,33 @@ static bool AddEMSText(GSM_SMSMessage *SMS, GSM_MultiPartSMSInfo *Info, int *Pos
 	    Info->Entries[Info->EntriesNum].ID!=0) {
 		(Info->EntriesNum)++;
 	}
-	BufferLen = UnicodeLength(Info->Entries[Info->EntriesNum].Buffer)*2;
+	BufferLen = UnicodeLength(Info->Entries[Info->EntriesNum].Buffer) * 2 + 2;
 	switch (SMS->Coding) {
+	case -1:
+		/* We will convert the text */
+		SMS->Coding = SMS_Coding_Default_No_Compression;
 	case SMS_Coding_8bit:
-//		memcpy(Info->Entries[Info->EntriesNum].Buffer+BufferLen,SMS->Text+(*Pos),Len);
-//		BufferLen+=Len;
-//		(*Pos)+=Len;
+		Info->Entries[Info->EntriesNum].Buffer = realloc(Info->Entries[Info->EntriesNum].Buffer, BufferLen + (Len * 2));
+		if (Info->Entries[Info->EntriesNum].Buffer == NULL) return false;
+		EncodeUnicode(Info->Entries[Info->EntriesNum].Buffer + BufferLen - 2, SMS->Text + (*Pos) *2, Len);
+		BufferLen += Len * 2;
 		break;
 	case SMS_Coding_Unicode_No_Compression:
 	case SMS_Coding_Default_No_Compression:
-		Info->Entries[Info->EntriesNum].Buffer = realloc(Info->Entries[Info->EntriesNum].Buffer, BufferLen + (Len * 2) + 2);
+		Info->Entries[Info->EntriesNum].Buffer = realloc(Info->Entries[Info->EntriesNum].Buffer, BufferLen + (Len * 2));
 		if (Info->Entries[Info->EntriesNum].Buffer == NULL) return false;
-		memcpy(Info->Entries[Info->EntriesNum].Buffer + BufferLen, SMS->Text + (*Pos) *2, Len * 2);
+		memcpy(Info->Entries[Info->EntriesNum].Buffer + BufferLen - 2, SMS->Text + (*Pos) *2, Len * 2);
 		BufferLen += Len * 2;
 		break;
 	default:
 		break;
 	}
-	(*Pos)+=Len;
-	Info->Entries[Info->EntriesNum].Buffer[BufferLen]	= 0;
-	Info->Entries[Info->EntriesNum].Buffer[BufferLen+1]	= 0;
-	Info->Entries[Info->EntriesNum].ID			= SMS_ConcatenatedTextLong;
+	if (Info->Entries[Info->EntriesNum].Buffer != NULL) {
+		(*Pos)+=Len;
+		Info->Entries[Info->EntriesNum].Buffer[BufferLen - 2]	= 0;
+		Info->Entries[Info->EntriesNum].Buffer[BufferLen - 1]	= 0;
+		Info->Entries[Info->EntriesNum].ID			= SMS_ConcatenatedTextLong;
+	}
 	return true;
 }
 
@@ -579,7 +583,7 @@ bool GSM_DecodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 	GSM_Phone_Bitmap_Types 	BitmapType;
 	GSM_Bitmap		Bitmap,Bitmap2;
 
-	for (i=0;i<MAX_MULTI_SMS;i++) {
+	for (i=0;i<GSM_MAX_MULTI_SMS;i++) {
 		Info->Entries[i].ID	   = 0;
 	}
 
@@ -587,7 +591,7 @@ bool GSM_DecodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 		Pos 	= 0;
 		w	= 1;
 		while (w < SMS->SMS[i].UDH.Length) {
-			if (Info->EntriesNum + 1 == MAX_MULTI_SMS) {
+			if (Info->EntriesNum + 1 == GSM_MAX_MULTI_SMS) {
 				dbgprintf("Couldn't parse SMS, contains too many EMS parts!\n");
 				return false;
 			}
@@ -595,6 +599,24 @@ bool GSM_DecodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 			case 0x00:
 				dbgprintf("UDH part - linked SMS with 8 bit ID\n");
 				break;
+			case 0x05:
+				/* Application data */
+				SMS->SMS[i].Coding = -1;
+				if (!AddEMSText(&SMS->SMS[i], Info, &Pos, SMS->SMS[i].Length-Pos)) return false;
+				Info->Entries[Info->EntriesNum].Number =
+					((0x00ff & SMS->SMS[i].UDH.Text[w + 2]) << 8) | ( 0x00ff &SMS->SMS[i].UDH.Text[w + 3]);
+				switch (Info->Entries[Info->EntriesNum].Number) {
+					case 0x23f4:
+						dbgprintf("UDH part - vCard");
+						Info->Entries[Info->EntriesNum].ID 	= SMS_NokiaVCARD21Long;
+					case 0x23f5:
+						dbgprintf("UDH part - vCalendar");
+						Info->Entries[Info->EntriesNum].ID 	= SMS_NokiaVCALENDAR10Long;
+					default:
+						dbgprintf("UDH part - unknown application data - 0x%04x", Info->Entries[Info->EntriesNum].Number);
+				}
+				RetVal = true;
+				break;
 			case 0x08:
 				dbgprintf("UDH part - linked SMS with 16 bit ID\n");
 				break;
@@ -758,7 +780,7 @@ bool GSM_DecodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 				dbgprintf("UDH part - Object Distribution Indicator (Media Rights Protecting) ignored now\n");
 				break;
 			default:
-				dbgprintf("UDH part - block %02x\n",SMS->SMS[i].UDH.Text[w]);
+				dbgprintf("UDH part - unknonw block %02x\n",SMS->SMS[i].UDH.Text[w]);
 				Info->Unknown = true;
 			} /* switch */
 			w=w+SMS->SMS[i].UDH.Text[w+1]+2;
