@@ -4,17 +4,17 @@
 #include <string.h>
 #include <time.h>
 
-#include "../../gsmcomon.h"
-#include "../../misc/coding/coding.h"
-#include "../gsmcal.h"
-#include "../gsmpbk.h"
-#include "../gsmlogo.h"
+#include <gammu-unicode.h>
+#include <gammu-debug.h>
+
+#include "gsmmulti.h"
 #include "../gsmring.h"
+#include "../gsmlogo.h"
+#include "../../misc/coding/coding.h"
+#include "../../misc/misc.h"
+#include "gsmems.h"
 #include "../gsmdata.h"
 #include "../gsmnet.h"
-#include "gsmsms.h"
-#include "gsmmulti.h"
-#include "gsmems.h"
 
 /* ----------------- Splitting SMS into parts ------------------------------ */
 
@@ -170,7 +170,7 @@ void GSM_MakeMultiPartSMS(GSM_MultiSMSMessage	*SMS,
 		Len += CopiedText;
 		dbgprintf("%i %i\n",Len,MessageLength);
 		if (Len == MessageLength) break;
-		if (SMS->Number == MAX_MULTI_SMS) break;
+		if (SMS->Number == GSM_MAX_MULTI_SMS) break;
 		SMS->Number++;
 	}
 
@@ -280,7 +280,7 @@ GSM_Error GSM_EncodeAlcatelMultiPartSMS(GSM_MultiSMSMessage 	*SMS,
 	int 		p,i;
 	GSM_UDHHeader	MyUDH;
 
-	for (i=0;i<MAX_MULTI_SMS;i++) {
+	for (i=0;i<GSM_MAX_MULTI_SMS;i++) {
 		GSM_SetDefaultSMSData(&SMS->SMS[i]);
 		SMS->SMS[i].UDH.Type    = UDH_UserUDH;
 		SMS->SMS[i].UDH.Text[1] = 0x80;	/* Alcatel */
@@ -335,8 +335,8 @@ GSM_Error GSM_EncodeAlcatelMultiPartSMS(GSM_MultiSMSMessage 	*SMS,
 GSM_Error GSM_EncodeMultiPartSMS(GSM_MultiPartSMSInfo		*Info,
 			    	 GSM_MultiSMSMessage		*SMS)
 {
-	unsigned char	Buffer[GSM_MAX_SMS_LENGTH*2*MAX_MULTI_SMS];
-	unsigned char	Buffer2[GSM_MAX_SMS_LENGTH*2*MAX_MULTI_SMS];
+	unsigned char	Buffer[GSM_MAX_SMS_LENGTH*2*GSM_MAX_MULTI_SMS];
+	unsigned char	Buffer2[GSM_MAX_SMS_LENGTH*2*GSM_MAX_MULTI_SMS];
 	int		Length = 0,smslen,i, Class = -1, j,p;
 	GSM_Error	error;
 	GSM_Coding_Type Coding 	= SMS_Coding_8bit;
@@ -716,7 +716,7 @@ void GSM_ClearMultiPartSMSInfo(GSM_MultiPartSMSInfo *Info)
 {
 	int i;
 
-	for (i=0;i<MAX_MULTI_SMS;i++) {
+	for (i=0;i<GSM_MAX_MULTI_SMS;i++) {
 		Info->Entries[i].Number		= 0;
 		Info->Entries[i].Ringtone	= NULL;
 		Info->Entries[i].Bitmap		= NULL;
@@ -753,7 +753,7 @@ void GSM_FreeMultiPartSMSInfo(GSM_MultiPartSMSInfo *Info)
 {
 	int i;
 
-	for (i=0;i<MAX_MULTI_SMS;i++) {
+	for (i=0;i<GSM_MAX_MULTI_SMS;i++) {
 		free(Info->Entries[i].Ringtone);
 		Info->Entries[i].Ringtone = NULL;
 		free(Info->Entries[i].Bitmap);
@@ -783,7 +783,7 @@ bool GSM_DecodeMultiPartSMS(GSM_MultiPartSMSInfo	*Info,
 {
 	int 			i, Length = 0;
 	unsigned int	j;
-	char			Buffer[GSM_MAX_SMS_LENGTH*2*MAX_MULTI_SMS];
+	char			Buffer[GSM_MAX_SMS_LENGTH*2*GSM_MAX_MULTI_SMS];
 	bool 			emsexist = false;
 	GSM_SiemensOTASMSInfo	SiemensInfo;
 
@@ -1018,11 +1018,13 @@ GSM_Error GSM_LinkSMS(GSM_MultiSMSMessage **INPUT, GSM_MultiSMSMessage **OUTPUT,
 					case 0x00:
 						dbgprintf("Adding ID to user UDH - linked SMS with 8 bit ID\n");
 						INPUT[i]->SMS[0].UDH.ID8bit	= INPUT[i]->SMS[0].UDH.Text[w+2];
+						INPUT[i]->SMS[0].UDH.ID16bit	= -1;
 						INPUT[i]->SMS[0].UDH.AllParts	= INPUT[i]->SMS[0].UDH.Text[w+3];
 						INPUT[i]->SMS[0].UDH.PartNumber	= INPUT[i]->SMS[0].UDH.Text[w+4];
 						break;
 					case 0x08:
 						dbgprintf("Adding ID to user UDH - linked SMS with 16 bit ID\n");
+						INPUT[i]->SMS[0].UDH.ID8bit	= -1;
 						INPUT[i]->SMS[0].UDH.ID16bit	= INPUT[i]->SMS[0].UDH.Text[w+2]*256+INPUT[i]->SMS[0].UDH.Text[w+3];
 						INPUT[i]->SMS[0].UDH.AllParts	= INPUT[i]->SMS[0].UDH.Text[w+4];
 						INPUT[i]->SMS[0].UDH.PartNumber	= INPUT[i]->SMS[0].UDH.Text[w+5];
