@@ -17,15 +17,13 @@
 #include <string.h>
 #include <time.h>
 
-#include "../../gsmcomon.h"
+#include <gammu-calendar.h>
+#include <gammu-message.h>
+#include <gammu-unicode.h>
+#include <gammu-debug.h>
+
 #include "../../misc/coding/coding.h"
-#include "../gsmcal.h"
-#include "../gsmpbk.h"
-#include "../gsmlogo.h"
-#include "../gsmring.h"
-#include "../gsmdata.h"
-#include "../gsmnet.h"
-#include "gsmsms.h"
+#include "../../misc/misc.h"
 
 /* User data headers */
 static GSM_UDHHeader UDHHeaders[] = {
@@ -123,9 +121,7 @@ void GSM_DecodeUDHHeader(GSM_UDHHeader *UDH)
 	UDH->AllParts	= -1;
 
 	i=-1;
-	while (true) {
-		i++;
-		if (UDHHeaders[i].Type==UDH_NoUDH) break;
+	while (UDHHeaders[++i].Type != UDH_NoUDH) {
 
 		tmp=UDHHeaders[i].Length;
 		/* if length is the same */
@@ -385,6 +381,7 @@ GSM_Error GSM_DecodeSMSFrame(GSM_SMSMessage *SMS, unsigned char *buffer, GSM_SMS
 	SMS->SMSC.Format		= SMS_FORMAT_Text;
 	SMS->Number[0]			= 0;
 	SMS->Number[1]			= 0;
+	SMS->OtherNumbersNum		= 0;
 	SMS->Name[0]			= 0;
 	SMS->Name[1]			= 0;
 	SMS->ReplyViaSameSMSC		= false;
@@ -670,8 +667,6 @@ void GSM_SetDefaultSMSData(GSM_SMSMessage *SMS)
 	SMS->ReplyViaSameSMSC		= 0;
 	SMS->Class			= 0;
 	SMS->MessageReference		= 0;
-	SMS->ReplaceMessage		= 0;
-	SMS->RejectDuplicates		= 0;
 	SMS->OtherNumbersNum		= 0;
 
 	/* This part is required to save SMS */
@@ -766,7 +761,7 @@ bool GSM_DecodeSiemensOTASMS(GSM_SiemensOTASMSInfo	*Info,
 	if (SMS->Text[20] > 9) return false;
 	memcpy(Info->DataType,SMS->Text+21,SMS->Text[20]);
 	Info->DataType[SMS->Text[20]] = 0;
-	dbgprintf("DataType '%s'\n",Info->DataType);	
+	dbgprintf("DataType '%s'\n",Info->DataType);
 
 	current = 21+SMS->Text[20];
 	if (SMS->Text[current] > 39) return false;
