@@ -1,16 +1,14 @@
 /* (c) 2002-2004 by Marcin Wiacek */
 /* MSID by Walek */
 
-#include "../../../common/gsmstate.h"
+#include <gammu.h>
 
 #ifdef GSM_ENABLE_NOKIA_DCT3
 
 #include <string.h>
 #include <signal.h>
 
-#include "../../../common/misc/coding/coding.h"
-#include "../../../common/gsmcomon.h"
-#include "../../../common/service/gsmpbk.h"
+#include "../../../common/gsmstate.h"
 #include "../../../common/phone/nokia/dct3/dct3func.h"
 #include "../../../common/phone/pfunc.h"
 #include "../../gammu.h"
@@ -25,20 +23,20 @@ GSM_Error CheckDCT3Only()
 
 	/* Checking if phone is DCT3 */
 #ifdef GSM_ENABLE_NOKIA6110
-	if (strstr(N6110Phone.models, s.Phone.Data.ModelInfo->model) != NULL) found = true;
+	if (strstr(N6110Phone.models, s->Phone.Data.ModelInfo->model) != NULL) found = true;
 #endif
 #ifdef GSM_ENABLE_NOKIA7110
-	if (strstr(N7110Phone.models, s.Phone.Data.ModelInfo->model) != NULL) found = true;
+	if (strstr(N7110Phone.models, s->Phone.Data.ModelInfo->model) != NULL) found = true;
 #endif
 #ifdef GSM_ENABLE_NOKIA9210
-	if (strstr(N9210Phone.models, s.Phone.Data.ModelInfo->model) != NULL) found = true;
+	if (strstr(N9210Phone.models, s->Phone.Data.ModelInfo->model) != NULL) found = true;
 #endif
 	if (!found) return ERR_NOTSUPPORTED;
 
-	if (s.ConnectionType!=GCT_MBUS2	    && s.ConnectionType!=GCT_FBUS2      &&
-	    s.ConnectionType!=GCT_FBUS2DLR3 && s.ConnectionType!=GCT_FBUS2BLUE  &&
-	    s.ConnectionType!=GCT_FBUS2IRDA && s.ConnectionType!=GCT_IRDAPHONET &&
-	    s.ConnectionType!=GCT_BLUEFBUS2) {
+	if (s->ConnectionType!=GCT_MBUS2	    && s->ConnectionType!=GCT_FBUS2      &&
+	    s->ConnectionType!=GCT_FBUS2DLR3 && s->ConnectionType!=GCT_FBUS2BLUE  &&
+	    s->ConnectionType!=GCT_FBUS2IRDA && s->ConnectionType!=GCT_IRDAPHONET &&
+	    s->ConnectionType!=GCT_BLUEFBUS2) {
 		return ERR_OTHERCONNECTIONREQUIRED;
 	}
 	return ERR_NONE;
@@ -55,7 +53,7 @@ static void CheckDCT3()
 		break;
 	case ERR_OTHERCONNECTIONREQUIRED:
 		printf("Can't do it with current phone protocol\n");
-		GSM_TerminateConnection(&s);
+		GSM_TerminateConnection(s);
 		exit(-1);
 	default:
 		break;
@@ -71,8 +69,8 @@ static bool answer_yes3(char *text)
 		printf("%s (yes/no) ? ",text);
 		len=GetLine(stdin, ans, 99);
 		if (len==-1) exit(-1);
-		if (mystrncasecmp(ans, "yes",0)) return true;
-		if (mystrncasecmp(ans, "no" ,0)) return false;
+		if (strcasecmp(ans, "yes") == 0) return true;
+		if (strcasecmp(ans, "no" ) == 0) return false;
 	}
 }
 
@@ -92,6 +90,7 @@ static GSM_Error DCT3_ReplyGetT9(GSM_Protocol_Message msg, GSM_StateMachine *s)
 void DCT3GetT9(int argc, char *argv[])
 {
 	int	      i;
+	GSM_Error error;
 	unsigned char req[] = {0x00, 0x01, 0xAE, 0x02, 0x00,
 			       0x00};	/* Part number */
 
@@ -102,11 +101,11 @@ void DCT3GetT9(int argc, char *argv[])
 	DCT3T9File = fopen("T9", "w");
 	if (DCT3T9File == NULL) return;
 
-	s.User.UserReplyFunctions=UserReplyFunctions3;
+	s->User.UserReplyFunctions=UserReplyFunctions3;
 
 	for (i=0;i<5;i++) {
 		req[5] = i;
-		error=GSM_WaitFor (&s, req, 6, 0x40, 4, ID_User3);
+		error=GSM_WaitFor (s, req, 6, 0x40, 4, ID_User3);
 		Print_Error(error);
 	}
 
@@ -118,22 +117,23 @@ void DCT3VibraTest(int argc, char *argv[])
 	unsigned char ans[200];
 	unsigned char SetLevel[4] = {0x00, 0x01, 0xA3,
 				     0xff};	/* Level */
+	GSM_Error error;
 
 	if (CheckDCT3Only()!=ERR_NONE) return;
 
-	s.User.UserReplyFunctions=UserReplyFunctions3;
+	s->User.UserReplyFunctions=UserReplyFunctions3;
 
-	error=DCT3_EnableSecurity (&s, 0x01);
+	error=DCT3_EnableSecurity (s, 0x01);
 	Print_Error(error);
 
-	error=GSM_WaitFor (&s, SetLevel, 4, 0x40, 4, ID_User3);
+	error=GSM_WaitFor (s, SetLevel, 4, 0x40, 4, ID_User3);
 	Print_Error(error);
 
 	printf("Press any key to continue...\n");
 	GetLine(stdin, ans, 99);
 
 	SetLevel[3] = 0x00;
-	error=GSM_WaitFor (&s, SetLevel, 4, 0x40, 4, ID_User3);
+	error=GSM_WaitFor (s, SetLevel, 4, 0x40, 4, ID_User3);
 }
 
 static GSM_Error DCT3_ReplyPhoneTests(GSM_Protocol_Message msg, GSM_StateMachine *s)
@@ -184,15 +184,16 @@ void DCT3SelfTests(int argc, char *argv[])
 	unsigned char 	buffer[3]  = {0x00,0x01,0xcf};
 	unsigned char 	buffer3[8] = {0x00,0x01,0xce,0x1d,0xfe,0x23,0x00,0x00};
 	int		i;
+	GSM_Error error;
 
 	if (CheckDCT3Only()!=ERR_NONE) return;
 
-	error=DCT3_EnableSecurity (&s, 0x01);
+	error=DCT3_EnableSecurity (s, 0x01);
 	Print_Error(error);
 
 	if (answer_yes3("Run all tests now ?")) {
 		/* make almost all tests */
-		error = s.Protocol.Functions->WriteMessage(&s, buffer3, 8, 0x40);
+		error = s->Protocol.Functions->WriteMessage(s, buffer3, 8, 0x40);
 		Print_Error(error);
 
 		GSM_Terminate();
@@ -206,10 +207,10 @@ void DCT3SelfTests(int argc, char *argv[])
 		my_sleep(400);
 	}
 
-	s.User.UserReplyFunctions=UserReplyFunctions3;
+	s->User.UserReplyFunctions=UserReplyFunctions3;
 
 	for (i=0;i<10;i++) {
-		error=GSM_WaitFor (&s, buffer, 3, 0x40, 4, ID_User1);
+		error=GSM_WaitFor (s, buffer, 3, 0x40, 4, ID_User1);
 		if (error == ERR_NONE) break;
 	}
 }
@@ -255,18 +256,19 @@ static GSM_Error DCT3_ReplyGetADC(GSM_Protocol_Message msg, GSM_StateMachine *s)
 void DCT3GetADC(int argc, char *argv[])
 {
 	int		i = 0;
+	GSM_Error error;
 	unsigned char	GetRaw[] = {0x00, 0x01, 0x68};
 	unsigned char	GetUnit[] = {0x00, 0x01, 0x91,
 				     0x02};		/* Test number */
 
 	if (CheckDCT3Only()!=ERR_NONE) return;
 
-	s.User.UserReplyFunctions=UserReplyFunctions3;
+	s->User.UserReplyFunctions=UserReplyFunctions3;
 
-	error=DCT3_EnableSecurity (&s, 0x02);
+	error=DCT3_EnableSecurity (s, 0x02);
 	Print_Error(error);
 
-	error=GSM_WaitFor (&s, GetRaw, 3, 0x40, 6, ID_User3);
+	error=GSM_WaitFor (s, GetRaw, 3, 0x40, 6, ID_User3);
 	Print_Error(error);
 
 	while (1) {
@@ -280,7 +282,7 @@ void DCT3GetADC(int argc, char *argv[])
 		if (DCT3ADC[i].pos2 != -1) {
 			printf("unit result ");
 			GetUnit[3] = DCT3ADC[i].pos2;
-			error=GSM_WaitFor (&s, GetUnit, 6, 0x40, 4, ID_User3);
+			error=GSM_WaitFor (s, GetUnit, 6, 0x40, 4, ID_User3);
 			Print_Error(error);
 			printf("%10i ",DCT3ADCInt*DCT3ADC[i].x);
 			printf("%s\n",DCT3ADC[i].unit);
@@ -289,13 +291,14 @@ void DCT3GetADC(int argc, char *argv[])
 		if (DCT3ADC[i].name[0] == 0x00) break;
 	}
 
-	error=DCT3_EnableSecurity (&s, 0x01);
+	error=DCT3_EnableSecurity (s, 0x01);
 	Print_Error(error);
 }
 
 void DCT3DisplayTest(int argc, char *argv[])
 {
 	unsigned char ans[200];
+	GSM_Error error;
 	unsigned char req[] =  {0x00, 0x01, 0xD3,
 				0x03,          	/* 3=set, 2=clear */
 				0x03};	 	/* test number */
@@ -306,31 +309,32 @@ void DCT3DisplayTest(int argc, char *argv[])
 		printf("Give 1 or 2 as test number\n");
 	}
 
-	s.User.UserReplyFunctions=UserReplyFunctions3;
+	s->User.UserReplyFunctions=UserReplyFunctions3;
 
 	req[4] = atoi(argv[2]);
-	s.Protocol.Functions->WriteMessage(&s, req, 5, 0x40);
+	s->Protocol.Functions->WriteMessage(s, req, 5, 0x40);
 
 	printf("Press any key to continue...\n");
 	GetLine(stdin, ans, 99);
 
 	req[3] = 0x02;
 	req[4] = 0x03;
-	s.Protocol.Functions->WriteMessage(&s, req, 5, 0x40);
+	s->Protocol.Functions->WriteMessage(s, req, 5, 0x40);
 
-	error=DCT3_EnableSecurity (&s, 0x03);
+	error=DCT3_EnableSecurity (s, 0x03);
 	Print_Error(error);
 }
 
 void DCT3netmonitor(int argc, char *argv[])
 {
+	GSM_Error error;
 	char value[100];
 
 	GSM_Init(true);
 
         CheckDCT3();
 
-	error=DCT3_Netmonitor(&s, atoi(argv[2]), value);
+	error=DCT3_Netmonitor(s, atoi(argv[2]), value);
 	Print_Error(error);
 
 	printf("%s\n",value);
@@ -438,26 +442,27 @@ void DCT3Info(int argc, char *argv[])
 	unsigned char 		req2[]  = {0x00, 0x01, 0xb4, 0x00, 0x00}; /* Get MSID */
 	unsigned char 		req3[]  = {0x00, 0x01, 0xc8, 0x02}; 	  /* Get MCU chksum */
 	unsigned char 		req4[]  = {0x00, 0x01, 0xc8, 0x09}; 	  /* Get DSP ROM */
+	GSM_Error error;
 
         if (CheckDCT3Only()!=ERR_NONE) return;
 
-	s.User.UserReplyFunctions=UserReplyFunctions3;
+	s->User.UserReplyFunctions=UserReplyFunctions3;
 
-	error=DCT3_EnableSecurity (&s, 0x01);
+	error=DCT3_EnableSecurity (s, 0x01);
 	Print_Error(error);
 
-	error=GSM_WaitFor (&s, req, 4, 0x40, 4, ID_User3);
+	error=GSM_WaitFor (s, req, 4, 0x40, 4, ID_User3);
 	Print_Error(error);
 
 	req2[3] = MSID1;
 	req2[4] = req2[2] + req2[3];
- 	error=GSM_WaitFor (&s, req2, 5, 0x40, 4, ID_User8);
+ 	error=GSM_WaitFor (s, req2, 5, 0x40, 4, ID_User8);
 	Print_Error(error);
 
-	error=GSM_WaitFor (&s, req3, 4, 0x40, 4, ID_User9);
+	error=GSM_WaitFor (s, req3, 4, 0x40, 4, ID_User9);
 	Print_Error(error);
 
-	error=GSM_WaitFor (&s, req4, 4, 0x40, 4, ID_User10);
+	error=GSM_WaitFor (s, req4, 4, 0x40, 4, ID_User10);
 	Print_Error(error);
 }
 
@@ -470,17 +475,18 @@ static GSM_Error DCT3_ReplyResetTest36(GSM_Protocol_Message msg, GSM_StateMachin
 void DCT3ResetTest36(int argc, char *argv[])
 {
 	unsigned char req[]  = {0x00, 0x01, 0x65, 0x40, 0x00}; /* Reset test 36 in netmon */
+	GSM_Error error;
 
 	GSM_Init(true);
 
         CheckDCT3();
 
-	error=DCT3_EnableSecurity (&s, 0x01);
+	error=DCT3_EnableSecurity (s, 0x01);
 	Print_Error(error);
 
-	s.User.UserReplyFunctions=UserReplyFunctions3;
+	s->User.UserReplyFunctions=UserReplyFunctions3;
 
-	error=GSM_WaitFor (&s, req, 5, 0x40, 4, ID_User2);
+	error=GSM_WaitFor (s, req, 5, 0x40, 4, ID_User2);
 	Print_Error(error);
 
 #ifdef GSM_ENABLE_BEEP
@@ -526,6 +532,7 @@ static GSM_Error DCT3_ReplySetPPS(GSM_Protocol_Message msg, GSM_StateMachine *s)
 void DCT3SetPhoneMenus(int argc, char *argv[])
 {
 	char 		value[100];
+	GSM_Error error;
 	int 		i,j,z;
 	unsigned char 	reqGet[] = {0x00, 0x01, 0x6a};
 	unsigned char 	reqSet[] = {
@@ -534,36 +541,36 @@ void DCT3SetPhoneMenus(int argc, char *argv[])
 
 	if (CheckDCT3Only()!=ERR_NONE) return;
 
-	error=DCT3_EnableSecurity (&s, 0x01);
+	error=DCT3_EnableSecurity (s, 0x01);
 	Print_Error(error);
 
-	s.User.UserReplyFunctions=UserReplyFunctions3;
+	s->User.UserReplyFunctions=UserReplyFunctions3;
 
-	error=GSM_WaitFor (&s, reqGet, 3, 0x40, 4, ID_User4);
+	error=GSM_WaitFor (s, reqGet, 3, 0x40, 4, ID_User4);
 	Print_Error(error);
 
 	printf("ALS : enabling menu\n");
 	PPS[10] = '1';
 
-	if (!strcmp(s.Phone.Data.ModelInfo->model,"3310") && s.Phone.Data.VerNum>5.87) {
+	if (!strcmp(s->Phone.Data.ModelInfo->model,"3310") && s->Phone.Data.VerNum>5.87) {
 		printf("3310: enabling control of SMS charsets\n");
 		PPS[11] = '0';//0 = ON, 1 = OFF
 	}
-	if (!strcmp(s.Phone.Data.ModelInfo->model,"6150")) {
+	if (!strcmp(s->Phone.Data.ModelInfo->model,"6150")) {
 		printf("6150: enabling WellMate menu\n");
 		PPS[18] = '1';
 	}
 	/* FIXME */
-	if (!strcmp(s.Phone.Data.ModelInfo->model,"3210")) {
+	if (!strcmp(s->Phone.Data.ModelInfo->model,"3210")) {
 		printf("3210: enabling vibra menu\n");
 		PPS[24] = '1';
 	}
-	if (!strcmp(s.Phone.Data.ModelInfo->model,"3310") && s.Phone.Data.VerNum>5.13) {
+	if (!strcmp(s->Phone.Data.ModelInfo->model,"3310") && s->Phone.Data.VerNum>5.13) {
 		printf("3310: enabling 3315 features\n");
 		PPS[25] = '1';
 	}
 	/* FIXME */
-	if (!strcmp(s.Phone.Data.ModelInfo->model,"3210") && s.Phone.Data.VerNum>=5.31) {
+	if (!strcmp(s->Phone.Data.ModelInfo->model,"3210") && s->Phone.Data.VerNum>=5.31) {
 		printf("3210: enabling React and Logic game\n");
 		PPS[26] = '1';
 	}
@@ -588,11 +595,11 @@ void DCT3SetPhoneMenus(int argc, char *argv[])
 //	reqSet[5]=0x00;
 //	reqSet[6]=0xe0;
 
-	error=GSM_WaitFor (&s, reqSet, 7, 0x40, 4, ID_User4);
+	error=GSM_WaitFor (s, reqSet, 7, 0x40, 4, ID_User4);
 	Print_Error(error);
 
 	printf("Enabling netmonitor\n");
-	error=DCT3_Netmonitor(&s, 243, value);
+	error=DCT3_Netmonitor(s, 243, value);
 	Print_Error(error);
 }
 
@@ -618,27 +625,28 @@ void DCT3GetSecurityCode(int argc, char *argv[])
 	unsigned char req71_91[] = {N7110_FRAME_HEADER, 0xee,
 				    0x1c};	/* Setting */
 #endif
+	GSM_Error error;
 
 	if (CheckDCT3Only()!=ERR_NONE) return;
 
-	error=DCT3_EnableSecurity (&s, 0x01);
+	error=DCT3_EnableSecurity (s, 0x01);
 	Print_Error(error);
 
-	s.User.UserReplyFunctions=UserReplyFunctions3;
+	s->User.UserReplyFunctions=UserReplyFunctions3;
 
 #ifdef GSM_ENABLE_NOKIA6110
-	if (strstr(N6110Phone.models, s.Phone.Data.ModelInfo->model) != NULL) {
-		error=GSM_WaitFor (&s, req6110, 4, 0x40, 4, ID_User6);
+	if (strstr(N6110Phone.models, s->Phone.Data.ModelInfo->model) != NULL) {
+		error=GSM_WaitFor (s, req6110, 4, 0x40, 4, ID_User6);
 	}
 #endif
 #ifdef GSM_ENABLE_NOKIA7110
-	if (strstr(N7110Phone.models, s.Phone.Data.ModelInfo->model) != NULL) {
-		error=GSM_WaitFor (&s, req71_91, 5, 0x7a, 4, ID_User6);
+	if (strstr(N7110Phone.models, s->Phone.Data.ModelInfo->model) != NULL) {
+		error=GSM_WaitFor (s, req71_91, 5, 0x7a, 4, ID_User6);
 	}
 #endif
 #ifdef GSM_ENABLE_NOKIA9210
-	if (strstr(N9210Phone.models, s.Phone.Data.ModelInfo->model) != NULL) {
-		error=GSM_WaitFor (&s, req71_91, 5, 0x7a, 4, ID_User6);
+	if (strstr(N9210Phone.models, s->Phone.Data.ModelInfo->model) != NULL) {
+		error=GSM_WaitFor (s, req71_91, 5, 0x7a, 4, ID_User6);
 	}
 #endif
 	Print_Error(error);
@@ -662,18 +670,19 @@ static GSM_Error DCT3_ReplyGetOperatorName(GSM_Protocol_Message msg, GSM_StateMa
 void DCT3GetOperatorName(int argc, char *argv[])
 {
 	unsigned char req[] = {0x00,0x01,0x8c,0x00};
+	GSM_Error error;
 
 	GSM_Init(true);
 
-	if (strstr(N6110Phone.models, s.Phone.Data.ModelInfo->model) == NULL) Print_Error(ERR_NOTSUPPORTED);
+	if (strstr(N6110Phone.models, s->Phone.Data.ModelInfo->model) == NULL) Print_Error(ERR_NOTSUPPORTED);
 	CheckDCT3();
 
-	error=DCT3_EnableSecurity (&s, 0x01);
+	error=DCT3_EnableSecurity (s, 0x01);
 	Print_Error(error);
 
-	s.User.UserReplyFunctions=UserReplyFunctions3;
+	s->User.UserReplyFunctions=UserReplyFunctions3;
 
-	error=GSM_WaitFor (&s, req, 4, 0x40, 4, ID_User5);
+	error=GSM_WaitFor (s, req, 4, 0x40, 4, ID_User5);
 	Print_Error(error);
 
 	GSM_Terminate();
@@ -688,19 +697,20 @@ static GSM_Error DCT3_ReplySetOperatorName(GSM_Protocol_Message msg, GSM_StateMa
 void DCT3SetOperatorName(int argc, char *argv[])
 {
 	int 		i = 0;
+	GSM_Error error;
 	unsigned char 	req[256] = {0x00,0x01,0x8b,0x00,
 				    0x00,0x00, /* MCC */
 				    0x00};     /* MNC */
 
 	GSM_Init(true);
 
-	if (strstr(N6110Phone.models, s.Phone.Data.ModelInfo->model) == NULL) Print_Error(ERR_NOTSUPPORTED);
+	if (strstr(N6110Phone.models, s->Phone.Data.ModelInfo->model) == NULL) Print_Error(ERR_NOTSUPPORTED);
 	CheckDCT3();
 
-	error=DCT3_EnableSecurity (&s, 0x01);
+	error=DCT3_EnableSecurity (s, 0x01);
 	Print_Error(error);
 
-	s.User.UserReplyFunctions=UserReplyFunctions3;
+	s->User.UserReplyFunctions=UserReplyFunctions3;
 
 	switch (argc) {
 	case 2:
@@ -713,7 +723,7 @@ void DCT3SetOperatorName(int argc, char *argv[])
 		i = strlen(argv[3]);
 	}
 
-	error=GSM_WaitFor (&s, req, 8+i, 0x40, 4, ID_User7);
+	error=GSM_WaitFor (s, req, 8+i, 0x40, 4, ID_User7);
 	Print_Error(error);
 
 	GSM_Terminate();
@@ -742,15 +752,16 @@ void DCT3DisplayOutput(int argc, char *argv[])
 {
 	unsigned char req[] = {N6110_FRAME_HEADER, 0x53,
 			       0x01}; //1 = enable, 2 = disable
+	GSM_Error error;
 
 	GSM_Init(true);
 
-	if (strstr(N6110Phone.models, s.Phone.Data.ModelInfo->model) == NULL) Print_Error(ERR_NOTSUPPORTED);
+	if (strstr(N6110Phone.models, s->Phone.Data.ModelInfo->model) == NULL) Print_Error(ERR_NOTSUPPORTED);
 	CheckDCT3();
 
-	s.User.UserReplyFunctions=UserReplyFunctions3;
+	s->User.UserReplyFunctions=UserReplyFunctions3;
 
-	error=GSM_WaitFor (&s, req, 5, 0x0d, 4, ID_User7);
+	error=GSM_WaitFor (s, req, 5, 0x0d, 4, ID_User7);
 	Print_Error(error);
 
 	signal(SIGINT, interrupt);
@@ -758,12 +769,12 @@ void DCT3DisplayOutput(int argc, char *argv[])
 	printf("Entering monitor mode...\n\n");
 
 	while (!gshutdown) {
-		GSM_ReadDevice(&s,true);
+		GSM_ReadDevice(s,true);
 		my_sleep(10);
 	}
 
 	req[4] = 0x02;
-	error=GSM_WaitFor (&s, req, 5, 0x0d, 4, ID_User7);
+	error=GSM_WaitFor (s, req, 5, 0x0d, 4, ID_User7);
 	Print_Error(error);
 
 	GSM_Terminate();
