@@ -12,6 +12,7 @@
 #include "../../../common/phone/nokia/nfunc.h"
 #include "../../../common/phone/nokia/dct4s40/dct4func.h"
 #include "../../../common/misc/coding/coding.h"
+#include "../../../common/misc/locales.h"
 
 extern GSM_Reply_Function UserReplyFunctions4[];
 
@@ -23,20 +24,20 @@ GSM_Error CheckDCT4Only()
 
 	/* Checking if phone is DCT4 */
 #ifdef GSM_ENABLE_NOKIA3650
- 	if (strstr(N3650Phone.models, s.Phone.Data.ModelInfo->model) != NULL) found = true;
+ 	if (strstr(N3650Phone.models, s->Phone.Data.ModelInfo->model) != NULL) found = true;
 #endif
 #ifdef GSM_ENABLE_NOKIA6510
- 	if (strstr(N6510Phone.models, s.Phone.Data.ModelInfo->model) != NULL) found = true;
+ 	if (strstr(N6510Phone.models, s->Phone.Data.ModelInfo->model) != NULL) found = true;
 #endif
 #ifdef GSM_ENABLE_NOKIA3320
- 	if (strstr(N3320Phone.models, s.Phone.Data.ModelInfo->model) != NULL) found = true;
+ 	if (strstr(N3320Phone.models, s->Phone.Data.ModelInfo->model) != NULL) found = true;
 #endif
 	if (!found) return ERR_NOTSUPPORTED;
 
-	if (s.ConnectionType!=GCT_MBUS2	     && s.ConnectionType!=GCT_FBUS2      &&
-	    s.ConnectionType!=GCT_FBUS2DLR3  && s.ConnectionType!=GCT_PHONETBLUE &&
-	    s.ConnectionType!=GCT_IRDAPHONET && s.ConnectionType!=GCT_BLUEPHONET &&
-	    s.ConnectionType!=GCT_DKU5FBUS2  && s.ConnectionType!=GCT_DKU2PHONET) {
+	if (s->ConnectionType!=GCT_MBUS2	     && s->ConnectionType!=GCT_FBUS2      &&
+	    s->ConnectionType!=GCT_FBUS2DLR3  && s->ConnectionType!=GCT_PHONETBLUE &&
+	    s->ConnectionType!=GCT_IRDAPHONET && s->ConnectionType!=GCT_BLUEPHONET &&
+	    s->ConnectionType!=GCT_DKU5FBUS2  && s->ConnectionType!=GCT_DKU2PHONET) {
 		return ERR_OTHERCONNECTIONREQUIRED;
 	}
 	return ERR_NONE;
@@ -52,8 +53,8 @@ static void CheckDCT4()
 		Print_Error(ERR_NOTSUPPORTED);
 		break;
 	case ERR_OTHERCONNECTIONREQUIRED:
-		printf("Can't do it with current phone protocol\n");
-		GSM_TerminateConnection(&s);
+		printf("%s\n", _("Can't do it with current phone protocol"));
+		GSM_TerminateConnection(s);
 		exit(-1);
 	default:
 		break;
@@ -66,11 +67,11 @@ static bool answer_yes2(char *text)
     	char        ans[99];
 
 	while (1) {
-		printf("%s (yes/no) ? ",text);
+		printf(_("%s (yes/no) ? "),text);
 		len=GetLine(stdin, ans, 99);
 		if (len==-1) exit(-1);
-		if (mystrncasecmp(ans, "yes",0)) return true;
-		if (mystrncasecmp(ans, "no" ,0)) return false;
+		if (strcasecmp(ans, _("yes")) == 0) return true;
+		if (strcasecmp(ans, _("no")) == 0) return false;
 	}
 }
 
@@ -249,7 +250,7 @@ static DCT4_Phone_Features DCT4PhoneFeatures[] = {
 
 static GSM_Error DCT4_ReplySetPPS(GSM_Protocol_Message msg, GSM_StateMachine *s)
 {
-	printf("Setting done OK\n");
+	printf("%s\n", _("Setting done"));
 	return ERR_NONE;
 }
 
@@ -259,13 +260,14 @@ void DCT4SetPhoneMenus(int argc, char *argv[])
 	unsigned char 	reqSet[200] = {
 		N7110_FRAME_HEADER,0x04,0x00,0x01,0x47,0x48,0x02,
 		0x00};		/* Number of changed features */
+	GSM_Error error;
 
 	if (CheckDCT4Only()!=ERR_NONE) return;
 
-	s.User.UserReplyFunctions=UserReplyFunctions4;
+	s->User.UserReplyFunctions=UserReplyFunctions4;
 
 	while (DCT4PhoneFeatures[i].Model[0] != 0x00) {
-		if (!strcmp(DCT4PhoneFeatures[i].Model,s.Phone.Data.Model)) {
+		if (!strcmp(DCT4PhoneFeatures[i].Model,s->Phone.Data.Model)) {
 			j = 0;
 			while (DCT4PhoneFeatures[i].Features[j].Name != 0x00) {
 				z = 0;
@@ -286,14 +288,14 @@ void DCT4SetPhoneMenus(int argc, char *argv[])
 	}
 
 	if (current == 10) {
-		printf("Sorry, but configuration matrix for this model is not added yet. Please report\n");
+		printf("%s\n", _("Sorry, but configuration matrix for this model is not added yet. Please report"));
 		return;
 	}
 
 	reqSet[current++] = 0x00;
 	reqSet[current++] = 0x00;
 
-	error=GSM_WaitFor (&s, reqSet, current, 0x1b, 4, ID_User1);
+	error=GSM_WaitFor (s, reqSet, current, 0x1b, 4, ID_User1);
 	Print_Error(error);
 }
 
@@ -352,14 +354,14 @@ static GSM_Error DCT4_ReplyTestsStatus(GSM_Protocol_Message msg, GSM_StateMachin
 			if (DCT4Tests.Tests[j].ID == msg.Buffer[pos+2]) {
 				printf("\"%40s\" : ",DCT4Tests.Tests[j].Name);
 				switch(msg.Buffer[pos+3]) {
-					case 0x00: printf("Passed"); 		break;
-					case 0x01: printf("Fail");   		break;
-					case 0x03: printf("Not executed"); 	break;
-					case 0x06: printf("No signal");		break;
-					case 0x0D: printf("Timeout");		break;
-					default  : printf("Unknown (%x)",msg.Buffer[pos+3]);
+					case 0x00: printf(_("Passed")); 		break;
+					case 0x01: printf(_("Fail"));   		break;
+					case 0x03: printf(_("Not executed")); 	break;
+					case 0x06: printf(_("No signal"));		break;
+					case 0x0D: printf(_("Timeout"));		break;
+					default  : printf(_("Unknown (%x)"),msg.Buffer[pos+3]);
 				}
-				if (DCT4Tests.Tests[j].Startup) printf(" (startup)");
+				if (DCT4Tests.Tests[j].Startup) printf(_(" (startup)"));
 				printf("\n");
 				break;
 			}
@@ -373,6 +375,7 @@ static GSM_Error DCT4_ReplyTestsStatus(GSM_Protocol_Message msg, GSM_StateMachin
 void DCT4SelfTests(int argc, char *argv[])
 {
 	int 	      j;
+	GSM_Error error;
 	unsigned char GetDoneST[6]    = {0x00, 0x08, 0x01, 0x04, 0x01, 0x00};
 	unsigned char GetDoneST2[6]   = {0x00, 0x08, 0x02, 0x04, 0x02, 0x00};
 	unsigned char GetNames[6]     = {0x00, 0x08, 0x03, 0x06, 0x03, 0x00};
@@ -384,25 +387,25 @@ void DCT4SelfTests(int argc, char *argv[])
 
 	if (CheckDCT4Only()!=ERR_NONE) return;
 
-	s.User.UserReplyFunctions=UserReplyFunctions4;
+	s->User.UserReplyFunctions=UserReplyFunctions4;
 
 	if (answer_yes2("Run all tests now ?")) {
-		error=GSM_WaitFor (&s, RunALL, 6, 0x35, 4, ID_User1);
+		error=GSM_WaitFor (s, RunALL, 6, 0x35, 4, ID_User1);
 		Print_Error(error);
 	}
 
-	error=GSM_WaitFor (&s, GetNames, 6, 0x35, 4, ID_User1);
+	error=GSM_WaitFor (s, GetNames, 6, 0x35, 4, ID_User1);
 	Print_Error(error);
 
 	for (j=0;j<DCT4Tests.Num;j++) DCT4Tests.Tests[j].Startup = false;
 
-	error=GSM_WaitFor (&s, GetDoneST, 6, 0x35, 4, ID_User3);
+	error=GSM_WaitFor (s, GetDoneST, 6, 0x35, 4, ID_User3);
 	Print_Error(error);
 
-	error=GSM_WaitFor (&s, GetDoneST2, 6, 0x35, 4, ID_User3);
+	error=GSM_WaitFor (s, GetDoneST2, 6, 0x35, 4, ID_User3);
 	Print_Error(error);
 
-	error=GSM_WaitFor (&s, GetStatus, 6, 0x35, 4, ID_User2);
+	error=GSM_WaitFor (s, GetStatus, 6, 0x35, 4, ID_User2);
 	Print_Error(error);
 }
 
@@ -432,6 +435,7 @@ void DCT4SetVibraLevel(int argc, char *argv[])
 {
 	GSM_DateTime	Date;
 	unsigned int	i,j;
+	GSM_Error error;
 
 	/* Set vibra level */
 	unsigned char 	SetLevel[6] = {N7110_FRAME_HEADER,0x0E,
@@ -442,13 +446,13 @@ void DCT4SetVibraLevel(int argc, char *argv[])
 
         CheckDCT4();
 
-	s.User.UserReplyFunctions=UserReplyFunctions4;
+	s->User.UserReplyFunctions=UserReplyFunctions4;
 
 	SetLevel[4] = atoi(argv[2]);
-	error=GSM_WaitFor (&s, SetLevel, 6, 0x1C, 4, ID_User3);
+	error=GSM_WaitFor (s, SetLevel, 6, 0x1C, 4, ID_User3);
 	Print_Error(error);
 
-	error=DCT4EnableVibra(&s, true);
+	error=DCT4EnableVibra(s, true);
 	Print_Error(error);
 
 	for (i=0;i<3;i++) {
@@ -460,7 +464,7 @@ void DCT4SetVibraLevel(int argc, char *argv[])
 		}
 	}
 
-	error=DCT4EnableVibra(&s, false);
+	error=DCT4EnableVibra(s, false);
 	Print_Error(error);
 
 	GSM_Terminate();
@@ -469,18 +473,19 @@ void DCT4SetVibraLevel(int argc, char *argv[])
 void DCT4VibraTest(int argc, char *argv[])
 {
 	unsigned char ans[200];
+	GSM_Error error;
 
 	if (CheckDCT4Only()!=ERR_NONE) return;
 
-	s.User.UserReplyFunctions=UserReplyFunctions4;
+	s->User.UserReplyFunctions=UserReplyFunctions4;
 
-	error=DCT4EnableVibra(&s, true);
+	error=DCT4EnableVibra(s, true);
 	Print_Error(error);
 
-	printf("Press any key to continue...\n");
+	printf("%s\n", _("Press any key to continue..."));
 	GetLine(stdin, ans, 99);
 
-	error=DCT4EnableVibra(&s, false);
+	error=DCT4EnableVibra(s, false);
 	Print_Error(error);
 }
 
@@ -489,10 +494,10 @@ static GSM_Error DCT4_ReplyResetSecurityCode(GSM_Protocol_Message msg, GSM_State
 {
 	switch (msg.Buffer[3]) {
 	case 0x05:
-		printf("Security code set to \"12345\"\n");
+		printf(_("Security code set to \"12345\"\n"));
 		return ERR_NONE;
 	case 0x06:
-		printf("Unknown reason. Can't reset your security code\n");
+		printf("%s\n", _("Unknown reason. Can't reset your security code"));
 		return ERR_UNKNOWN;
 	}
 	return ERR_UNKNOWNRESPONSE;
@@ -501,6 +506,7 @@ static GSM_Error DCT4_ReplyResetSecurityCode(GSM_Protocol_Message msg, GSM_State
 void DCT4ResetSecurityCode(int argc, char *argv[])
 {
 	unsigned int	i;
+	GSM_Error error;
 	unsigned char 	ResetCode[30] = {0x00,0x06,0x03,0x04,0x01,
 		'1','2','3','4','5','6','7','8','9','0',	/* Old code */
 		0x00,
@@ -509,17 +515,17 @@ void DCT4ResetSecurityCode(int argc, char *argv[])
 
 	if (CheckDCT4Only()!=ERR_NONE) return;
 
-	s.User.UserReplyFunctions=UserReplyFunctions4;
+	s->User.UserReplyFunctions=UserReplyFunctions4;
 
-	error=GSM_WaitFor (&s, ResetCode, 27, 0x08, 4, ID_User2);
+	error=GSM_WaitFor (s, ResetCode, 27, 0x08, 4, ID_User2);
 	if (error == ERR_UNKNOWN) {
 		if (answer_yes2("Try brutal force ?")) {
 			for (i=10000;i<9999999;i++) {
-				printf("Trying %i\n",i);
+				printf(_("Trying %i\n"),i);
 				memset(ResetCode+6,0,22);
 				sprintf(ResetCode+5,"%i",i);
 				sprintf(ResetCode+16,"12345");
-				error=GSM_WaitFor (&s, ResetCode, 27, 0x08, 4, ID_User2);
+				error=GSM_WaitFor (s, ResetCode, 27, 0x08, 4, ID_User2);
 				if (error == ERR_NONE) break;
 			}
 		}
@@ -534,7 +540,7 @@ static GSM_Error DCT4_ReplyGetSecurityCode(GSM_Protocol_Message msg, GSM_StateMa
 	if (msg.Length > 12) {
 		SecLength = msg.Buffer[13];
 		if ((msg.Buffer[17]+18) == msg.Length) {
-			printf("Security code is %s\n",msg.Buffer+18);
+			printf(_("Security code is %s\n"),msg.Buffer+18);
 //			DumpMessage(stdout, msg.Buffer, msg.Length);
 		}
 	}
@@ -557,14 +563,14 @@ void DCT4GetSecurityCode(int argc, char *argv[])
 
 	if (CheckDCT4Only()!=ERR_NONE) return;
 
-	s.User.UserReplyFunctions=UserReplyFunctions4;
+	s->User.UserReplyFunctions=UserReplyFunctions4;
 
 	SecLength = 0;
-	error=GSM_WaitFor (&s, getlen, sizeof(getlen), 0x23, 1, ID_User1);
+	error=GSM_WaitFor (s, getlen, sizeof(getlen), 0x23, 1, ID_User1);
 	Print_Error(error);
 	if (SecLength != 0) {
 		read[17] = SecLength;
-		error=GSM_WaitFor (&s, read, sizeof(read), 0x23, 5, ID_User1);
+		error=GSM_WaitFor (s, read, sizeof(read), 0x23, 5, ID_User1);
 		Print_Error(error);
 	}
 }
@@ -605,7 +611,7 @@ static GSM_Error DCT4_ReplyGetVoiceRecord(GSM_Protocol_Message msg, GSM_StateMac
 			Buffer[msg.Buffer[j]+1] = 0;
 			dbgprintf("%i. \"%s\"\n",i+1,DecodeUnicodeString(Buffer));
  			if (i==*s->Phone.Data.VoiceRecord) {
- 				sprintf(s->Phone.Data.PhoneString,"%s.wav",DecodeUnicodeString(Buffer));
+ 				sprintf(s->Phone.Data.PhoneString,"%s->wav",DecodeUnicodeString(Buffer));
 				return ERR_NONE;
 			}
 			if (i != msg.Buffer[9] - 1) {
@@ -663,10 +669,11 @@ void DCT4GetVoiceRecord(int argc, char *argv[])
 	unsigned int 	Location, size=0, CurrentLocation = 0, TokenLocation;
 	int		i;
 	FILE		*WAVFile;
+	GSM_Error error;
 
 	Location = atoi(argv[2]);
 	if (Location == 0x00) {
-		printf("Please numerate locations from 1\n");
+		printf("%s\n", _("Please enumerate locations from 1"));
 		return;
 	}
 	Location--;
@@ -675,18 +682,18 @@ void DCT4GetVoiceRecord(int argc, char *argv[])
 
         CheckDCT4();
 
-	s.User.UserReplyFunctions=UserReplyFunctions4;
+	s->User.UserReplyFunctions=UserReplyFunctions4;
 
-	s.Phone.Data.VoiceRecord 	= &Location;
-	s.Phone.Data.PhoneString 	= FileName;
+	s->Phone.Data.VoiceRecord 	= &Location;
+	s->Phone.Data.PhoneString 	= FileName;
 	dbgprintf("Getting voice record name\n");
-	error=GSM_WaitFor (&s, ReqNames, 14, 0x4A, 4, ID_User4);
+	error=GSM_WaitFor (s, ReqNames, 14, 0x4A, 4, ID_User4);
 	Print_Error(error);
 
-	s.Phone.Data.PhoneString 	= Buffer;
+	s->Phone.Data.PhoneString 	= Buffer;
 	ReqToken[7] 			= Location;
 	dbgprintf("Getting voice record token\n");
-	error=GSM_WaitFor (&s, ReqToken, 10, 0x23, 4, ID_User4);
+	error=GSM_WaitFor (s, ReqToken, 10, 0x23, 4, ID_User4);
 	Print_Error(error);
 	TokenLocation 			= Buffer[0] * 256 + Buffer[1];
 	Token				= Buffer[2];
@@ -697,14 +704,14 @@ void DCT4GetVoiceRecord(int argc, char *argv[])
 	fwrite(&FMT_Header,	1, sizeof(FMT_Header),	WAVFile);
 	fwrite(&DATA_Header,	1, sizeof(DATA_Header),	WAVFile);
 
-	s.Phone.Data.VoiceRecord 	= &size;
-	s.Phone.Data.PhoneString 	= Buffer;
+	s->Phone.Data.VoiceRecord 	= &size;
+	s->Phone.Data.PhoneString 	= Buffer;
 	ReqGet[7]			= Location;
 	fprintf(stderr,"Getting voice record and saving to \"%s\": ",FileName);
 	while (1) {
 		dbgprintf("Getting next part of voice record\n");
 		fprintf(stderr,".");
-		error=GSM_WaitFor (&s, ReqGet, 18, 0x23, 4, ID_User4);
+		error=GSM_WaitFor (s, ReqGet, 18, 0x23, 4, ID_User4);
 		if (error == ERR_NONE) {
 			wavfilesize += size;
 			fwrite(Buffer,1,size,WAVFile);
@@ -721,7 +728,7 @@ void DCT4GetVoiceRecord(int argc, char *argv[])
 		ReqGet[16] = i;
 		ReqGet[17] = Token;
 		fprintf(stderr,".");
-		error=GSM_WaitFor (&s, ReqGet, 18, 0x23, 4, ID_User4);
+		error=GSM_WaitFor (s, ReqGet, 18, 0x23, 4, ID_User4);
 		if (error == ERR_NONE) {
 			wavfilesize += size;
 			fwrite(Buffer,1,size,WAVFile);
@@ -735,7 +742,7 @@ void DCT4GetVoiceRecord(int argc, char *argv[])
 		ReqGet[11] = CurrentLocation / 256;
 		ReqGet[12] = CurrentLocation % 256;
 		fprintf(stderr,".");
-		error=GSM_WaitFor (&s, ReqGet, 18, 0x23, 4, ID_User4);
+		error=GSM_WaitFor (s, ReqGet, 18, 0x23, 4, ID_User4);
 		if (error == ERR_NONE) {
 			wavfilesize += size;
 			fwrite(Buffer,1,size,WAVFile);
@@ -775,7 +782,7 @@ void DCT4GetVoiceRecord(int argc, char *argv[])
 
 static GSM_Error DCT4_ReplyGetBTInfo(GSM_Protocol_Message msg, GSM_StateMachine *s)
 {
-	printf("device address %02x%02x%02x%02x%02x%02x\n",
+	printf(_("device address %02x%02x%02x%02x%02x%02x\n"),
 		msg.Buffer[9],msg.Buffer[10],msg.Buffer[11],
 		msg.Buffer[12],msg.Buffer[13],msg.Buffer[14]);
 	return ERR_NONE;
@@ -799,7 +806,7 @@ static GSM_Error DCT4_ReplyGetSimlock(GSM_Protocol_Message msg, GSM_StateMachine
 		buff[4] = buff[3];
 		buff[3] = ' ';
 		if (strcmp(DecodeUnicodeString(GSM_GetNetworkName(buff)),"unknown")) {
-			printf("Old simlock   : %s (%s)\n",DecodeUnicodeString(GSM_GetNetworkName(buff)),buff);
+			printf(_("Old simlock   : %s (%s)\n"),DecodeUnicodeString(GSM_GetNetworkName(buff)),buff);
 		}
 
 		dbgprintf("\n");
@@ -817,7 +824,7 @@ static GSM_Error DCT4_ReplyGetSimlock(GSM_Protocol_Message msg, GSM_StateMachine
 				dbgprintf("%02x",msg.Buffer[i]);
 			}
 			dbgprintf("\n");
-			printf("Simlock data  : ");
+			printf(_("Simlock data  : "));
 			for (i=60;i<63;i++) {
 				printf("%02x",msg.Buffer[i]);
 			}
@@ -833,31 +840,32 @@ void DCT4Info(int argc, char *argv[])
 	unsigned char GetBTAddress[8] = {N6110_FRAME_HEADER, 0x09, 0x19, 0x01, 0x03, 0x06};
 	unsigned char GetSimlock[5] = {N6110_FRAME_HEADER, 0x12, 0x0D};
 	unsigned char value[10];
+	GSM_Error error;
 
         if (CheckDCT4Only()!=ERR_NONE) return;
 
-	if (IsPhoneFeatureAvailable(s.Phone.Data.ModelInfo, F_SERIES40_30)) return;
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_SERIES40_30)) return;
 
-	s.User.UserReplyFunctions=UserReplyFunctions4;
+	s->User.UserReplyFunctions=UserReplyFunctions4;
 
-	if (IsPhoneFeatureAvailable(s.Phone.Data.ModelInfo, F_BLUETOOTH)) {
-		printf("Bluetooth     : ");
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_BLUETOOTH)) {
+		printf(_("Bluetooth     : "));
 
-		error=GSM_WaitFor (&s, GetBTAddress, 8, 0xD7, 4, ID_User6);
+		error=GSM_WaitFor (s, GetBTAddress, 8, 0xD7, 4, ID_User6);
 		Print_Error(error);
 	}
 
-	error=GSM_WaitFor (&s, GetSimlock, 5, 0x53, 4, ID_User6);
+	error=GSM_WaitFor (s, GetSimlock, 5, 0x53, 4, ID_User6);
 	Print_Error(error);
 	GetSimlock[4] = 0x0E;
-	error=GSM_WaitFor (&s, GetSimlock, 5, 0x53, 4, ID_User6);
+	error=GSM_WaitFor (s, GetSimlock, 5, 0x53, 4, ID_User6);
 	Print_Error(error);
 	GetSimlock[3] = 0x0C;
-	error=GSM_WaitFor (&s, GetSimlock, 4, 0x53, 4, ID_User6);
+	error=GSM_WaitFor (s, GetSimlock, 4, 0x53, 4, ID_User6);
 	Print_Error(error);
-	error=NOKIA_GetPhoneString(&s,"\x00\x03\x02\x07\x00\x08",6,0x1b,value,ID_User6,10);
+	error=NOKIA_GetPhoneString(s,"\x00\x03\x02\x07\x00\x08",6,0x1b,value,ID_User6,10);
 	Print_Error(error);
-	printf("UEM           : %s\n",value);
+	printf(_("UEM           : %s\n"),value);
 }
 
 static FILE 	*T9File;
@@ -880,13 +888,14 @@ void DCT4GetT9(int argc, char *argv[])
 			       0x00, 0x00,	/* Start position */
 			       0x00, 0x00,
 			       0x02, 0xBC};	/* How many bytes to read */
+	GSM_Error error;
 
 	if (CheckDCT4Only()!=ERR_NONE) return;
 
 	T9File = fopen("T9", "w");
 	if (T9File == NULL) return;
 
-	s.User.UserReplyFunctions=UserReplyFunctions4;
+	s->User.UserReplyFunctions=UserReplyFunctions4;
 
 	i = 0;
 	while (1) {
@@ -899,7 +908,7 @@ void DCT4GetT9(int argc, char *argv[])
 			}
 			if (T9Dictionary - i == 0) break;
 		}
-		error=GSM_WaitFor (&s, req, 18, 0x23, 4, ID_User3);
+		error=GSM_WaitFor (s, req, 18, 0x23, 4, ID_User3);
 		Print_Error(error);
 		if (i==0) {
 			T9Dictionary = T9FullSize;
@@ -920,31 +929,33 @@ void DCT4SetLight(int argc, char *argv[])
 	int			i;
 	N6510_PHONE_LIGHTS 	type;
 	bool			enable;
+	GSM_Error error;
 
-	if (mystrncasecmp(argv[2],"display",0)) { 	type = N6510_LIGHT_DISPLAY;
-	} else if (mystrncasecmp(argv[2],"keypad",0)) {	type = N6510_LIGHT_KEYPAD;
-	} else if (mystrncasecmp(argv[2],"torch",0)) {	type = N6510_LIGHT_TORCH;
+	if (strcasecmp(argv[2],"display") == 0) { 	type = N6510_LIGHT_DISPLAY;
+	} else if (strcasecmp(argv[2],"keypad") == 0) {	type = N6510_LIGHT_KEYPAD;
+	} else if (strcasecmp(argv[2],"torch") == 0) {	type = N6510_LIGHT_TORCH;
 	} else {
-		printf("What lights should I enable (\"%s\") ?\n",argv[2]);
+		printf(_("What lights should I enable (\"%s\") ?\n"),argv[2]);
 		exit(-1);
 	}
 
-	if (mystrncasecmp(argv[3],"on",0)) { 		enable = true;
-	} else if (mystrncasecmp(argv[3],"off",0)) {	enable = false;
+	if (strcasecmp(argv[3],"on") == 0) { 		enable = true;
+	} else if (strcasecmp(argv[3],"off") == 0) {	enable = false;
 	} else {
-		printf("What should I do (\"%s\") ?\n",argv[3]);
+		printf(_("What should I do (\"%s\") ?\n"),argv[3]);
 		exit(-1);
 	}
 
-	for (i=0;i<s.ConfigNum;i++) {
-		s.Config[i].StartInfo = "false";
+	for (i=0;i<s->ConfigNum;i++) {
+		free(s->Config[i].StartInfo);
+		s->Config[i].StartInfo = strdup("false");
 	}
 
 	GSM_Init(true);
 
         CheckDCT4();
 
-	error=N6510_SetLight(&s, type, enable);
+	error=N6510_SetLight(s, type, enable);
 	Print_Error(error);
 
 	GSM_Terminate();
@@ -961,19 +972,19 @@ void DCT4DisplayTest(int argc, char *argv[])
 
 	if (CheckDCT4Only()!=ERR_NONE) return;
 
-	s.User.UserReplyFunctions=UserReplyFunctions4;
+	s->User.UserReplyFunctions=UserReplyFunctions4;
 
-	DCT4_SetPhoneMode(&s, DCT4_MODE_TEST);
+	DCT4_SetPhoneMode(s, DCT4_MODE_TEST);
 
-	s.Protocol.Functions->WriteMessage(&s, req0, 6, 0x40);
+	s->Protocol.Functions->WriteMessage(s, req0, 6, 0x40);
 
 	req[8] = atoi(argv[2]);
-	s.Protocol.Functions->WriteMessage(&s, req, 10, 0x40);
+	s->Protocol.Functions->WriteMessage(s, req, 10, 0x40);
 
-	printf("Press any key to continue...\n");
+	printf("%s\n", _("Press any key to continue..."));
 	GetLine(stdin, ans, 99);
 
-	DCT4_SetPhoneMode(&s, DCT4_MODE_NORMAL);
+	DCT4_SetPhoneMode(s, DCT4_MODE_NORMAL);
 }
 
 int ADC;
@@ -983,11 +994,11 @@ static GSM_Error DCT4_ReplyGetADC(GSM_Protocol_Message msg, GSM_StateMachine *s)
 	if (msg.Buffer[6] == 0xff && msg.Buffer[7] == 0xff) return ERR_NONE;
 	switch (msg.Buffer[3]) {
 	case 0x10:
-		printf("raw ");
+		printf(_("raw "));
 		printf("%10i ",msg.Buffer[8]*256+msg.Buffer[9]);
 		break;
 	case 0x12:
-		printf("unit result ");
+		printf(_("unit result "));
 		printf("%10i ",(msg.Buffer[8]*256+msg.Buffer[9])*ADC);
 		break;
 	}
@@ -1024,6 +1035,7 @@ static struct DCT4ADCInfo DCT4ADC[] = {
 void DCT4GetADC(int argc, char *argv[])
 {
 	int		i = 0;
+	GSM_Error error;
 	unsigned char 	GetRaw[]  = {N6110_FRAME_HEADER, 0x0F,
 				     0x00,		/* Test number */
 			             0x01};
@@ -1033,16 +1045,16 @@ void DCT4GetADC(int argc, char *argv[])
 
 	if (CheckDCT4Only()!=ERR_NONE) return;
 
-	s.User.UserReplyFunctions=UserReplyFunctions4;
+	s->User.UserReplyFunctions=UserReplyFunctions4;
 
 	while (1) {
 		printf(" %30s ",DCT4ADC[i].name);
 		GetRaw[4] = i;
-		error=GSM_WaitFor (&s, GetRaw, 6, 0x17, 4, ID_User3);
+		error=GSM_WaitFor (s, GetRaw, 6, 0x17, 4, ID_User3);
 		Print_Error(error);
 		GetUnit[4] 	= i;
 		ADC		= DCT4ADC[i].x;
-		error=GSM_WaitFor (&s, GetUnit, 6, 0x17, 4, ID_User3);
+		error=GSM_WaitFor (s, GetUnit, 6, 0x17, 4, ID_User3);
 		Print_Error(error);
 		printf("%s\n",DCT4ADC[i].unit);
 		i++;
@@ -1087,6 +1099,7 @@ static GSM_Error DCT4_ReplyTuneRadio(GSM_Protocol_Message msg, GSM_StateMachine
 void DCT4TuneRadio(int argc, char *argv[])
 {
 	double		Freq, diff;
+	GSM_Error error;
  	GSM_FMStation 	FMStation[50],FMStat;
 	int		i, j, num;
 	bool		found;
@@ -1115,19 +1128,19 @@ void DCT4TuneRadio(int argc, char *argv[])
 
         CheckDCT4();
 
-	s.User.UserReplyFunctions=UserReplyFunctions4;
+	s->User.UserReplyFunctions=UserReplyFunctions4;
 
 	FMStat.Location = 1;
-	error = Phone->GetFMStation(&s,&FMStat);
+	error = GSM_GetFMStation(s,&FMStat);
 	if (error != ERR_NONE && error != ERR_EMPTY) {
-		printf("Phone seems not to support radio\n");
+		printf("%s\n", _("Phone seems not to support radio"));
 		GSM_Terminate();
 		exit(-1);
 	}
 
-	error=GSM_WaitFor (&s, Enable, 6, 0x3E, 4, ID_User3);
+	error=GSM_WaitFor (s, Enable, 6, 0x3E, 4, ID_User3);
 	if (error == ERR_PERMISSION) {
-		printf("Please connect headset. Required as antenna\n");
+		printf("%s\n", _("Please connect headset. Required as antenna"));
 		GSM_Terminate();
 		exit(-1);
 	}
@@ -1138,10 +1151,10 @@ void DCT4TuneRadio(int argc, char *argv[])
 		fprintf(stderr,"%cSearching: %i percent",13,(i-88)*100/(108-88));
 		Freq = i;
 		N6510_EncodeFMFrequency(Freq, SetFreq+8);
-		error=GSM_WaitFor (&s, SetFreq, 10, 0x3E, 4, ID_User3);
+		error=GSM_WaitFor (s, SetFreq, 10, 0x3E, 4, ID_User3);
 		Print_Error(error);
 
-		error=GSM_WaitFor (&s, Find2, 10, 0x3E, 4, ID_User3);
+		error=GSM_WaitFor (s, Find2, 10, 0x3E, 4, ID_User3);
 		Print_Error(error);
 		found = false;
 		for (j=0;j<num;j++) {
@@ -1188,19 +1201,19 @@ void DCT4TuneRadio(int argc, char *argv[])
 
 	if (answer_yes2("Do you want to save found stations")) {
 		fprintf(stderr,"Deleting old FM stations: ");
-		error=Phone->ClearFMStations(&s);
+		error=GSM_ClearFMStations(s);
 		Print_Error(error);
 		fprintf(stderr,"Done\n");
 		for (i=0;i<num;i++) {
 			FMStation[i].Location = i+1;
-			error=Phone->SetFMStation(&s,&FMStation[i]);
+			error=GSM_SetFMStation(s,&FMStation[i]);
 			Print_Error(error);
 			fprintf(stderr,"%cWriting: %i percent",13,(i+1)*100/num);
 		}
 		fprintf(stderr,"\n");
 	}
 
-	error=GSM_WaitFor (&s, Disable, 6, 0x3E, 4, ID_User3);
+	error=GSM_WaitFor (s, Disable, 6, 0x3E, 4, ID_User3);
 	Print_Error(error);
 
 	GSM_Terminate();
@@ -1217,15 +1230,16 @@ void DCT4PlaySavedRingtone(int argc, char *argv[])
 				0x0A,		//volume
 				0x00,0x00,0x00,0x00,0x00,0x00,0x00};
 	GSM_AllRingtonesInfo Info;
+	GSM_Error error;
 //	int i;
 
 	GSM_Init(true);
 
         CheckDCT4();
 
-	s.User.UserReplyFunctions=UserReplyFunctions4;
+	s->User.UserReplyFunctions=UserReplyFunctions4;
 
-	error=Phone->GetRingtonesInfo(&s,&Info);
+	error=GSM_GetRingtonesInfo(s,&Info);
 	Print_Error(error);
 
 	if (atoi(argv[2]) > Info.Number-1) {
@@ -1236,10 +1250,10 @@ void DCT4PlaySavedRingtone(int argc, char *argv[])
 	req[5] = Info.Ringtone[atoi(argv[2])-1].ID % 256;
 	req[6] = Info.Ringtone[atoi(argv[2])-1].Group;
 
-	error=GSM_WaitFor (&s, req, 18, 0x1F, 4, ID_User3);
+	error=GSM_WaitFor (s, req, 18, 0x1F, 4, ID_User3);
 	Print_Error(error);
 
-//	for (i=0;i<Info.Number;i++) printmsg("%i. \"%s\"\n",i,DecodeUnicodeConsole(Info.Ringtone[i].Name));
+//	for (i=0;i<Info.Number;i++) printf(_("%i. \")%s\"\n",i,DecodeUnicodeConsole(Info.Ringtone[i].Name));
 
 	GSM_Terminate();
 }
@@ -1261,27 +1275,28 @@ void DCT4MakeCameraShoot(int argc, char *argv[])
 		0x00, 0x02,	//master folder id
 		0x00, 0x14}; 	//length
 	unsigned char CameraOFF[] = {N6110_FRAME_HEADER, 0x04, 0x01, 0x00};
+	GSM_Error error;
 
 	GSM_Init(true);
 
         CheckDCT4();
 
-	s.User.UserReplyFunctions=UserReplyFunctions4;
+	s->User.UserReplyFunctions=UserReplyFunctions4;
 
-	error=GSM_WaitFor (&s, SetCamera, 6, 0x61, 4, ID_User3);
+	error=GSM_WaitFor (s, SetCamera, 6, 0x61, 4, ID_User3);
 	Print_Error(error);
-	error=GSM_WaitFor (&s, CameraON, 10, 0x61, 4, ID_User3);
+	error=GSM_WaitFor (s, CameraON, 10, 0x61, 4, ID_User3);
 	Print_Error(error);
-	error=GSM_WaitFor (&s, CameraON2, 6, 0x61, 4, ID_User3);
+	error=GSM_WaitFor (s, CameraON2, 6, 0x61, 4, ID_User3);
 	Print_Error(error);
 	EncodeUnicode(MakeShot+24,"GammuShot",9);
 	MakeShot[15] = 9+9*2;
 	MakeShot[23] = 9*2;
-	error=GSM_WaitFor (&s, MakeShot, 24+MakeShot[23], 0x61, 4, ID_User3);
+	error=GSM_WaitFor (s, MakeShot, 24+MakeShot[23], 0x61, 4, ID_User3);
 	Print_Error(error);
-	error=GSM_WaitFor (&s, SetCamera, 6, 0x61, 4, ID_User3);
+	error=GSM_WaitFor (s, SetCamera, 6, 0x61, 4, ID_User3);
 	Print_Error(error);
-	error=GSM_WaitFor (&s, CameraOFF, 6, 0x61, 4, ID_User3);
+	error=GSM_WaitFor (s, CameraOFF, 6, 0x61, 4, ID_User3);
 	Print_Error(error);
 
 	GSM_Terminate();
@@ -1298,18 +1313,19 @@ static GSM_Error DCT4_ReplyGetScreenDump(GSM_Protocol_Message msg, GSM_StateMach
 void DCT4GetScreenDump(int argc, char *argv[])
 {
 	unsigned char req[] = {N6110_FRAME_HEADER, 0x07, 0x01, 0x00};
+	GSM_Error error;
 	//n6110_frameheader 06//screen info
 
 	GSM_Init(true);
 
         CheckDCT4();
 
-	s.User.UserReplyFunctions=UserReplyFunctions4;
+	s->User.UserReplyFunctions=UserReplyFunctions4;
 
-	error=GSM_WaitFor (&s, req, 6, 0x0E, 4, ID_User3);
+	error=GSM_WaitFor (s, req, 6, 0x0E, 4, ID_User3);
 	Print_Error(error);
 	len = 2000;
-	while (len >= 200) GSM_ReadDevice(&s,true);
+	while (len >= 200) GSM_ReadDevice(s,true);
 
 	GSM_Terminate();
 }
@@ -1318,59 +1334,59 @@ static GSM_Error DCT4_ReplyGetPBKFeatures(GSM_Protocol_Message msg, GSM_StateMac
 {
 	int i,pos=6;
 
-	printf("%i entries types\n",msg.Buffer[5]-1);
+	printf(_("%i entries types\n"),msg.Buffer[5]-1);
 
 	for (i=0;i<msg.Buffer[5]-1;i++) {
-		printf("  entry ID %02X",msg.Buffer[pos+4]);
+		printf(_("  entry ID %02X"),msg.Buffer[pos+4]);
 		switch (msg.Buffer[pos+4]) {
-		case N7110_PBK_SIM_SPEEDDIAL	: printf(" (Speed dial on SIM)"); 			break;
-		case N7110_PBK_NAME	    	: printf(" (Text: name (always the only one)");		break;
-		case N7110_PBK_EMAIL	    	: printf(" (Text: email adress)");			break;
-		case N7110_PBK_POSTAL	    	: printf(" (Text: postal address)");			break;
-		case N7110_PBK_NOTE	    	: printf(" (Text: note)");				break;
-		case N7110_PBK_NUMBER 	    	: printf(" (Phone number)");				break;
-		case N7110_PBK_RINGTONE_ID  	: printf(" (Ringtone ID)");				break;
-		case N7110_PBK_DATETIME    	: printf(" (Call register: date and time)");		break;
-		case N7110_PBK_UNKNOWN1	    	: printf(" (Call register: with missed calls)");	break;
-		case N7110_PBK_SPEEDDIAL    	: printf(" (Speed dial)");				break;
-		case N7110_PBK_GROUPLOGO    	: printf(" (Caller group: logo)");			break;
-		case N7110_PBK_LOGOON	    	: printf(" (Caller group: is logo on ?)");		break;
-		case N7110_PBK_GROUP	    	: printf(" (Caller group number in pbk entry)");	break;
-	
+		case N7110_PBK_SIM_SPEEDDIAL	: printf(_(" (Speed dial on SIM)")); 			break;
+		case N7110_PBK_NAME	    	: printf(_(" (Text: name (always the only one)"));		break;
+		case N7110_PBK_EMAIL	    	: printf(_(" (Text: email adress)"));			break;
+		case N7110_PBK_POSTAL	    	: printf(_(" (Text: postal address)"));			break;
+		case N7110_PBK_NOTE	    	: printf(_(" (Text: note)"));				break;
+		case N7110_PBK_NUMBER 	    	: printf(_(" (Phone number)"));				break;
+		case N7110_PBK_RINGTONE_ID  	: printf(_(" (Ringtone ID)"));				break;
+		case N7110_PBK_DATETIME    	: printf(_(" (Call register: date and time)"));		break;
+		case N7110_PBK_UNKNOWN1	    	: printf(_(" (Call register: with missed calls)"));	break;
+		case N7110_PBK_SPEEDDIAL    	: printf(_(" (Speed dial)"));				break;
+		case N7110_PBK_GROUPLOGO    	: printf(_(" (Caller group: logo)"));			break;
+		case N7110_PBK_LOGOON	    	: printf(_(" (Caller group: is logo on ?)"));		break;
+		case N7110_PBK_GROUP	    	: printf(_(" (Caller group number in pbk entry)"));	break;
+
 		/* DCT4 only */
-		case N6510_PBK_URL		: printf(" (Text: URL address)");			break;
-		case N6510_PBK_SMSLIST_ID	: printf(" (SMS list assigment)");			break;
-		case N6510_PBK_VOICETAG_ID	: printf(" (Voice tag assigment)");			break;
-		case N6510_PBK_PICTURE_ID	: printf(" (Picture ID assigment)");			break;
-		case N6510_PBK_RINGTONEFILE_ID  : printf(" (Ringtone ID from filesystem/internal)");	break;
-		case N6510_PBK_USER_ID          : printf(" (Text: user ID)");				break;
-		case N6510_PBK_UNKNOWN2	        : printf(" (conversation list ID)");			break;
-		case N6510_PBK_UNKNOWN3	 	: printf(" (Instant Messaging service list ID ?)");	break;
-		case N6510_PBK_UNKNOWN4	 	: printf(" (presence list ID ?)");			break;
-		case N6510_PBK_PUSHTOTALK_ID	: printf(" (SIP Address (Push to Talk address))");	break;
-		case N6510_PBK_GROUP2_ID	: printf(" (Group ID (6230i or later))");		break;
+		case N6510_PBK_URL		: printf(_(" (Text: URL address)"));			break;
+		case N6510_PBK_SMSLIST_ID	: printf(_(" (SMS list assigment)"));			break;
+		case N6510_PBK_VOICETAG_ID	: printf(_(" (Voice tag assigment)"));			break;
+		case N6510_PBK_PICTURE_ID	: printf(_(" (Picture ID assigment)"));			break;
+		case N6510_PBK_RINGTONEFILE_ID  : printf(_(" (Ringtone ID from filesystem/internal)"));	break;
+		case N6510_PBK_USER_ID          : printf(_(" (Text: user ID)"));				break;
+		case N6510_PBK_UNKNOWN2	        : printf(_(" (conversation list ID)"));			break;
+		case N6510_PBK_UNKNOWN3	 	: printf(_(" (Instant Messaging service list ID ?)"));	break;
+		case N6510_PBK_UNKNOWN4	 	: printf(_(" (presence list ID ?)"));			break;
+		case N6510_PBK_PUSHTOTALK_ID	: printf(_(" (SIP Address (Push to Talk address))"));	break;
+		case N6510_PBK_GROUP2_ID	: printf(_(" (Group ID (6230i or later))"));		break;
 		}
-		printf(", type ");
+		printf(_(", type "));
 		switch (msg.Buffer[pos+5]) {
-		case 0x05: printf("string"); 	break;
-		case 0x02: printf("byte"); 	break;
-		case 0x03: printf("2 bytes"); 	break;
-		case 0x06: printf("4 bytes"); 	break;
+		case 0x05: printf(_("string")); 	break;
+		case 0x02: printf(_("byte")); 	break;
+		case 0x03: printf(_("2 bytes")); 	break;
+		case 0x06: printf(_("4 bytes")); 	break;
 		default  : printf("%02X",msg.Buffer[pos+5]);
 		}
 		printf("\n");
 		pos+=msg.Buffer[pos+3];
 	}
 
-	printf("%i phone number types\n",msg.Buffer[pos+4]);
+	printf(_("%i phone number types\n"),msg.Buffer[pos+4]);
 	for (i=0;i<msg.Buffer[pos+4];i++) {
 		switch (msg.Buffer[pos+5+i]) {
-		case 0x02: printf("  Home Number\n"); 		break;
-		case 0x03: printf("  Mobile Number\n"); 	break;
-		case 0x04: printf("  Fax Number\n"); 		break;
-		case 0x06: printf("  Office Number\n"); 	break;
-		case 0x0A: printf("  Standard Number\n"); 	break;
-		default:   printf("  unknown\n");
+		case 0x02: printf("  %s\n", _("Home number")); 		break;
+		case 0x03: printf("  %s\n", _("Mobile number")); 	break;
+		case 0x04: printf("  %s\n", _("Fax number")); 		break;
+		case 0x06: printf("  %s\n", _("Office number")); 	break;
+		case 0x0A: printf("  %s\n", _("Standard number")); 	break;
+		default:   printf("  %s\n", _("Unknown number"));
 		}
 	}
 
@@ -1380,40 +1396,41 @@ static GSM_Error DCT4_ReplyGetPBKFeatures(GSM_Protocol_Message msg, GSM_StateMac
 void DCT4GetPBKFeatures(int argc, char *argv[])
 {
 	GSM_MemoryType	MemoryType=0;
-	unsigned char 	req[] = {N6110_FRAME_HEADER, 0x25, 
+	GSM_Error error;
+	unsigned char 	req[] = {N6110_FRAME_HEADER, 0x25,
 				 0x05, 	// memory type
 				 0x00};
 
-	if (mystrncasecmp(argv[2],"DC",0)) MemoryType=MEM_DC;
-	if (mystrncasecmp(argv[2],"ON",0)) MemoryType=MEM_ON;
-	if (mystrncasecmp(argv[2],"RC",0)) MemoryType=MEM_RC;
-	if (mystrncasecmp(argv[2],"MC",0)) MemoryType=MEM_MC;
-	if (mystrncasecmp(argv[2],"ME",0)) MemoryType=MEM_ME;
-	if (mystrncasecmp(argv[2],"SM",0)) MemoryType=MEM_SM;
-	if (mystrncasecmp(argv[2],"VM",0)) MemoryType=MEM_VM;
-	if (mystrncasecmp(argv[2],"FD",0)) MemoryType=MEM_FD;
+	if (strcasecmp(argv[2],"DC") == 0) MemoryType=MEM_DC;
+	if (strcasecmp(argv[2],"ON") == 0) MemoryType=MEM_ON;
+	if (strcasecmp(argv[2],"RC") == 0) MemoryType=MEM_RC;
+	if (strcasecmp(argv[2],"MC") == 0) MemoryType=MEM_MC;
+	if (strcasecmp(argv[2],"ME") == 0) MemoryType=MEM_ME;
+	if (strcasecmp(argv[2],"SM") == 0) MemoryType=MEM_SM;
+	if (strcasecmp(argv[2],"VM") == 0) MemoryType=MEM_VM;
+	if (strcasecmp(argv[2],"FD") == 0) MemoryType=MEM_FD;
 	if (MemoryType==0) {
-		printf("ERROR: unknown memory type (\"%s\")\n",argv[2]);
+		printf(_("ERROR: unknown memory type (\"%s\")\n"),argv[2]);
 		exit (-1);
 	}
 
-	req[4] = NOKIA_GetMemoryType(&s, MemoryType,N71_65_MEMORY_TYPES);
+	req[4] = NOKIA_GetMemoryType(s, MemoryType,N71_65_MEMORY_TYPES);
 	if (req[4]==0xff) exit(-1);
 
 	GSM_Init(true);
 
         CheckDCT4();
 
-	s.User.UserReplyFunctions=UserReplyFunctions4;
+	s->User.UserReplyFunctions=UserReplyFunctions4;
 
-	error=GSM_WaitFor (&s, req, 6, 0x03, 4, ID_User3);
+	error=GSM_WaitFor (s, req, 6, 0x03, 4, ID_User3);
 	Print_Error(error);
 
 	GSM_Terminate();
 }
 
 GSM_Reply_Function UserReplyFunctions4[] = {
-	{DCT4_ReplyGetPBKFeatures,	"\x03",0x03,0x26,ID_User3	},	
+	{DCT4_ReplyGetPBKFeatures,	"\x03",0x03,0x26,ID_User3	},
 
 #ifdef DEBUG
 	{DCT4_ReplyResetSecurityCode,	"\x08",0x03,0x05,ID_User2	},
