@@ -1,6 +1,6 @@
 /* (c) 2004 by Marcin Wiacek */
 
-#include "../../cfg/config.h"
+#include <gammu.h>
 
 #ifdef HAVE_MYSQL_MYSQL_H
 
@@ -8,15 +8,18 @@
 #include <stdio.h>
 #include <errno.h>
 #include <time.h>
+#include <stdlib.h>
+
 #ifdef WIN32
 #  include <windows.h>
+#ifndef __GNUC__
 #  pragma comment(lib, "libmysql.lib")
 #endif
+#endif
 
-#include "../../common/misc/coding/coding.h"
-#include "../../common/service/backup/gsmback.h"
 #include "../gammu.h"
 #include "smsdcore.h"
+#include "../../common/misc/locales.h"
 
 /* Connects to database */
 static GSM_Error SMSDMySQL_Init(GSM_SMSDConfig *Config)
@@ -34,90 +37,91 @@ static GSM_Error SMSDMySQL_Init(GSM_SMSDConfig *Config)
 		port = atoi( pport );
 	}
 
-	mysql_init(&Config->DB);
-	if (!mysql_real_connect(&Config->DB,Config->PC,Config->user,Config->password,Config->database,port,NULL,0)) {
-	    	WriteSMSDLog("Error connecting to database: %s\n", mysql_error(&Config->DB));
+	mysql_init(&Config->DBConnMySQL);
+	if (!mysql_real_connect(&Config->DBConnMySQL,Config->PC,Config->user,Config->password,Config->database,port,NULL,0)) {
+	    	WriteSMSDLog(_("Error connecting to database: %s\n"), mysql_error(&Config->DBConnMySQL));
 	    	return ERR_UNKNOWN;
 	}
 	sprintf(buf, "SELECT ID FROM `outbox` WHERE 1");
-	if (mysql_real_query(&Config->DB,buf,strlen(buf))) {
-		WriteSMSDLog("No table for outbox sms: %s\n", mysql_error(&Config->DB));
+	if (mysql_real_query(&Config->DBConnMySQL,buf,strlen(buf))) {
+		WriteSMSDLog(_("No table for outbox sms: %s\n"), mysql_error(&Config->DBConnMySQL));
 		return ERR_UNKNOWN;
 	}
-	if (!(Res = mysql_store_result(&Config->DB))) {
-		WriteSMSDLog("No table for outbox sms: %s\n", mysql_error(&Config->DB));
+	if (!(Res = mysql_store_result(&Config->DBConnMySQL))) {
+		WriteSMSDLog(_("No table for outbox sms: %s\n"), mysql_error(&Config->DBConnMySQL));
 		return ERR_UNKNOWN;
 	}
 	mysql_free_result(Res);
 	sprintf(buf, "SELECT ID FROM `outbox_multipart` WHERE 1");
-	if (mysql_real_query(&Config->DB,buf,strlen(buf))) {
-		WriteSMSDLog("No table for outbox sms: %s\n", mysql_error(&Config->DB));
+	if (mysql_real_query(&Config->DBConnMySQL,buf,strlen(buf))) {
+		WriteSMSDLog(_("No table for outbox sms: %s\n"), mysql_error(&Config->DBConnMySQL));
 		return ERR_UNKNOWN;
 	}
-	if (!(Res = mysql_store_result(&Config->DB))) {
-		WriteSMSDLog("No table for outbox sms: %s\n", mysql_error(&Config->DB));
+	if (!(Res = mysql_store_result(&Config->DBConnMySQL))) {
+		WriteSMSDLog(_("No table for outbox sms: %s\n"), mysql_error(&Config->DBConnMySQL));
 		return ERR_UNKNOWN;
 	}
 	mysql_free_result(Res);
 	sprintf(buf, "SELECT ID FROM `sentitems` WHERE 1");
-	if (mysql_real_query(&Config->DB,buf,strlen(buf))) {
-		WriteSMSDLog("No table for sent sms: %s\n", mysql_error(&Config->DB));
+	if (mysql_real_query(&Config->DBConnMySQL,buf,strlen(buf))) {
+		WriteSMSDLog(_("No table for sent sms: %s\n"), mysql_error(&Config->DBConnMySQL));
 		return ERR_UNKNOWN;
 	}
-	if (!(Res = mysql_store_result(&Config->DB))) {
-		WriteSMSDLog("No table for sent sms: %s\n", mysql_error(&Config->DB));
+	if (!(Res = mysql_store_result(&Config->DBConnMySQL))) {
+		WriteSMSDLog(_("No table for sent sms: %s\n"), mysql_error(&Config->DBConnMySQL));
 		return ERR_UNKNOWN;
 	}
 	mysql_free_result(Res);
 	sprintf(buf, "SELECT ID FROM `inbox` WHERE 1");
-	if (mysql_real_query(&Config->DB,buf,strlen(buf))) {
-		WriteSMSDLog("No table for inbox sms: %s\n", mysql_error(&Config->DB));
+	if (mysql_real_query(&Config->DBConnMySQL,buf,strlen(buf))) {
+		WriteSMSDLog(_("No table for inbox sms: %s\n"), mysql_error(&Config->DBConnMySQL));
 		return ERR_UNKNOWN;
 	}
-	if (!(Res = mysql_store_result(&Config->DB))) {
-		WriteSMSDLog("No table for inbox sms: %s\n", mysql_error(&Config->DB));
+	if (!(Res = mysql_store_result(&Config->DBConnMySQL))) {
+		WriteSMSDLog(_("No table for inbox sms: %s\n"), mysql_error(&Config->DBConnMySQL));
 		return ERR_UNKNOWN;
 	}
 	mysql_free_result(Res);
 	sprintf(buf, "SELECT Version FROM `gammu` WHERE 1");
-	if (mysql_real_query(&Config->DB,buf,strlen(buf))) {
-		WriteSMSDLog("No Gammu table: %s\n", mysql_error(&Config->DB));
+	if (mysql_real_query(&Config->DBConnMySQL,buf,strlen(buf))) {
+		WriteSMSDLog(_("No Gammu table: %s\n"), mysql_error(&Config->DBConnMySQL));
 		return ERR_UNKNOWN;
 	}
-	if (!(Res = mysql_store_result(&Config->DB))) {
-		WriteSMSDLog("No Gammu table: %s\n", mysql_error(&Config->DB));
+	if (!(Res = mysql_store_result(&Config->DBConnMySQL))) {
+		WriteSMSDLog(_("No Gammu table: %s\n"), mysql_error(&Config->DBConnMySQL));
 		return ERR_UNKNOWN;
 	}
 	if (!(Row = mysql_fetch_row(Res))) {
 		mysql_free_result(Res);
-		WriteSMSDLog("No version info in Gammu table: %s\n", mysql_error(&Config->DB));
+		WriteSMSDLog(_("No version info in Gammu table: %s\n"), mysql_error(&Config->DBConnMySQL));
 		return ERR_UNKNOWN;
 	}
 	if (atoi(Row[0]) > 7) {
 		mysql_free_result(Res);
-		WriteSMSDLog("DataBase structures are from higher Gammu version");
-		WriteSMSDLog("Please update this client application");
+		WriteSMSDLog(_("DataBase structures are from higher Gammu version"));
+		WriteSMSDLog(_("Please update this client application"));
 		return ERR_UNKNOWN;
 	}
 	if (atoi(Row[0]) < 7) {
 		mysql_free_result(Res);
-		WriteSMSDLog("DataBase structures are from older Gammu version");
-		WriteSMSDLog("Please update DataBase, if you want to use this client application");
+		WriteSMSDLog(_("DataBase structures are from older Gammu version"));
+		WriteSMSDLog(_("Please update DataBase, if you want to use this client application"));
 		return ERR_UNKNOWN;
 	}
 	mysql_free_result(Res);
-	mysql_query(&Config->DB,"SET NAMES utf8;");
+	mysql_query(&Config->DBConnMySQL,"SET NAMES utf8;");
 	return ERR_NONE;
 }
 
 static GSM_Error SMSDMySQL_InitAfterConnect(GSM_SMSDConfig *Config)
 {
-	unsigned char buf[400],buf2[200];
+	unsigned char buf[400],buf2[200],imei[100];
 
-	sprintf(buf,"DELETE FROM `phones` WHERE `IMEI` = '%s'",s.Phone.Data.IMEI);
+	GSM_GetIMEI(s, imei);
+	sprintf(buf,"DELETE FROM `phones` WHERE `IMEI` = '%s'", imei);
 	dbgprintf("%s\n",buf);
-	if (mysql_real_query(&Config->DB,buf,strlen(buf))) {
-		WriteSMSDLog("Error deleting from database (Init): %d %s\n", mysql_errno(&Config->DB), mysql_error(&Config->DB));
+	if (mysql_real_query(&Config->DBConnMySQL,buf,strlen(buf))) {
+		WriteSMSDLog(_("Error deleting from database (Init): %d %s\n"), mysql_errno(&Config->DBConnMySQL), mysql_error(&Config->DBConnMySQL));
 		return ERR_UNKNOWN;
 	}
 
@@ -131,10 +135,10 @@ static GSM_Error SMSDMySQL_InitAfterConnect(GSM_SMSDConfig *Config)
 		strcat(buf2+strlen(buf2),GetCompiler());
 	}
 
-	sprintf(buf,"INSERT INTO `phones` (`IMEI`,`ID`,`Send`,`Receive`,`InsertIntoDB`,`TimeOut`,`Client`) VALUES ('%s','%s','yes','yes',NOW(),(NOW() + INTERVAL 10 SECOND)+0,'%s')",s.Phone.Data.IMEI,Config->PhoneID,buf2);
+	sprintf(buf,"INSERT INTO `phones` (`IMEI`,`ID`,`Send`,`Receive`,`InsertIntoDB`,`TimeOut`,`Client`) VALUES ('%s','%s','yes','yes',NOW(),(NOW() + INTERVAL 10 SECOND)+0,'%s')",imei,Config->PhoneID,buf2);
 	dbgprintf("%s\n",buf);
-	if (mysql_real_query(&Config->DB,buf,strlen(buf))) {
-		WriteSMSDLog("Error inserting into database (Init): %d %s\n", mysql_errno(&Config->DB), mysql_error(&Config->DB));
+	if (mysql_real_query(&Config->DBConnMySQL,buf,strlen(buf))) {
+		WriteSMSDLog(_("Error inserting into database (Init): %d %s\n"), mysql_errno(&Config->DBConnMySQL), mysql_error(&Config->DBConnMySQL));
 		return ERR_UNKNOWN;
 	}
 
@@ -148,7 +152,6 @@ static GSM_Error SMSDMySQL_SaveInboxSMS(GSM_MultiSMSMessage sms, GSM_SMSDConfig
 	MYSQL_ROW 		Row;
 	unsigned char		buffer[10000],buffer2[400],buffer3[50],buffer4[800];
 	int 			i;
-	GSM_DateTime		DT;
 	time_t     		t_time1,t_time2;
 	bool			found;
 	long 			diff;
@@ -156,21 +159,21 @@ static GSM_Error SMSDMySQL_SaveInboxSMS(GSM_MultiSMSMessage sms, GSM_SMSDConfig
 	for (i=0;i<sms.Number;i++) {
 		if (sms.SMS[i].PDU == SMS_Status_Report) {
 			strcpy(buffer2, DecodeUnicodeString(sms.SMS[i].Number));
-			if (mystrncasecmp(Config->deliveryreport, "log", 3)) {
-				WriteSMSDLog("Delivery report: %s to %s", DecodeUnicodeString(sms.SMS[i].Text), buffer2);
+			if (strncasecmp(Config->deliveryreport, "log", 3) == 0) {
+				WriteSMSDLog(_("Delivery report: %s to %s"), DecodeUnicodeString(sms.SMS[i].Text), buffer2);
 			}
 
-			sprintf(buffer, "SELECT ID,Status,SendingDateTime,DeliveryDateTime,SMSCNumber FROM `sentitems` WHERE \
+			sprintf(buffer, "SELECT ID,Status,UNIX_TIMESTAMP(SendingDateTime),DeliveryDateTime,SMSCNumber FROM `sentitems` WHERE \
 					DeliveryDateTime='0000-00-00 00:00:00' AND \
 					SenderID='%s' AND TPMR='%i' AND DestinationNumber='%s'",
 					Config->PhoneID, sms.SMS[i].MessageReference, buffer2);
 			dbgprintf("%s\n",buffer);
-			if (mysql_real_query(&Config->DB,buffer,strlen(buffer))) {
-				WriteSMSDLog("Error reading from database (SaveInbox): %s %s\n", buffer, mysql_error(&Config->DB));
+			if (mysql_real_query(&Config->DBConnMySQL,buffer,strlen(buffer))) {
+				WriteSMSDLog(_("Error reading from database (SaveInbox): %s %s\n"), buffer, mysql_error(&Config->DBConnMySQL));
 				return ERR_UNKNOWN;
 			}
-			if (!(Res = mysql_store_result(&Config->DB))) {
-				WriteSMSDLog("Error reading from database (SaveInbox): %s\n", mysql_error(&Config->DB));
+			if (!(Res = mysql_store_result(&Config->DBConnMySQL))) {
+				WriteSMSDLog(_("Error reading from database (SaveInbox): %s\n"), mysql_error(&Config->DBConnMySQL));
 				return ERR_UNKNOWN;
 			}
 			found = false;
@@ -180,32 +183,18 @@ static GSM_Error SMSDMySQL_SaveInboxSMS(GSM_MultiSMSMessage sms, GSM_SMSDConfig
 					if (strcmp(Config->skipsmscnumber,Row[4])) continue;
 				}
 				if (!strcmp(Row[1],"SendingOK") || !strcmp(Row[1],"DeliveryPending")) {
-					sprintf(buffer,"%c%c%c%c",Row[2][0],Row[2][1],Row[2][2],Row[2][3]);
-					DT.Year = atoi(buffer);
-					sprintf(buffer,"%c%c",Row[2][5],Row[2][6]);
-					DT.Month = atoi(buffer);
-					sprintf(buffer,"%c%c",Row[2][8],Row[2][9]);
-					DT.Day = atoi(buffer);
-					sprintf(buffer,"%c%c",Row[2][11],Row[2][12]);
-					DT.Hour = atoi(buffer);
-					sprintf(buffer,"%c%c",Row[2][14],Row[2][15]);
-					DT.Minute = atoi(buffer);
-					sprintf(buffer,"%c%c",Row[2][17],Row[2][18]);
-					DT.Second = atoi(buffer);
-					t_time1 = Fill_Time_T(DT);
+					t_time1 = atoi(Row[2]);
 					t_time2 = Fill_Time_T(sms.SMS[i].DateTime);
 					diff = t_time2 - t_time1;
-	//				fprintf(stderr,"diff is %i, %i-%i-%i-%i-%i and %i-%i-%i-%i-%i-%i\n",diff,
-	//					DT.Year,DT.Month,DT.Day,DT.Hour,DT.Minute,DT.Second,
-	//					sms.SMS[i].DateTime.Year,sms.SMS[i].DateTime.Month,sms.SMS[i].DateTime.Day,sms.SMS[i].DateTime.Hour,sms.SMS[i].DateTime.Minute,sms.SMS[i].DateTime.Second);
-					if (diff > -10 && diff < 10) {
+
+					if (diff > -Config->deliveryreportdelay && diff < Config->deliveryreportdelay) {
 						found = true;
 						break;
 					}
 				}
 			}
 			if (found) {
-				sprintf(buffer,"UPDATE `sentitems` SET `DeliveryDateTime`='%04i%02i%02i%02i%02i%02i', `Status`='",
+				sprintf(buffer,"UPDATE `sentitems` SET `DeliveryDateTime`='%04i-%02i-%02i %02i:%02i:%02i', `Status`='",
 					sms.SMS[i].SMSCTime.Year,sms.SMS[i].SMSCTime.Month,sms.SMS[i].SMSCTime.Day,
 					sms.SMS[i].SMSCTime.Hour,sms.SMS[i].SMSCTime.Minute,sms.SMS[i].SMSCTime.Second);
 				sprintf(buffer3,"%s",DecodeUnicodeString(sms.SMS[i].Text));
@@ -221,8 +210,8 @@ static GSM_Error SMSDMySQL_SaveInboxSMS(GSM_MultiSMSMessage sms, GSM_SMSDConfig
 				sprintf(buffer+strlen(buffer),"', `StatusError` = '%i'",sms.SMS[i].DeliveryStatus);
 				sprintf(buffer+strlen(buffer)," WHERE `ID` = '%s' AND `TPMR` = '%i'",Row[0],sms.SMS[i].MessageReference);
 				dbgprintf("%s\n",buffer);
-				if (mysql_real_query(&Config->DB,buffer,strlen(buffer))) {
-					WriteSMSDLog("Error writing to database (SaveInboxSMS): %d %s\n", mysql_errno(&Config->DB), mysql_error(&Config->DB));
+				if (mysql_real_query(&Config->DBConnMySQL,buffer,strlen(buffer))) {
+					WriteSMSDLog(_("Error writing to database (SaveInboxSMS): %d %s\n"), mysql_errno(&Config->DBConnMySQL), mysql_error(&Config->DBConnMySQL));
 					return ERR_UNKNOWN;
 				}
 			}
@@ -277,7 +266,7 @@ static GSM_Error SMSDMySQL_SaveInboxSMS(GSM_MultiSMSMessage sms, GSM_SMSDConfig
 		case SMS_Coding_Unicode_No_Compression:
 	    	case SMS_Coding_Default_No_Compression:
 			EncodeUTF8(buffer2, sms.SMS[i].Text);
-			mysql_real_escape_string(&Config->DB,buffer4,buffer2,strlen(buffer2));
+			mysql_real_escape_string(&Config->DBConnMySQL,buffer4,buffer2,strlen(buffer2));
 			memcpy(buffer+strlen(buffer),buffer4,strlen(buffer4)+1);
 			break;
 		case SMS_Coding_8bit:
@@ -287,8 +276,8 @@ static GSM_Error SMSDMySQL_SaveInboxSMS(GSM_MultiSMSMessage sms, GSM_SMSDConfig
 		}
 		sprintf(buffer+strlen(buffer),"','%s')",Config->PhoneID);
 		dbgprintf("%s\n",buffer);
-		if (mysql_real_query(&Config->DB,buffer,strlen(buffer))) {
-			WriteSMSDLog("Error writing to database (SaveInbox): %d %s\n", mysql_errno(&Config->DB), mysql_error(&Config->DB));
+		if (mysql_real_query(&Config->DBConnMySQL,buffer,strlen(buffer))) {
+			WriteSMSDLog(_("Error writing to database (SaveInbox): %d %s\n"), mysql_errno(&Config->DBConnMySQL), mysql_error(&Config->DBConnMySQL));
 			return ERR_UNKNOWN;
 		}
 	}
@@ -301,12 +290,12 @@ static GSM_Error SMSDMySQL_RefreshSendStatus(GSM_SMSDConfig *Config, unsigned ch
 	unsigned char buffer[10000];
 
 	sprintf(buffer,"UPDATE `outbox` SET `SendingTimeOut`=(now() + INTERVAL 15 SECOND)+0 WHERE `ID` = '%s' AND `SendingTimeOut` < now()",ID);
-	if (mysql_real_query(&Config->DB,buffer,strlen(buffer))) {
-		WriteSMSDLog("Error writing to database (RefreshSendStatus): %d %s\n", mysql_errno(&Config->DB), mysql_error(&Config->DB));
+	if (mysql_real_query(&Config->DBConnMySQL,buffer,strlen(buffer))) {
+		WriteSMSDLog(_("Error writing to database (RefreshSendStatus): %d %s\n"), mysql_errno(&Config->DBConnMySQL), mysql_error(&Config->DBConnMySQL));
 		return ERR_UNKNOWN;
 	}
 	dbgprintf("%s\n",buffer);
-	if (mysql_affected_rows(&Config->DB) == 0) return ERR_UNKNOWN;
+	if (mysql_affected_rows(&Config->DBConnMySQL) == 0) return ERR_UNKNOWN;
   	return ERR_NONE;
 }
 
@@ -322,12 +311,12 @@ static GSM_Error SMSDMySQL_FindOutboxSMS(GSM_MultiSMSMessage *sms, GSM_SMSDConfi
 	bool			found = false;
 
 	sprintf(buf, "SELECT ID,InsertIntoDB,SendingDateTime,SenderID FROM `outbox` WHERE SendingDateTime < NOW() AND SendingTimeOut < NOW()");
-	if (mysql_real_query(&Config->DB,buf,strlen(buf))) {
-		WriteSMSDLog("Error reading from database (FindOutbox): %s\n", mysql_error(&Config->DB));
+	if (mysql_real_query(&Config->DBConnMySQL,buf,strlen(buf))) {
+		WriteSMSDLog(_("Error reading from database (FindOutbox): %s\n"), mysql_error(&Config->DBConnMySQL));
 		return ERR_UNKNOWN;
 	}
-	if (!(Res = mysql_store_result(&Config->DB))) {
-		WriteSMSDLog("Error reading from database (FindOutbox): %s\n", mysql_error(&Config->DB));
+	if (!(Res = mysql_store_result(&Config->DBConnMySQL))) {
+		WriteSMSDLog(_("Error reading from database (FindOutbox): %s\n"), mysql_error(&Config->DBConnMySQL));
 		return ERR_UNKNOWN;
 	}
 	while ((Row = mysql_fetch_row(Res))) {
@@ -346,24 +335,24 @@ static GSM_Error SMSDMySQL_FindOutboxSMS(GSM_MultiSMSMessage *sms, GSM_SMSDConfi
 	}
 	mysql_free_result(Res);
 	sms->Number = 0;
-	for (i=0;i<MAX_MULTI_SMS;i++) {
+	for (i=0;i<GSM_MAX_MULTI_SMS;i++) {
 		GSM_SetDefaultSMSData(&sms->SMS[i]);
 		sms->SMS[i].SMSC.Number[0] = 0;
 		sms->SMS[i].SMSC.Number[1] = 0;
 	}
-	for (i=1;i<MAX_MULTI_SMS+1;i++) {
+	for (i=1;i<GSM_MAX_MULTI_SMS+1;i++) {
 		if (i==1) {
 			sprintf(buf, "SELECT Text,Coding,UDH,Class,TextDecoded,ID,DestinationNumber,MultiPart,RelativeValidity,DeliveryReport,CreatorID FROM `outbox` WHERE ID='%s'",ID);
 		} else {
 			sprintf(buf, "SELECT Text,Coding,UDH,Class,TextDecoded,ID,SequencePosition FROM `outbox_multipart` WHERE ID='%s' AND SequencePosition='%i'",ID,i);
 		}
 		dbgprintf("%s\n",buf);
-    		if (mysql_real_query(&Config->DB,buf,strlen(buf))) {
-			WriteSMSDLog("Error reading from database (FindOutbox): %s\n", mysql_error(&Config->DB));
+    		if (mysql_real_query(&Config->DBConnMySQL,buf,strlen(buf))) {
+			WriteSMSDLog(_("Error reading from database (FindOutbox): %s\n"), mysql_error(&Config->DBConnMySQL));
 			return ERR_UNKNOWN;
 		}
-		if (!(Res = mysql_store_result(&Config->DB))) {
-			WriteSMSDLog("Error reading from database (FindOutbox): %s\n", mysql_error(&Config->DB));
+		if (!(Res = mysql_store_result(&Config->DBConnMySQL))) {
+			WriteSMSDLog(_("Error reading from database (FindOutbox): %s\n"), mysql_error(&Config->DBConnMySQL));
 			return ERR_UNKNOWN;
 		}
 		if (!(Row = mysql_fetch_row(Res))) {
@@ -431,13 +420,13 @@ static GSM_Error SMSDMySQL_MoveSMS(GSM_MultiSMSMessage *sms, GSM_SMSDConfig *Con
 	unsigned char buffer[10000];
 
 	sprintf(buffer,"DELETE FROM `outbox` WHERE `ID` = '%s'",ID);
-	if (mysql_real_query(&Config->DB,buffer,strlen(buffer))) {
-		WriteSMSDLog("Error deleting from database (MoveSMS): %d %s\n", mysql_errno(&Config->DB), mysql_error(&Config->DB));
+	if (mysql_real_query(&Config->DBConnMySQL,buffer,strlen(buffer))) {
+		WriteSMSDLog(_("Error deleting from database (MoveSMS): %d %s\n"), mysql_errno(&Config->DBConnMySQL), mysql_error(&Config->DBConnMySQL));
 		return ERR_UNKNOWN;
 	}
 	sprintf(buffer,"DELETE FROM `outbox_multipart` WHERE `ID` = '%s'",ID);
-	if (mysql_real_query(&Config->DB,buffer,strlen(buffer))) {
-		WriteSMSDLog("Error deleting from database (MoveSMS): %d %s\n", mysql_errno(&Config->DB), mysql_error(&Config->DB));
+	if (mysql_real_query(&Config->DBConnMySQL,buffer,strlen(buffer))) {
+		WriteSMSDLog(_("Error deleting from database (MoveSMS): %d %s\n"), mysql_errno(&Config->DBConnMySQL), mysql_error(&Config->DBConnMySQL));
 		return ERR_UNKNOWN;
 	}
   	return ERR_NONE;
@@ -452,12 +441,12 @@ static GSM_Error SMSDMySQL_CreateOutboxSMS(GSM_MultiSMSMessage *sms, GSM_SMSDCon
 	MYSQL_ROW 		Row;
 
 	sprintf(buffer,"SELECT ID FROM outbox ORDER BY ID DESC LIMIT 1");
-	if (mysql_real_query(&Config->DB,buffer,strlen(buffer))) {
-		WriteSMSDLog("Error reading from database (CreateOutbox): %s\n", mysql_error(&Config->DB));
+	if (mysql_real_query(&Config->DBConnMySQL,buffer,strlen(buffer))) {
+		WriteSMSDLog(_("Error reading from database (CreateOutbox): %s\n"), mysql_error(&Config->DBConnMySQL));
 		return ERR_UNKNOWN;
 	}
-	if (!(Res = mysql_store_result(&Config->DB))) {
-		WriteSMSDLog("Error reading from database (CreateOutbox): %s\n", mysql_error(&Config->DB));
+	if (!(Res = mysql_store_result(&Config->DBConnMySQL))) {
+		WriteSMSDLog(_("Error reading from database (CreateOutbox): %s\n"), mysql_error(&Config->DBConnMySQL));
 		return ERR_UNKNOWN;
 	}
 	if ((Row = mysql_fetch_row(Res))) {
@@ -482,8 +471,8 @@ static GSM_Error SMSDMySQL_CreateOutboxSMS(GSM_MultiSMSMessage *sms, GSM_SMSDCon
 		sprintf(buffer+strlen(buffer),"`Coding`,`UDH`, \
 			`Class`,`TextDecoded`,`ID`) VALUES (");
 		if (i==0) {
-			sprintf(buffer+strlen(buffer),"'Gammu %s',",VERSION);			
-			sprintf(buffer+strlen(buffer),"'','");			
+			sprintf(buffer+strlen(buffer),"'Gammu %s',",VERSION);
+			sprintf(buffer+strlen(buffer),"'','");
 			if (sms->SMS[i].PDU == SMS_Status_Report) {
 				sprintf(buffer+strlen(buffer),"yes','");
 			} else {
@@ -546,7 +535,7 @@ static GSM_Error SMSDMySQL_CreateOutboxSMS(GSM_MultiSMSMessage *sms, GSM_SMSDCon
 		case SMS_Coding_Unicode_No_Compression:
 	    	case SMS_Coding_Default_No_Compression:
 			EncodeUTF8(buffer2, sms->SMS[i].Text);
-			mysql_real_escape_string(&Config->DB,buffer5,buffer2,strlen(buffer2));
+			mysql_real_escape_string(&Config->DBConnMySQL,buffer5,buffer2,strlen(buffer2));
 			memcpy(buffer+strlen(buffer),buffer5,strlen(buffer5)+1);
 			break;
 		default:
@@ -558,12 +547,12 @@ static GSM_Error SMSDMySQL_CreateOutboxSMS(GSM_MultiSMSMessage *sms, GSM_SMSDCon
 				ID++;
 				sprintf(buffer4,"SELECT ID FROM sentitems WHERE ID='%i'",ID);
 				dbgprintf("%s\n",buffer4);
-				if (mysql_real_query(&Config->DB,buffer4,strlen(buffer4))) {
-					WriteSMSDLog("Error reading from database (CreateOutbox): %s\n", mysql_error(&Config->DB));
+				if (mysql_real_query(&Config->DBConnMySQL,buffer4,strlen(buffer4))) {
+					WriteSMSDLog(_("Error reading from database (CreateOutbox): %s\n"), mysql_error(&Config->DBConnMySQL));
 					return ERR_UNKNOWN;
 				}
-				if (!(Res = mysql_store_result(&Config->DB))) {
-					WriteSMSDLog("Error reading from database (CreateOutbox): %s\n", mysql_error(&Config->DB));
+				if (!(Res = mysql_store_result(&Config->DBConnMySQL))) {
+					WriteSMSDLog(_("Error reading from database (CreateOutbox): %s\n"), mysql_error(&Config->DBConnMySQL));
 					return ERR_UNKNOWN;
 				}
 				if (!(Row = mysql_fetch_row(Res))) {
@@ -571,12 +560,12 @@ static GSM_Error SMSDMySQL_CreateOutboxSMS(GSM_MultiSMSMessage *sms, GSM_SMSDCon
 					strcpy(buffer4,buffer);
 					sprintf(buffer4+strlen(buffer4),"%i')",ID);
 					dbgprintf("%s\n",buffer4);
-					if (mysql_real_query(&Config->DB,buffer4,strlen(buffer4))) {
-						if (mysql_errno(&Config->DB) == ER_DUP_ENTRY) {
-							WriteSMSDLog("Duplicated outgoing SMS ID\n");
+					if (mysql_real_query(&Config->DBConnMySQL,buffer4,strlen(buffer4))) {
+						if (mysql_errno(&Config->DBConnMySQL) == ER_DUP_ENTRY) {
+							WriteSMSDLog(_("Duplicated outgoing SMS ID\n"));
 							continue;
 						}
-						WriteSMSDLog("Error writing to database (CreateOutbox): %d %s %s\n", mysql_errno(&Config->DB), mysql_error(&Config->DB),buffer4);
+						WriteSMSDLog(_("Error writing to database (CreateOutbox): %d %s %s\n"), mysql_errno(&Config->DBConnMySQL), mysql_error(&Config->DBConnMySQL),buffer4);
 						return ERR_UNKNOWN;
 					}
 					mysql_free_result(Res);
@@ -588,8 +577,8 @@ static GSM_Error SMSDMySQL_CreateOutboxSMS(GSM_MultiSMSMessage *sms, GSM_SMSDCon
 			strcpy(buffer4,buffer);
 			sprintf(buffer4+strlen(buffer4),"%i')",ID);
 			dbgprintf("%s\n",buffer4);
-			if (mysql_real_query(&Config->DB,buffer4,strlen(buffer4))) {
-				WriteSMSDLog("Error writing to database (CreateOutbox): %d %s\n", mysql_errno(&Config->DB), mysql_error(&Config->DB));
+			if (mysql_real_query(&Config->DBConnMySQL,buffer4,strlen(buffer4))) {
+				WriteSMSDLog(_("Error writing to database (CreateOutbox): %d %s\n"), mysql_errno(&Config->DBConnMySQL), mysql_error(&Config->DBConnMySQL));
 				return ERR_UNKNOWN;
 			}
 		}
@@ -602,7 +591,7 @@ static GSM_Error SMSDMySQL_AddSentSMSInfo(GSM_MultiSMSMessage *sms, GSM_SMSDConf
 {
 	unsigned char	buffer[10000],buffer2[400],buff[50],buffer5[400];
 
-	if (err == SMSD_SEND_OK) WriteSMSDLog("Transmitted %s (%s: %i) to %s", Config->SMSID, (Part == sms->Number?"total":"part"),Part,DecodeUnicodeString(sms->SMS[0].Number));
+	if (err == SMSD_SEND_OK) WriteSMSDLog(_("Transmitted %s (%s: %i) to %s"), Config->SMSID, (Part == sms->Number?"total":"part"),Part,DecodeUnicodeString(sms->SMS[0].Number));
 
 	buff[0] = 0;
 	if (err == SMSD_SEND_OK) {
@@ -657,7 +646,7 @@ static GSM_Error SMSDMySQL_AddSentSMSInfo(GSM_MultiSMSMessage *sms, GSM_SMSDConf
 	case SMS_Coding_Unicode_No_Compression:
 	case SMS_Coding_Default_No_Compression:
 		EncodeUTF8(buffer2, sms->SMS[Part-1].Text);
-		mysql_real_escape_string(&Config->DB,buffer5,buffer2,strlen(buffer2));
+		mysql_real_escape_string(&Config->DBConnMySQL,buffer5,buffer2,strlen(buffer2));
 		memcpy(buffer+strlen(buffer),buffer5,strlen(buffer5)+1);
 		break;
 	default:
@@ -670,8 +659,8 @@ static GSM_Error SMSDMySQL_AddSentSMSInfo(GSM_MultiSMSMessage *sms, GSM_SMSDConf
 		sprintf(buffer+strlen(buffer),"-1')");
 	}
 	dbgprintf("%s\n",buffer);
-	if (mysql_real_query(&Config->DB,buffer,strlen(buffer))) {
-		WriteSMSDLog("Error writing to database (AddSent): %d %s\n", mysql_errno(&Config->DB), mysql_error(&Config->DB));
+	if (mysql_real_query(&Config->DBConnMySQL,buffer,strlen(buffer))) {
+		WriteSMSDLog(_("Error writing to database (AddSent): %d %s\n"), mysql_errno(&Config->DBConnMySQL), mysql_error(&Config->DBConnMySQL));
 		return ERR_UNKNOWN;
 	}
   	return ERR_NONE;
@@ -679,13 +668,15 @@ static GSM_Error SMSDMySQL_AddSentSMSInfo(GSM_MultiSMSMessage *sms, GSM_SMSDConf
 
 static GSM_Error SMSDMySQL_RefreshPhoneStatus(GSM_SMSDConfig *Config)
 {
-	unsigned char buffer[500];
+	unsigned char buffer[500],imei[100];
+
+	GSM_GetIMEI(s, imei);
 
 	sprintf(buffer,"UPDATE `phones` SET `TimeOut`= (NOW() + INTERVAL 10 SECOND)+0");
-	sprintf(buffer+strlen(buffer)," WHERE `IMEI` = '%s'",s.Phone.Data.IMEI);
+	sprintf(buffer+strlen(buffer)," WHERE `IMEI` = '%s'",imei);
 	dbgprintf("%s\n",buffer);
-	if (mysql_real_query(&Config->DB,buffer,strlen(buffer))) {
-		WriteSMSDLog("Error writing to database (SaveInboxSMS): %d %s\n", mysql_errno(&Config->DB), mysql_error(&Config->DB));
+	if (mysql_real_query(&Config->DBConnMySQL,buffer,strlen(buffer))) {
+		WriteSMSDLog(_("Error writing to database (SaveInboxSMS): %d %s\n"), mysql_errno(&Config->DBConnMySQL), mysql_error(&Config->DBConnMySQL));
 		return ERR_UNKNOWN;
 	}
 	return ERR_NONE;
