@@ -4,22 +4,30 @@
 #include <signal.h>
 #include <stdarg.h>
 #include <time.h>
+#include <unistd.h>
 
 #include "../../common/misc/coding/coding.h"
+#include "../../common/misc/locales.h"
 #include "../gammu.h"
 #include "smsdcore.h"
 #include "s_files.h"
 #ifdef HAVE_MYSQL_MYSQL_H
 #  include "s_mysql.h"
 #endif
+#ifdef HAVE_POSTGRESQL_LIBPQ_FE_H
+#  include "s_pgsql.h"
+#endif
 
 FILE 		 *smsd_log_file = NULL;
 static int	 TPMR;
 static GSM_Error SendingSMSStatus;
 
-static void SMSSendingSMSStatus (char *Device, int status, int mr)
+void SMSSendingSMSStatus (GSM_StateMachine *s, int status, int mr)
 {
-	dbgprintf("Incoming SMS device: \"%s\" status=%d, reference=%d\n",Device, status, mr);
+	dbgprintf("Incoming SMS device: \"%s\" status=%d, reference=%d\n",
+			GSM_GetConfig(s, -1)->Device, 
+			status, 
+			mr);
 	TPMR = mr;
 	if (status==0) {
 		SendingSMSStatus = ERR_NONE;
@@ -32,17 +40,19 @@ void GSM_Terminate_SMSD(char *msg, int error, bool exitprogram, int rc)
 {
 	int ret = ERR_NONE;
 
-	if (s.opened) {
-		WriteSMSDLog("Terminating communication");
-		ret=GSM_TerminateConnection(&s);
+	if (GSM_IsConnected(s)) {
+		WriteSMSDLog(_("Terminating communication"));
+		ret=GSM_TerminateConnection(s);
 		if (ret!=ERR_NONE) {
-			printf("%s\n",print_error(error,s.di.df,s.msg));
-			if (s.opened) GSM_TerminateConnection(&s);
+			printf("%s\n",GSM_ErrorString(error));
+			if (GSM_IsConnected(s)) {
+				GSM_TerminateConnection(s);
+			}
 		}
 	}
 	if (error != 0) {
-		WriteSMSDLog(msg, error, print_error(error,s.di.df,s.msg));
-		fprintf(stderr, msg, error, print_error(error,s.di.df,s.msg));
+		WriteSMSDLog(msg, error, GSM_ErrorString(error));
+		fprintf(stderr, msg, error, GSM_ErrorString(error));
 		fprintf(stderr, "\n");
 	}
 	if (exitprogram) {
@@ -51,7 +61,7 @@ void GSM_Terminate_SMSD(char *msg, int error, bool exitprogram, int rc)
 	}
 }
 
-#ifdef __GNUC__
+#if defined(__GNUC__) && !defined(printf)
 __attribute__((format(printf, 1, 2)))
 #endif
 void WriteSMSDLog(char *format, ...)
@@ -63,7 +73,7 @@ void WriteSMSDLog(char *format, ...)
 
 	if (smsd_log_file != NULL) {
 		va_start(argp, format);
-		result = vsprintf(Buffer,GetMsg(s.msg,format),argp);
+		result = vsprintf(Buffer,format,argp);
 		va_end(argp);
 
 		GSM_GetCurrentDateTime(&date_time);
@@ -80,14 +90,20 @@ void SMSD_ReadConfig(char *filename, GSM_SMSDConfig *Config, bool log, char *ser
 {
 	INI_Section 		*smsdcfgfile = NULL;
 	GSM_Config 		smsdcfg;
+	GSM_Config 		*gammucfg;
 	unsigned char		*str;
 	static unsigned char	emptyPath[1] = "\0";
+	GSM_Error		error;
 
 	memset(&smsdcfg, 0, sizeof(smsdcfg));
 
-	smsdcfgfile=INI_ReadFile(filename, false);
-	if (smsdcfgfile==NULL) {
-		fprintf(stderr,"Can't find file \"%s\"\n",filename);
+	error = INI_ReadFile(filename, false, &smsdcfgfile);
+	if (smsdcfgfile == NULL || error != ERR_NONE) {
+		if (error == ERR_FILENOTSUPPORTED) {
+			fprintf(stderr, _("Could not parse config file \"%s\"\n"),filename);
+		} else {
+			fprintf(stderr, _("Can't find file \"%s\"\n"),filename);
+		}
 		exit(-1);
 	}
 
@@ -95,42 +111,45 @@ void SMSD_ReadConfig(char *filename, GSM_SMSDConfig *Config, bool log, char *ser
 	if (Config->logfilename != NULL) {
 		smsd_log_file=fopen(Config->logfilename,"ab");
 		if (smsd_log_file == NULL) {
-			fprintf(stderr,"Can't open file \"%s\"\n",Config->logfilename);
+			fprintf(stderr, _("Can't open file \"%s\"\n"),Config->logfilename);
 			exit(-1);
 		}
-		fprintf(stderr,"Log filename is \"%s\"\n",Config->logfilename);
+		fprintf(stderr, _("Log filename is \"%s\"\n"),Config->logfilename);
 	}
-	if (log) WriteSMSDLog("Start GAMMU smsd");
+	if (log) WriteSMSDLog(_("Starting GAMMU smsd"));
 
 	/* Include Numbers used, because we don't want create new variable */
 	Config->IncludeNumbers=INI_FindLastSectionEntry(smsdcfgfile, "gammu", false);
 	if (Config->IncludeNumbers) {
 		GSM_ReadConfig(smsdcfgfile, &smsdcfg, 0);
-		memcpy(&s.Config,&smsdcfg,sizeof(GSM_Config));
-		error=GSM_SetDebugFile(s.Config[0].DebugFile, &di);
+		gammucfg = GSM_GetConfig(s, 0);
+		*gammucfg = smsdcfg;
+		error = GSM_SetDebugFile(gammucfg->DebugFile, GSM_GetGlobalDebug());
 	}
 
 	Config->PINCode=INI_GetValue(smsdcfgfile, "smsd", "PIN", false);
 	if (Config->PINCode == NULL) {
- 		if (log) WriteSMSDLog("Warning: No PIN code in %s file",filename);
- 		fprintf(stderr,"Warning: No PIN code in %s file\n",filename);
+ 		if (log) WriteSMSDLog(_("Warning: No PIN code in %s file"),filename);
+ 		fprintf(stderr, _("Warning: No PIN code in %s file\n"),filename);
 	} else {
-		if (log) WriteSMSDLog("PIN code is \"%s\"",Config->PINCode);
+		if (log) WriteSMSDLog(_("PIN code is \"%s\""),Config->PINCode);
 	}
 
 	str = INI_GetValue(smsdcfgfile, "smsd", "commtimeout", false);
 	if (str) Config->commtimeout=atoi(str); else Config->commtimeout = 1;
+	str = INI_GetValue(smsdcfgfile, "smsd", "deliveryreportdelay", false);
+	if (str) Config->deliveryreportdelay=atoi(str); else Config->deliveryreportdelay = 10;
 	str = INI_GetValue(smsdcfgfile, "smsd", "sendtimeout", false);
 	if (str) Config->sendtimeout=atoi(str); else Config->sendtimeout = 10;
 	str = INI_GetValue(smsdcfgfile, "smsd", "receivefrequency", false);
 	if (str) Config->receivefrequency=atoi(str); else Config->receivefrequency = 0;
 	str = INI_GetValue(smsdcfgfile, "smsd", "resetfrequency", false);
 	if (str) Config->resetfrequency=atoi(str); else Config->resetfrequency = 0;
-	if (log) WriteSMSDLog("commtimeout=%i, sendtimeout=%i, receivefrequency=%i, resetfrequency=%i",
+	if (log) WriteSMSDLog(_("commtimeout=%i, sendtimeout=%i, receivefrequency=%i, resetfrequency=%i"),
 			Config->commtimeout, Config->sendtimeout, Config->receivefrequency, Config->resetfrequency);
 
 	Config->deliveryreport = INI_GetValue(smsdcfgfile, "smsd", "deliveryreport", false);
-	if (Config->deliveryreport == NULL || (!mystrncasecmp(Config->deliveryreport, "log", 3) && !mystrncasecmp(Config->deliveryreport, "sms", 3))) {
+	if (Config->deliveryreport == NULL || (strncasecmp(Config->deliveryreport, "log", 3) != 0 && strncasecmp(Config->deliveryreport, "sms", 3) != 0)) {
 		Config->deliveryreport = "no";
 	}
 	if (log) WriteSMSDLog("deliveryreport = %s", Config->deliveryreport);
@@ -139,36 +158,65 @@ void SMSD_ReadConfig(char *filename, GSM_SMSDConfig *Config, bool log, char *ser
 	if (Config->PhoneID == NULL) Config->PhoneID = "";
 	if (log) WriteSMSDLog("phoneid = %s", Config->PhoneID);
 
-	if (!strcmp(service,"FILES")) {
+	str = INI_GetValue(smsdcfgfile, "smsd", "smsc", false);
+	if (str) {
+		Config->SMSC.Location		= 1;
+		Config->SMSC.DefaultNumber[0]	= 0;
+		Config->SMSC.DefaultNumber[1]	= 0;
+		Config->SMSC.Name[0]		= 0;
+		Config->SMSC.Name[1]		= 0;
+		Config->SMSC.Validity.Format	= SMS_Validity_NotAvailable;
+		Config->SMSC.Format		= SMS_FORMAT_Text;
+		EncodeUnicode(Config->SMSC.Number, str, strlen(str));
+	} else {
+		Config->SMSC.Location     = 0;
+	}
+
+	if (!strcasecmp(service,"FILES")) {
 		Config->inboxpath=INI_GetValue(smsdcfgfile, "smsd", "inboxpath", false);
 		if (Config->inboxpath == NULL) Config->inboxpath = emptyPath;
 
 		Config->inboxformat=INI_GetValue(smsdcfgfile, "smsd", "inboxformat", false);
-		if (Config->inboxformat == NULL || (!mystrncasecmp(Config->inboxformat, "detail", 6) && !mystrncasecmp(Config->inboxformat, "unicode", 7))) {
+		if (Config->inboxformat == NULL || (strncasecmp(Config->inboxformat, "detail", 6) != 0 && strncasecmp(Config->inboxformat, "unicode", 7) != 0)) {
 			Config->inboxformat = "standard";
 		}
-		if (log) WriteSMSDLog("Inbox is \"%s\" with format \"%s\"", Config->inboxpath, Config->inboxformat);
+		if (log) WriteSMSDLog(_("Inbox is \"%s\" with format \"%s\""), Config->inboxpath, Config->inboxformat);
 
 		Config->outboxpath=INI_GetValue(smsdcfgfile, "smsd", "outboxpath", false);
 		if (Config->outboxpath == NULL) Config->outboxpath = emptyPath;
 
 		Config->transmitformat=INI_GetValue(smsdcfgfile, "smsd", "transmitformat", false);
-		if (Config->transmitformat == NULL || (!mystrncasecmp(Config->transmitformat, "auto", 4) && !mystrncasecmp(Config->transmitformat, "unicode", 7))) {
+		if (Config->transmitformat == NULL || (strncasecmp(Config->transmitformat, "auto", 4) != 0 && strncasecmp(Config->transmitformat, "unicode", 7) != 0)) {
 			Config->transmitformat = "7bit";
 		}
-		if (log) WriteSMSDLog("Outbox is \"%s\" with transmission format \"%s\"", Config->outboxpath, Config->transmitformat);
+		if (log) WriteSMSDLog(_("Outbox is \"%s\" with transmission format \"%s\""), Config->outboxpath, Config->transmitformat);
 
 		Config->sentsmspath=INI_GetValue(smsdcfgfile, "smsd", "sentsmspath", false);
 		if (Config->sentsmspath == NULL) Config->sentsmspath = Config->outboxpath;
-		if (log) WriteSMSDLog("Sent SMS moved to \"%s\"",Config->sentsmspath);
+		if (log) WriteSMSDLog(_("Sent SMS moved to \"%s\""),Config->sentsmspath);
 
 		Config->errorsmspath=INI_GetValue(smsdcfgfile, "smsd", "errorsmspath", false);
 		if (Config->errorsmspath == NULL) Config->errorsmspath = Config->sentsmspath;
-		if (log) WriteSMSDLog("SMS with errors moved to \"%s\"",Config->errorsmspath);
+		if (log) WriteSMSDLog(_("SMS with errors moved to \"%s\""),Config->errorsmspath);
 	}
 
 #ifdef HAVE_MYSQL_MYSQL_H
-	if (!strcmp(service,"MYSQL")) {
+	if (!strcasecmp(service,"MYSQL")) {
+		Config->skipsmscnumber = INI_GetValue(smsdcfgfile, "smsd", "skipsmscnumber", false);
+		if (Config->skipsmscnumber == NULL) Config->skipsmscnumber="";
+		Config->user = INI_GetValue(smsdcfgfile, "smsd", "user", false);
+		if (Config->user == NULL) Config->user="root";
+		Config->password = INI_GetValue(smsdcfgfile, "smsd", "password", false);
+		if (Config->password == NULL) Config->password="";
+		Config->PC = INI_GetValue(smsdcfgfile, "smsd", "pc", false);
+		if (Config->PC == NULL) Config->PC="localhost";
+		Config->database = INI_GetValue(smsdcfgfile, "smsd", "database", false);
+		if (Config->database == NULL) Config->database="sms";
+	}
+#endif
+
+#ifdef HAVE_POSTGRESQL_LIBPQ_FE_H
+	if (!strcasecmp(service,"PGSQL")) {
 		Config->skipsmscnumber = INI_GetValue(smsdcfgfile, "smsd", "skipsmscnumber", false);
 		if (Config->skipsmscnumber == NULL) Config->skipsmscnumber="";
 		Config->user = INI_GetValue(smsdcfgfile, "smsd", "user", false);
@@ -185,19 +233,18 @@ void SMSD_ReadConfig(char *filename, GSM_SMSDConfig *Config, bool log, char *ser
 	Config->IncludeNumbers=INI_FindLastSectionEntry(smsdcfgfile, "include_numbers", false);
 	Config->ExcludeNumbers=INI_FindLastSectionEntry(smsdcfgfile, "exclude_numbers", false);
 	if (Config->IncludeNumbers != NULL) {
-		if (log) WriteSMSDLog("Include numbers available");
+		if (log) WriteSMSDLog(_("Include numbers available"));
 	}
 	if (Config->ExcludeNumbers != NULL) {
 		if (Config->IncludeNumbers == NULL) {
-			if (log) WriteSMSDLog("Exclude numbers available");
+			if (log) WriteSMSDLog(_("Exclude numbers available"));
 		} else {
-			if (log) WriteSMSDLog("Exclude numbers available, but IGNORED");
+			if (log) WriteSMSDLog(_("Exclude numbers available, but IGNORED"));
 		}
 	}
 
 	Config->retries 	  = 0;
 	Config->prevSMSID[0] 	  = 0;
-	Config->SMSC.Location     = 0;
 	Config->relativevalidity  = -1;
 }
 
@@ -207,10 +254,10 @@ bool SMSD_CheckSecurity(GSM_SMSDConfig *Config)
 	GSM_Error		error;
 
 	/* Need PIN ? */
-	error=Phone->GetSecurityStatus(&s,&SecurityCode.Type);
+	error=GSM_GetSecurityStatus(s,&SecurityCode.Type);
 	/* Unknown error */
 	if (error != ERR_NOTSUPPORTED && error != ERR_NONE) {
-		WriteSMSDLog("Error getting security status (%i)", error);
+		WriteSMSDLog(_("Error getting security status (%i)"), error);
 		return false;
 	}
 	/* No supported - do not check more */
@@ -220,17 +267,17 @@ bool SMSD_CheckSecurity(GSM_SMSDConfig *Config)
 	switch (SecurityCode.Type) {
 	case SEC_Pin:
 		if (Config->PINCode==NULL) {
-			WriteSMSDLog("Warning: no PIN in config");
+			WriteSMSDLog(_("Warning: no PIN in config"));
 			return false;
 		} else {
-			WriteSMSDLog("Trying to enter PIN");
+			WriteSMSDLog(_("Trying to enter PIN"));
 			strcpy(SecurityCode.Code,Config->PINCode);
-			error=Phone->EnterSecurityCode(&s,SecurityCode);
+			error=GSM_EnterSecurityCode(s,SecurityCode);
 			if (error == ERR_SECURITYERROR) {
-				GSM_Terminate_SMSD("ERROR: incorrect PIN", error, true, -1);
+				GSM_Terminate_SMSD(_("ERROR: incorrect PIN"), error, true, -1);
 			}
 			if (error != ERR_NONE) {
-				WriteSMSDLog("Error entering PIN (%i)", error);
+				WriteSMSDLog(_("Error entering PIN (%i)"), error);
 				return false;
 		  	}
 		}
@@ -239,7 +286,7 @@ bool SMSD_CheckSecurity(GSM_SMSDConfig *Config)
 	case SEC_Pin2:
 	case SEC_Puk:
 	case SEC_Puk2:
-		GSM_Terminate_SMSD("ERROR: phone requires not supported code type", 0, true, -1);
+		GSM_Terminate_SMSD(_("ERROR: phone requires not supported code type"), 0, true, -1);
 	case SEC_None:
 		break;
 	}
@@ -258,7 +305,7 @@ bool SMSD_ReadDeleteSMS(GSM_SMSDConfig *Config, GSM_SMSDService *Service)
 	start=true;
 	while (error == ERR_NONE && !gshutdown) {
 		sms.SMS[0].Folder=0x00;
-		error=Phone->GetNextSMS(&s, &sms, start);
+		error=GSM_GetNextSMS(s, &sms, start);
 		switch (error) {
 		case ERR_EMPTY:
 			break;
@@ -293,23 +340,23 @@ bool SMSD_ReadDeleteSMS(GSM_SMSDConfig *Config, GSM_SMSDService *Service)
 			if (process) {
 	 			Service->SaveInboxSMS(sms, Config);
 			} else {
-				WriteSMSDLog("Excluded %s", buffer);
+				WriteSMSDLog(_("Excluded %s"), buffer);
 			}
 			break;
 		default:
-	 		WriteSMSDLog("Error getting SMS (%i)", error);
+	 		WriteSMSDLog(_("Error getting SMS (%i)"), error);
 			return false;
 		}
 		if (error == ERR_NONE && sms.SMS[0].InboxFolder) {
 			for (i=0;i<sms.Number;i++) {
 				sms.SMS[i].Folder=0;
-				error=Phone->DeleteSMS(&s,&sms.SMS[i]);
+				error=GSM_DeleteSMS(s,&sms.SMS[i]);
 				switch (error) {
 				case ERR_NONE:
 				case ERR_EMPTY:
 					break;
 				default:
-					WriteSMSDLog("Error deleting SMS (%i)", error);
+					WriteSMSDLog(_("Error deleting SMS (%i)"), error);
 					return false;
 				}
 			}
@@ -325,9 +372,9 @@ bool SMSD_CheckSMSStatus(GSM_SMSDConfig *Config,GSM_SMSDService *Service)
 	GSM_Error		error;
 
 	/* Do we have any SMS in phone ? */
-	error=Phone->GetSMSStatus(&s,&SMSStatus);
+	error=GSM_GetSMSStatus(s,&SMSStatus);
 	if (error != ERR_NONE) {
-		WriteSMSDLog("Error getting SMS status (%i)", error);
+		WriteSMSDLog(_("Error getting SMS status (%i)"), error);
 		return false;
 	}
 	/* Yes. We have SMS in phone */
@@ -361,7 +408,7 @@ bool SMSD_SendSMS(GSM_SMSDConfig *Config,GSM_SMSDService *Service)
 	}
 	if (error != ERR_NONE) {
 		/* Unknown error - escape */
-		WriteSMSDLog("Error in outbox on %s", Config->SMSID);
+		WriteSMSDLog(_("Error in outbox on %s"), Config->SMSID);
 		for (i=0;i<sms.Number;i++) {
 			Service->AddSentSMSInfo(&sms, Config, Config->SMSID, i+1, SMSD_SEND_ERROR, -1);
 		}
@@ -375,7 +422,7 @@ bool SMSD_SendSMS(GSM_SMSDConfig *Config,GSM_SMSDService *Service)
 			if (Config->retries > MAX_RETRIES) {
 				Config->retries = 0;
 				strcpy(Config->prevSMSID, "");
-				WriteSMSDLog("Moved to errorbox: %s", Config->SMSID);
+				WriteSMSDLog(_("Moved to errorbox: %s"), Config->SMSID);
 				for (i=0;i<sms.Number;i++) {
 					Service->AddSentSMSInfo(&sms, Config, Config->SMSID, i+1, SMSD_SEND_ERROR, -1);
 				}
@@ -390,9 +437,9 @@ bool SMSD_SendSMS(GSM_SMSDConfig *Config,GSM_SMSDService *Service)
 			if (sms.SMS[i].SMSC.Location == 1) {
 			    	if (Config->SMSC.Location == 0) {
 					Config->SMSC.Location = 1;
-					error = Phone->GetSMSC(&s,&Config->SMSC);
+					error = GSM_GetSMSC(s,&Config->SMSC);
 					if (error!=ERR_NONE) {
-						WriteSMSDLog("Error getting SMSC from phone");
+						WriteSMSDLog(_("Error getting SMSC from phone"));
 						return false;
 					}
 
@@ -411,10 +458,10 @@ bool SMSD_SendSMS(GSM_SMSDConfig *Config,GSM_SMSDService *Service)
 				if ((strcmp(Config->deliveryreport, "no") != 0 && (Config->currdeliveryreport == -1))) sms.SMS[i].PDU = SMS_Status_Report;
 			}
 
-			error=Phone->SendSMS(&s, &sms.SMS[i]);
+			error=GSM_SendSMS(s, &sms.SMS[i]);
 			if (error!=ERR_NONE) {
 				Service->AddSentSMSInfo(&sms, Config, Config->SMSID, i+1, SMSD_SEND_SENDING_ERROR, -1);
-				WriteSMSDLog("Error sending SMS %s (%i): %s", Config->SMSID, error,print_error(error,s.di.df,s.msg));
+				WriteSMSDLog(_("Error sending SMS %s (%i): %s"), Config->SMSID, error,GSM_ErrorString(error));
 				return false;
 			}
 			Service->RefreshPhoneStatus(Config);
@@ -427,7 +474,7 @@ bool SMSD_SendSMS(GSM_SMSDConfig *Config,GSM_SMSDService *Service)
 				while (z==Date.Second) {
 					my_sleep(10);
 					GSM_GetCurrentDateTime(&Date);
-					GSM_ReadDevice(&s,true);
+					GSM_ReadDevice(s,true);
 					if (SendingSMSStatus != ERR_TIMEOUT) break;
 				}
 				Service->RefreshSendStatus(Config, Config->SMSID);
@@ -438,7 +485,7 @@ bool SMSD_SendSMS(GSM_SMSDConfig *Config,GSM_SMSDService *Service)
 			}
 			if (SendingSMSStatus != ERR_NONE) {
 				Service->AddSentSMSInfo(&sms, Config, Config->SMSID, i+1, SMSD_SEND_SENDING_ERROR, TPMR);
-				WriteSMSDLog("Error getting send status of %s (%i): %s", Config->SMSID, SendingSMSStatus,print_error(SendingSMSStatus,s.di.df,s.msg));
+				WriteSMSDLog(_("Error getting send status of %s (%i): %s"), Config->SMSID, SendingSMSStatus,GSM_ErrorString(SendingSMSStatus));
 				return false;
 			}
 			error = Service->AddSentSMSInfo(&sms, Config, Config->SMSID, i+1, SMSD_SEND_OK, TPMR);
@@ -454,7 +501,7 @@ bool SMSD_SendSMS(GSM_SMSDConfig *Config,GSM_SMSDService *Service)
 	return true;
 }
 
-void SMSDaemon(int argc, char *argv[])
+void SMSDaemon(int argc UNUSED, char *argv[])
 {
 	int                     errors = 255, initerrors=0;
 	GSM_SMSDService		*Service;
@@ -462,14 +509,18 @@ void SMSDaemon(int argc, char *argv[])
  	time_t			lastreceive, lastreset = 0;
 	GSM_SMSDConfig		Config;
 
-	if (!strcmp(argv[2],"FILES")) {
+	if (!strcasecmp(argv[2],"FILES")) {
 		Service = &SMSDFiles;
 #ifdef HAVE_MYSQL_MYSQL_H
-	} else if (!strcmp(argv[2],"MYSQL")) {
+	} else if (!strcasecmp(argv[2],"MYSQL")) {
 		Service = &SMSDMySQL;
+#endif
+#ifdef HAVE_POSTGRESQL_LIBPQ_FE_H
+	} else if (!strcasecmp(argv[2],"PGSQL")) {
+		Service = &SMSDPgSQL;
 #endif
 	} else {
-		fprintf(stderr,"Unknown service type (\"%s\")\n",argv[2]);
+		fprintf(stderr, _("Unknown SMSD service type (\"%s\")\n"), argv[2]);
 		exit(-1);
 	}
 
@@ -477,7 +528,7 @@ void SMSDaemon(int argc, char *argv[])
 
 	error = Service->Init(&Config);
 	if (error!=ERR_NONE) {
-		GSM_Terminate_SMSD("Stop GAMMU smsd (%i)", error, true, -1);
+		GSM_Terminate_SMSD(_("Initialisation failed, stopping Gammu smsd (%i)"), error, true, -1);
 	}
 
 	signal(SIGINT, interrupt);
@@ -492,44 +543,40 @@ void SMSDaemon(int argc, char *argv[])
 		/* There were errors in communication - try to recover */
 		if (errors > 2) {
 			if (errors != 255) {
-				WriteSMSDLog("Terminating communication (%i,%i)", error, errors);
-				error=GSM_TerminateConnection(&s);
+				WriteSMSDLog(_("Terminating communication (%i,%i)"), error, errors);
+				error=GSM_TerminateConnection(s);
 			}
 			if (initerrors++ > 3) my_sleep(30000);
-			WriteSMSDLog("Starting communication");
-			error=GSM_InitConnection(&s,2);
+			WriteSMSDLog(_("Starting communication"));
+			error=GSM_InitConnection(s,2);
 			switch (error) {
 			case ERR_NONE:
-				s.User.SendSMSStatus 	= SMSSendingSMSStatus;
-				Phone			= s.Phone.Functions;
+				GSM_SetSendSMSStatusCallback(s, SMSSendingSMSStatus);
 				if (errors == 255) {
 					errors = 0;
-					s.Phone.Data.IMEI[0] = 0;
-					if (!(Phone->GetIMEI(&s))) {
+					if (GSM_GetIMEI(s, NULL) != ERR_NONE) {
 						errors++;
 					} else {
 						error = Service->InitAfterConnect(&Config);
 						if (error!=ERR_NONE) {
-							GSM_Terminate_SMSD("Stop GAMMU smsd (%i)", error, true, -1);
+							GSM_Terminate_SMSD(_("Post initialisation failed, stopping Gammu smsd (%i)"), error, true, -1);
 						}
-						Phone->SetFastSMSSending(&s,true);
+						GSM_SetFastSMSSending(s,true);
 					}
 				} else {
 					errors = 0;
 				}
 				if (initerrors > 3 || initerrors < 0) {
-					error=Phone->Reset(&s, false); /* soft reset */
-					WriteSMSDLog("Reset return code: %s (%i) ", error == ERR_NONE? "OK":"ERROR", error);
+					error=GSM_Reset(s, false); /* soft reset */
+					WriteSMSDLog(_("Reset return code: %s (%i) "), error == ERR_NONE? "OK":"ERROR", error);
 					lastreset = time(NULL);
 					my_sleep(5000);
 				}
-				/* Marcin Wiacek: FIXME. To check */
-//				di 			= s.di;
 				break;
 			case ERR_DEVICEOPENERROR:
-				GSM_Terminate_SMSD("Can't open device (%i)", error, true, -1);
+				GSM_Terminate_SMSD(_("Can't open device (%i)"), error, true, -1);
 			default:
-				WriteSMSDLog("Error at init connection (%i)", error);
+				WriteSMSDLog(_("Error at init connection (%i)"), error);
 				errors = 250;
 			}
 			continue;
@@ -557,20 +604,25 @@ void SMSDaemon(int argc, char *argv[])
 		}
 		if (!SMSD_SendSMS(&Config,Service)) continue;
 	}
-	Phone->SetFastSMSSending(&s,false);
-	GSM_Terminate_SMSD("Stop GAMMU smsd", 0, false, 0);
+	GSM_SetFastSMSSending(s,false);
+	GSM_Terminate_SMSD(_("Stopping Gammu smsd"), 0, false, 0);
 }
 
 GSM_Error SMSDaemonSendSMS(char *service, char *filename, GSM_MultiSMSMessage *sms)
 {
 	GSM_SMSDService		*Service;
 	GSM_SMSDConfig		Config;
+	GSM_Error error;
 
-	if (!strcmp(service,"FILES")) {
+	if (!strcasecmp(service,"FILES")) {
 		Service = &SMSDFiles;
 #ifdef HAVE_MYSQL_MYSQL_H
-	} else if (!strcmp(service,"MYSQL")) {
+	} else if (!strcasecmp(service,"MYSQL")) {
 		Service = &SMSDMySQL;
+#endif
+#ifdef HAVE_POSTGRESQL_LIBPQ_FE_H
+	} else if (!strcasecmp(service,"PGSQL")) {
+		Service = &SMSDPgSQL;
 #endif
 	} else {
 		fprintf(stderr,"Unknown service type (\"%s\")\n",service);
@@ -585,6 +637,20 @@ GSM_Error SMSDaemonSendSMS(char *service, char *filename, GSM_MultiSMSMessage *s
 	return Service->CreateOutboxSMS(sms,&Config);
 }
 
+GSM_Error SMSD_NoneFunction(void)
+{
+	return ERR_NONE;
+}
+
+GSM_Error SMSD_NotImplementedFunction(void)
+{
+	return ERR_NOTIMPLEMENTED;
+}
+
+GSM_Error SMSD_NotSupportedFunction(void)
+{
+	return ERR_NOTSUPPORTED;
+}
 /* How should editor hadle tabs in this file? Add editor commands here.
  * vim: noexpandtab sw=8 ts=8 sts=8:
  */
